html,#edit-slug-box,#trackback,#category_nicename,#slug,#dbuser,#dbpass,#dbname,#dbhost,#dbprefix,#dccharset,input[name=gmpath],input[name=archivespath],input[name=lastentry],input[name=email],input[name=url],input[name=aim],input[name=yim],input[name=jabber],#newcontent,#email,#newcomment_author_url,#pass1,#pass2,#siteurl,#home,#admin_email,#date_format,#time_format,#mailserver_url,#mailserver_login,#mailserver_pass,#ping_sites,#blog_charset,#moderation_keys,#blacklist_keys,#permalink_structure,#category_base,#tag_base,#upload_path,#upload_url_path,#insertonly[src],#insertonly[href],#mailserver_port,#thumbnail_size_w,#thumbnail_size_h,#medium_size_w,#medium_size_h,#posts_per_page,#posts_per_rss,#comment_max_links,#insertonly[src],#link_url,#link_image,#rss_uri,#user_login,#key
{
	direction:ltr;
	text-align:left
}

body, td {
	font-family: Tahoma,Verdana,sans-serif;
}

#wpwrap,#footer,#TB_title,.media-upload-form,#media-upload-header
{
	direction:rtl!important;
	text-align:right!important
}

#user_info
{
	left:15px;
	right:auto
}

#wphead h1
{
	margin:0 0 0 15%;
	padding:11px 15px 16px 170px
}

#wphead #viewsite
{
	margin-left:0;
	margin-right:10px
}

#sidemenu
{
	float:left;
	margin:-30px 315px 0 15px;
	padding-left:0;
	padding-right:10px
}

#dashmenu
{
	font-size:11px;
	left:auto;
	padding-left:0;
	padding-right:9px;
	right:0
}

#dashmenu a
{
	margin-left:8px;
	margin-right:8px;
}

#adminmenu
{
	height:32px;
	padding:5px 8px 0 0
}

#adminmenu li a
{
	display:block;
	float:right;
	font:700 16px/100% "Times New Roman", Times, serif;
	margin:0 0 0 10px;
	padding:5px 7px 9px
}

#adminmenu li a.current
{
	font-weight:700
}

#adminmenu li a #awaiting-mod
{
	background:url(images/comment-stalk-rtl.gif) -309px bottom !important;
	font-family: Tahoma, Verdana, sans-serif;
	margin-left:-45px;
	margin-right:.2em;
	margin-top:-5px
}

#adminmenu li a:hover #awaiting-mod
{
	background-position: -229px bottom !important;
}

#adminmenu li a #awaiting-mod span
{
	left:0;
	right:auto
}

#submenu li a
{
	display:block;
	float:right;
	margin:0 10px 10px 0;
	padding:2px 4px
}

p#post-search
{
	left:0;
	right:auto
}

.tablenav div
{
	float:right!important
}

.tablenav .delete 
{
	margin-left: 40px;
	margin-right: 0;
}

ul.view-switch
{
	float:left;
	margin:-28px 0 -2px 5px
}

#currenttheme img
{
	float:right;
	margin-left:1em;
	margin-right:0
}

form#themeselector
{
	float:left
}

.form-table input.tog
{
	float:right;
	margin-right:0;
	margin-left:2px;
}

.curtime
{
	background-position:right 2px;
	padding-left:0;
	padding-right:18px;
}

#poststuff #edButtonPreview,#poststuff #edButtonHTML
{
	float:left;
	margin-left:8px;
	margin-right:0
}

#poststuff #media-buttons
{
	direction: ltr;
	float:left;
	margin-left:20px;
	margin-right:0
}

#poststuff #media-buttons a
{
	margin-left:8px;
	margin-right:0;
	padding: 0 5px 2px 2px;
}

#poststuff .togbox
{
	margin-left:0;
	margin-right:-19px
}

#categorydiv ul#category-tabs,#linkcategorydiv ul#category-tabs
{
	float:right;
	margin:0 0 0 -120px;
	text-align:left
}

ul#category-tabs li.ui-tabs-selected,#poststuff .togbox
{
	-khtml-border-bottom-left-radius:0;
	-khtml-border-bottom-right-radius:3px;
	-khtml-border-top-left-radius:0;
	-khtml-border-top-right-radius:3px;
	-moz-border-radius-bottomleft:0;
	-moz-border-radius-bottomright:3px;
	-moz-border-radius-topleft:0;
	-moz-border-radius-topright:3px;
	-webkit-border-bottom-left-radius:0;
	-webkit-border-bottom-right-radius:3px;
	-webkit-border-top-left-radius:0;
	-webkit-border-top-right-radius:3px;
	border-bottom-left-radius:0;
	border-bottom-right-radius:3px;
	border-top-left-radius:0;
	border-top-right-radius:3px
}

div.ui-tabs-panel
{
	margin:0 120px 0 0
}

#tagchecklist span
{
	float:right;
	margin-left:25px;
	margin-right:0
}

#tagchecklist span a {
	margin: 6px -9px 0 0;
	float: right;
}


.tablenav .tablenav-pages
{
	float:left !important;
	width:200px
}

.tablenav-pages .page-numbers
{
	display:block;
	float:right
}

.side-info ul
{
	padding-left:0;
	padding-right:18px
}

.form-table table.color-palette {
	float:right;
}

#pass-strength-result
{
	float:right;
	margin-right:0;
	margin-left:5px;
}

td.comment p.comment-author img.avatar
{
	float:right !important;
	margin-right:0 !important;
	margin-left:8px !important;
}

#footer
{
	padding:10px 0 0
}

#footer p
{
	height:35px;
	margin-right:20px;
	padding:12px 40px 0 0;
}

#submenu,#wpbody
{
	clear:both;
}

#submenu li,ul.subsubsub li a
{
	padding:0;
}

table.widefat th,.form-table th
{
	text-align:right
}

.widefat th input {
	margin: 0 8px 0 0;
	padding: 0;
}

.available-theme,#tagchecklist span a
{
	float:right
}
#template div {
	margin-right:0;
	margin-left:190px;
}
.submitbox .submit {
	text-align:right;
}

#editorcontainer {
	border:1px solid;
}

.wrap h2 {
	margin: 5px -4px 0 0;
	padding-right: 0px;
	padding-left: 280px;
}

div#message {
	clear: both;
}