div.widget-liquid-left-holder {
	clear:right;
	float:right;
	margin-right:0;
	margin-left:-310px;
}
div.widget-liquid-left {
	margin-right:0;
	margin-left:310px;
}
div.widget-liquid-right {
	clear:left;
	float:left;
}
p#widget-search {
	left:0;
	right:auto
}
ul#widget-list li.widget-list-item h4.widget-title {
	float:right;
	text-align:right;
}
ul#widget-list li.widget-list-item div.widget-description {
	margin:0 200px 0 0;
	padding:0 4em 0 0;
}
.widget-control-save, .widget-control-remove {
	float:right;
	margin-right:0;
	margin-left:8px;
}
h4.widget-title a {
	right:auto;
	left:1em;
}
li.widget-list-control-item h4.widget-title a, #dragHelper li.widget-list-control-item h4.widget-title a, #draghelper li.widget-list-control-item h4.widget-title a:visited {
	right:auto;
	left:1em;
}

li.widget-list-control-item div.widget-control {
	padding:0 0 0 10px;
}
ul.widget-control-list div.widget-control-actions {
	margin-right:0;
	margin-left:-10px;
}
ul.widget-control-list .widget-title {
	text-align:right;
}