-- MySQL dump 10.11
--
-- Host: localhost    Database: kkfungne_sswp01
-- ------------------------------------------------------
-- Server version	5.0.45-community-log

/*!40101 SET @OLD_CHARACTER_SET_CLIENT=@@CHARACTER_SET_CLIENT */;
/*!40101 SET @OLD_CHARACTER_SET_RESULTS=@@CHARACTER_SET_RESULTS */;
/*!40101 SET @OLD_COLLATION_CONNECTION=@@COLLATION_CONNECTION */;
/*!40101 SET NAMES utf8 */;
/*!40103 SET @OLD_TIME_ZONE=@@TIME_ZONE */;
/*!40103 SET TIME_ZONE='+00:00' */;
/*!40014 SET @OLD_UNIQUE_CHECKS=@@UNIQUE_CHECKS, UNIQUE_CHECKS=0 */;
/*!40014 SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0 */;
/*!40101 SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='NO_AUTO_VALUE_ON_ZERO' */;
/*!40111 SET @OLD_SQL_NOTES=@@SQL_NOTES, SQL_NOTES=0 */;

--
-- Table structure for table `comments`
--

DROP TABLE IF EXISTS `comments`;
CREATE TABLE `comments` (
  `comment_ID` bigint(20) unsigned NOT NULL auto_increment,
  `comment_post_ID` int(11) NOT NULL default '0',
  `comment_author` tinytext NOT NULL,
  `comment_author_email` varchar(100) NOT NULL default '',
  `comment_author_url` varchar(200) NOT NULL default '',
  `comment_author_IP` varchar(100) NOT NULL default '',
  `comment_date` datetime NOT NULL default '0000-00-00 00:00:00',
  `comment_date_gmt` datetime NOT NULL default '0000-00-00 00:00:00',
  `comment_content` text NOT NULL,
  `comment_karma` int(11) NOT NULL default '0',
  `comment_approved` varchar(20) NOT NULL default '1',
  `comment_agent` varchar(255) NOT NULL default '',
  `comment_type` varchar(20) NOT NULL default '',
  `comment_parent` bigint(20) NOT NULL default '0',
  `user_id` bigint(20) NOT NULL default '0',
  PRIMARY KEY  (`comment_ID`),
  KEY `comment_approved` (`comment_approved`),
  KEY `comment_post_ID` (`comment_post_ID`),
  KEY `comment_approved_date_gmt` (`comment_approved`,`comment_date_gmt`),
  KEY `comment_date_gmt` (`comment_date_gmt`)
) ENGINE=MyISAM AUTO_INCREMENT=3 DEFAULT CHARSET=utf8;

--
-- Dumping data for table `comments`
--

LOCK TABLES `comments` WRITE;
/*!40000 ALTER TABLE `comments` DISABLE KEYS */;
INSERT INTO `comments` VALUES (1,1,'Mr WordPress','','http://wordpress.org/','','2008-04-25 11:43:50','2008-04-25 17:43:50','Hi, this is a comment.<br />To delete a comment, just log in and view the post&#039;s comments. There you will have the option to edit or delete them.',0,'1','','',0,0),(2,6,'Axel','abdullah@operamail.com','http://njdokj.info/9f66aa6cf45f561c66de9df39deb72f1/cef166b4c7d1f69e5c2edc281037f48a','69.14.192.85','2008-06-29 22:37:14','2008-06-29 14:37:14','cef166b4c7d1f69e5c2edc281037f48a\n<a href=\"http://njdokj.info/9f66aa6cf45f561c66de9df39deb72f1/cef166b4c7d1f69e5c2edc281037f48a\" rel=\"nofollow\"> http://njdokj.info/9f66aa6cf45f561c66de9df39deb72f1/cef166b4c7d1f69e5c2edc281037f48a </a>\n http://njdokj.info/9f66aa6cf45f561c66de9df39deb72f1/cef166b4c7d1f69e5c2edc281037f48a\n[url]http://njdokj.info/9f66aa6cf45f561c66de9df39deb72f1/cef166b4c7d1f69e5c2edc281037f48a[url]',0,'spam','','',0,0);
/*!40000 ALTER TABLE `comments` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `links`
--

DROP TABLE IF EXISTS `links`;
CREATE TABLE `links` (
  `link_id` bigint(20) NOT NULL auto_increment,
  `link_url` varchar(255) NOT NULL default '',
  `link_name` varchar(255) NOT NULL default '',
  `link_image` varchar(255) NOT NULL default '',
  `link_target` varchar(25) NOT NULL default '',
  `link_category` bigint(20) NOT NULL default '0',
  `link_description` varchar(255) NOT NULL default '',
  `link_visible` varchar(20) NOT NULL default 'Y',
  `link_owner` int(11) NOT NULL default '1',
  `link_rating` int(11) NOT NULL default '0',
  `link_updated` datetime NOT NULL default '0000-00-00 00:00:00',
  `link_rel` varchar(255) NOT NULL default '',
  `link_notes` mediumtext NOT NULL,
  `link_rss` varchar(255) NOT NULL default '',
  PRIMARY KEY  (`link_id`),
  KEY `link_category` (`link_category`),
  KEY `link_visible` (`link_visible`)
) ENGINE=MyISAM AUTO_INCREMENT=9 DEFAULT CHARSET=utf8;

--
-- Dumping data for table `links`
--

LOCK TABLES `links` WRITE;
/*!40000 ALTER TABLE `links` DISABLE KEYS */;
INSERT INTO `links` VALUES (1,'http://codex.wordpress.org/','Documentation','','',0,'','Y',1,0,'0000-00-00 00:00:00','','',''),(2,'http://wordpress.org/development/','Development Blog','','',0,'','Y',1,0,'0000-00-00 00:00:00','','','http://wordpress.org/development/feed/'),(3,'http://wordpress.org/extend/ideas/','Suggest Ideas','','',0,'','Y',1,0,'0000-00-00 00:00:00','','',''),(4,'http://wordpress.org/support/','Support Forum','','',0,'','Y',1,0,'0000-00-00 00:00:00','','',''),(5,'http://wordpress.org/extend/plugins/','Plugins','','',0,'','Y',1,0,'0000-00-00 00:00:00','','',''),(6,'http://wordpress.org/extend/themes/','Themes','','',0,'','Y',1,0,'0000-00-00 00:00:00','','',''),(7,'http://planet.wordpress.org/','WordPress Planet','','',0,'','Y',1,0,'0000-00-00 00:00:00','','',''),(8,'http://www.simplescripts.com/','SimpleScripts','','',0,'','Y',1,0,'0000-00-00 00:00:00','','','');
/*!40000 ALTER TABLE `links` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `options`
--

DROP TABLE IF EXISTS `options`;
CREATE TABLE `options` (
  `option_id` bigint(20) NOT NULL auto_increment,
  `blog_id` int(11) NOT NULL default '0',
  `option_name` varchar(64) NOT NULL default '',
  `option_value` longtext NOT NULL,
  `autoload` varchar(20) NOT NULL default 'yes',
  PRIMARY KEY  (`option_id`,`blog_id`,`option_name`),
  KEY `option_name` (`option_name`)
) ENGINE=MyISAM AUTO_INCREMENT=97 DEFAULT CHARSET=utf8;

--
-- Dumping data for table `options`
--

LOCK TABLES `options` WRITE;
/*!40000 ALTER TABLE `options` DISABLE KEYS */;
INSERT INTO `options` VALUES (1,0,'siteurl','http://www.a-bunch-of-geniuses.net/blog','yes'),(2,0,'blogname','A bunch of geniuses','yes'),(3,0,'blogdescription','Yes we are','yes'),(4,0,'users_can_register','0','yes'),(5,0,'admin_email','you@yourdomain.com','yes'),(6,0,'start_of_week','1','yes'),(7,0,'use_balanceTags','0','yes'),(8,0,'use_smilies','1','yes'),(9,0,'require_name_email','1','yes'),(10,0,'comments_notify','1','yes'),(11,0,'posts_per_rss','10','yes'),(12,0,'rss_excerpt_length','50','yes'),(13,0,'rss_use_excerpt','0','yes'),(14,0,'mailserver_url','mail.example.com','yes'),(15,0,'mailserver_login','login@example.com','yes'),(16,0,'mailserver_pass','password','yes'),(17,0,'mailserver_port','110','yes'),(18,0,'default_category','1','yes'),(19,0,'default_comment_status','open','yes'),(20,0,'default_ping_status','open','yes'),(21,0,'default_pingback_flag','1','yes'),(22,0,'default_post_edit_rows','10','yes'),(23,0,'posts_per_page','10','yes'),(24,0,'what_to_show','posts','yes'),(25,0,'date_format','F j, Y','yes'),(26,0,'time_format','g:i a','yes'),(27,0,'links_updated_date_format','F j, Y g:i a','yes'),(28,0,'links_recently_updated_prepend','<em>','yes'),(29,0,'links_recently_updated_append','</em>','yes'),(30,0,'links_recently_updated_time','120','yes'),(31,0,'comment_moderation','0','yes'),(32,0,'moderation_notify','1','yes'),(33,0,'permalink_structure','','yes'),(34,0,'gzipcompression','0','yes'),(35,0,'hack_file','0','yes'),(36,0,'blog_charset','UTF-8','yes'),(37,0,'moderation_keys','','no'),(38,0,'active_plugins','','yes'),(39,0,'home','http://www.a-bunch-of-geniuses.net/blog','yes'),(40,0,'category_base','','yes'),(41,0,'ping_sites','http://rpc.pingomatic.com/','yes'),(42,0,'advanced_edit','0','yes'),(43,0,'comment_max_links','2','yes'),(44,0,'gmt_offset','-6','yes'),(45,0,'default_email_category','1','yes'),(46,0,'recently_edited','','no'),(47,0,'use_linksupdate','0','yes'),(48,0,'template','default','yes'),(49,0,'stylesheet','default','yes'),(50,0,'comment_whitelist','1','yes'),(51,0,'page_uris','','yes'),(52,0,'blacklist_keys','','no'),(53,0,'comment_registration','0','yes'),(54,0,'rss_language','en','yes'),(55,0,'html_type','text/html','yes'),(56,0,'use_trackback','0','yes'),(57,0,'default_role','subscriber','yes'),(58,0,'db_version','7796','yes'),(59,0,'uploads_use_yearmonth_folders','1','yes'),(60,0,'upload_path','wp-content/uploads','yes'),(61,0,'secret','ShUb#&jwYk9nbZjh6LB3l*53A)E!N@8P%rgesqcyqPM*kn5@DZHCv%znj&HniFvW','yes'),(62,0,'blog_public','1','yes'),(63,0,'default_link_category','2','yes'),(64,0,'show_on_front','posts','yes'),(65,0,'tag_base','','yes'),(66,0,'show_avatars','1','yes'),(67,0,'avatar_rating','G','yes'),(68,0,'upload_url_path','','yes'),(69,0,'thumbnail_size_w','150','yes'),(70,0,'thumbnail_size_h','150','yes'),(71,0,'thumbnail_crop','1','yes'),(72,0,'medium_size_w','300','yes'),(73,0,'medium_size_h','300','yes'),(74,0,'user_roles','a:5:{s:13:\"administrator\";a:2:{s:4:\"name\";s:23:\"Administrator|User role\";s:12:\"capabilities\";a:49:{s:13:\"switch_themes\";b:1;s:11:\"edit_themes\";b:1;s:16:\"activate_plugins\";b:1;s:12:\"edit_plugins\";b:1;s:10:\"edit_users\";b:1;s:10:\"edit_files\";b:1;s:14:\"manage_options\";b:1;s:17:\"moderate_comments\";b:1;s:17:\"manage_categories\";b:1;s:12:\"manage_links\";b:1;s:12:\"upload_files\";b:1;s:6:\"import\";b:1;s:15:\"unfiltered_html\";b:1;s:10:\"edit_posts\";b:1;s:17:\"edit_others_posts\";b:1;s:20:\"edit_published_posts\";b:1;s:13:\"publish_posts\";b:1;s:10:\"edit_pages\";b:1;s:4:\"read\";b:1;s:8:\"level_10\";b:1;s:7:\"level_9\";b:1;s:7:\"level_8\";b:1;s:7:\"level_7\";b:1;s:7:\"level_6\";b:1;s:7:\"level_5\";b:1;s:7:\"level_4\";b:1;s:7:\"level_3\";b:1;s:7:\"level_2\";b:1;s:7:\"level_1\";b:1;s:7:\"level_0\";b:1;s:17:\"edit_others_pages\";b:1;s:20:\"edit_published_pages\";b:1;s:13:\"publish_pages\";b:1;s:12:\"delete_pages\";b:1;s:19:\"delete_others_pages\";b:1;s:22:\"delete_published_pages\";b:1;s:12:\"delete_posts\";b:1;s:19:\"delete_others_posts\";b:1;s:22:\"delete_published_posts\";b:1;s:20:\"delete_private_posts\";b:1;s:18:\"edit_private_posts\";b:1;s:18:\"read_private_posts\";b:1;s:20:\"delete_private_pages\";b:1;s:18:\"edit_private_pages\";b:1;s:18:\"read_private_pages\";b:1;s:12:\"delete_users\";b:1;s:12:\"create_users\";b:1;s:17:\"unfiltered_upload\";b:1;s:14:\"edit_dashboard\";b:1;}}s:6:\"editor\";a:2:{s:4:\"name\";s:16:\"Editor|User role\";s:12:\"capabilities\";a:34:{s:17:\"moderate_comments\";b:1;s:17:\"manage_categories\";b:1;s:12:\"manage_links\";b:1;s:12:\"upload_files\";b:1;s:15:\"unfiltered_html\";b:1;s:10:\"edit_posts\";b:1;s:17:\"edit_others_posts\";b:1;s:20:\"edit_published_posts\";b:1;s:13:\"publish_posts\";b:1;s:10:\"edit_pages\";b:1;s:4:\"read\";b:1;s:7:\"level_7\";b:1;s:7:\"level_6\";b:1;s:7:\"level_5\";b:1;s:7:\"level_4\";b:1;s:7:\"level_3\";b:1;s:7:\"level_2\";b:1;s:7:\"level_1\";b:1;s:7:\"level_0\";b:1;s:17:\"edit_others_pages\";b:1;s:20:\"edit_published_pages\";b:1;s:13:\"publish_pages\";b:1;s:12:\"delete_pages\";b:1;s:19:\"delete_others_pages\";b:1;s:22:\"delete_published_pages\";b:1;s:12:\"delete_posts\";b:1;s:19:\"delete_others_posts\";b:1;s:22:\"delete_published_posts\";b:1;s:20:\"delete_private_posts\";b:1;s:18:\"edit_private_posts\";b:1;s:18:\"read_private_posts\";b:1;s:20:\"delete_private_pages\";b:1;s:18:\"edit_private_pages\";b:1;s:18:\"read_private_pages\";b:1;}}s:6:\"author\";a:2:{s:4:\"name\";s:16:\"Author|User role\";s:12:\"capabilities\";a:10:{s:12:\"upload_files\";b:1;s:10:\"edit_posts\";b:1;s:20:\"edit_published_posts\";b:1;s:13:\"publish_posts\";b:1;s:4:\"read\";b:1;s:7:\"level_2\";b:1;s:7:\"level_1\";b:1;s:7:\"level_0\";b:1;s:12:\"delete_posts\";b:1;s:22:\"delete_published_posts\";b:1;}}s:11:\"contributor\";a:2:{s:4:\"name\";s:21:\"Contributor|User role\";s:12:\"capabilities\";a:5:{s:10:\"edit_posts\";b:1;s:4:\"read\";b:1;s:7:\"level_1\";b:1;s:7:\"level_0\";b:1;s:12:\"delete_posts\";b:1;}}s:10:\"subscriber\";a:2:{s:4:\"name\";s:20:\"Subscriber|User role\";s:12:\"capabilities\";a:2:{s:4:\"read\";b:1;s:7:\"level_0\";b:1;}}}','yes'),(75,0,'update_core','O:8:\"stdClass\":5:{s:12:\"last_checked\";i:1216297054;s:15:\"version_checked\";s:5:\"2.5.1\";s:8:\"response\";s:7:\"upgrade\";s:3:\"url\";s:30:\"http://wordpress.org/download/\";s:7:\"current\";s:3:\"2.6\";}','yes'),(76,0,'dashboard_widget_options','a:3:{s:24:\"dashboard_incoming_links\";a:4:{s:4:\"link\";s:121:\"http://blogsearch.google.com/blogsearch?hl=en&scoring=d&partner=wordpress&q=link:http://www.a-bunch-of-geniuses.net/blog/\";s:3:\"url\";s:154:\"http://blogsearch.google.com/blogsearch_feeds?hl=en&scoring=d&ie=utf-8&num=10&output=rss&partner=wordpress&q=link:http://www.a-bunch-of-geniuses.net/blog/\";s:5:\"items\";i:5;s:9:\"show_date\";i:0;}s:17:\"dashboard_primary\";a:7:{s:4:\"link\";s:33:\"http://wordpress.org/development/\";s:3:\"url\";s:38:\"http://wordpress.org/development/feed/\";s:5:\"title\";s:26:\"WordPress Development Blog\";s:5:\"items\";i:2;s:12:\"show_summary\";i:1;s:11:\"show_author\";i:0;s:9:\"show_date\";i:1;}s:19:\"dashboard_secondary\";a:4:{s:4:\"link\";s:28:\"http://planet.wordpress.org/\";s:3:\"url\";s:33:\"http://planet.wordpress.org/feed/\";s:5:\"title\";s:20:\"Other WordPress News\";s:5:\"items\";i:15;}}','yes'),(77,0,'current_theme','WordPress Default','yes'),(78,0,'sidebars_widgets','a:1:{s:13:\"array_version\";i:3;}','yes'),(79,0,'rss_deabea44a890150a645279f79a180bc6','O:9:\"MagpieRSS\":17:{s:6:\"parser\";i:0;s:12:\"current_item\";a:0:{}s:5:\"items\";a:0:{}s:7:\"channel\";a:4:{s:5:\"title\";s:66:\"link:http://www.a-bunch-of-geniuses.net/blog/ - Google Blog Search\";s:4:\"link\";s:124:\"http://blogsearch.google.com/blogsearch?hl=en&scoring=d&ie=ISO-8859-1&num=10&q=link:http://www.a-bunch-of-geniuses.net/blog/\";s:11:\"description\";s:100:\"Your search - <b>link:http://www.a-bunch-of-geniuses.net/blog/</b> - did not match any documents.   \";s:7:\"tagline\";s:100:\"Your search - <b>link:http://www.a-bunch-of-geniuses.net/blog/</b> - did not match any documents.   \";}s:9:\"textinput\";a:0:{}s:5:\"image\";a:0:{}s:9:\"feed_type\";s:3:\"RSS\";s:12:\"feed_version\";s:3:\"2.0\";s:5:\"stack\";a:0:{}s:9:\"inchannel\";b:0;s:6:\"initem\";b:0;s:9:\"incontent\";b:0;s:11:\"intextinput\";b:0;s:7:\"inimage\";b:0;s:13:\"current_field\";s:0:\"\";s:17:\"current_namespace\";b:0;s:19:\"_CONTENT_CONSTRUCTS\";a:6:{i:0;s:7:\"content\";i:1;s:7:\"summary\";i:2;s:4:\"info\";i:3;s:5:\"title\";i:4;s:7:\"tagline\";i:5;s:9:\"copyright\";}}','no'),(80,0,'rss_deabea44a890150a645279f79a180bc6_ts','1214839987','no'),(81,0,'rss_0ff4b43bd116a9d8720d689c80e7dfd4','O:9:\"MagpieRSS\":19:{s:6:\"parser\";i:0;s:12:\"current_item\";a:0:{}s:5:\"items\";a:10:{i:0;a:12:{s:5:\"title\";s:24:\"WordPress Birthday Party\";s:4:\"link\";s:56:\"http://wordpress.org/development/2008/05/birthday-party/\";s:8:\"comments\";s:65:\"http://wordpress.org/development/2008/05/birthday-party/#comments\";s:7:\"pubdate\";s:31:\"Sun, 25 May 2008 05:51:03 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:8:\"category\";s:6:\"Events\";s:4:\"guid\";s:39:\"http://wordpress.org/development/?p=243\";s:11:\"description\";s:321:\"On Tuesday, May 27th, WordPress will turn 5 years old. We&#8217;ve come a long way from that original 0.7 release.\nTo celebrate we&#8217;re throwing a party in San Francisco at 111 Minna, starting at 9PM. You can get the full details and RSVP on Upcoming.org or on Facebook.\nI hope you see some of you there, should [...]\";s:7:\"content\";a:1:{s:7:\"encoded\";s:874:\"<p>On Tuesday, May 27th, WordPress will turn 5 years old. We&#8217;ve come a long way from <a href=\"http://wordpress.org/development/2003/05/wordpress-now-available/\">that original 0.7 release</a>.</p>\n<p>To celebrate we&#8217;re throwing a party in San Francisco at 111 Minna, starting at 9PM. You can get the full details and RSVP <a href=\"http://upcoming.yahoo.com/event/714298\">on Upcoming.org</a> or <a href=\"http://www.facebook.com/event.php?eid=44552835108\">on Facebook</a>.</p>\n<p>I hope you see some of you there, should be a fun time.</p>\n<p>If you host a party in your area for WordPress&#8217; 5th, let us know and we&#8217;ll post it here.</p>\n<p><strong>Update:</strong> Party in Sydney! <a href=\"http://bethesignal.org/blog/2008/05/25/wordpress-party-sydney-style/\">Blog post</a>, <a href=\"http://www.facebook.com/event.php?eid=13815594694\">Facebook</a>.</p>\n\";}s:3:\"wfw\";a:1:{s:10:\"commentrss\";s:61:\"http://wordpress.org/development/2008/05/birthday-party/feed/\";}s:7:\"summary\";s:321:\"On Tuesday, May 27th, WordPress will turn 5 years old. We&#8217;ve come a long way from that original 0.7 release.\nTo celebrate we&#8217;re throwing a party in San Francisco at 111 Minna, starting at 9PM. You can get the full details and RSVP on Upcoming.org or on Facebook.\nI hope you see some of you there, should [...]\";s:12:\"atom_content\";s:874:\"<p>On Tuesday, May 27th, WordPress will turn 5 years old. We&#8217;ve come a long way from <a href=\"http://wordpress.org/development/2003/05/wordpress-now-available/\">that original 0.7 release</a>.</p>\n<p>To celebrate we&#8217;re throwing a party in San Francisco at 111 Minna, starting at 9PM. You can get the full details and RSVP <a href=\"http://upcoming.yahoo.com/event/714298\">on Upcoming.org</a> or <a href=\"http://www.facebook.com/event.php?eid=44552835108\">on Facebook</a>.</p>\n<p>I hope you see some of you there, should be a fun time.</p>\n<p>If you host a party in your area for WordPress&#8217; 5th, let us know and we&#8217;ll post it here.</p>\n<p><strong>Update:</strong> Party in Sydney! <a href=\"http://bethesignal.org/blog/2008/05/25/wordpress-party-sydney-style/\">Blog post</a>, <a href=\"http://www.facebook.com/event.php?eid=13815594694\">Facebook</a>.</p>\n\";}i:1;a:12:{s:5:\"title\";s:29:\"Usability Testing in New York\";s:4:\"link\";s:71:\"http://wordpress.org/development/2008/05/usability-testing-in-new-york/\";s:8:\"comments\";s:80:\"http://wordpress.org/development/2008/05/usability-testing-in-new-york/#comments\";s:7:\"pubdate\";s:31:\"Tue, 20 May 2008 22:13:49 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Ryan\";}s:8:\"category\";s:11:\"Development\";s:4:\"guid\";s:39:\"http://wordpress.org/development/?p=241\";s:11:\"description\";s:98:\"We&#8217;re doing some usability testing in New York City.  Join in if you&#8217;re in the area.\n\";s:7:\"content\";a:1:{s:7:\"encoded\";s:187:\"<p><a href=\"http://en.blog.wordpress.com/2008/05/20/new-york-usability-testing/\">We&#8217;re doing some usability testing in New York City</a>.  Join in if you&#8217;re in the area.</p>\n\";}s:3:\"wfw\";a:1:{s:10:\"commentrss\";s:76:\"http://wordpress.org/development/2008/05/usability-testing-in-new-york/feed/\";}s:7:\"summary\";s:98:\"We&#8217;re doing some usability testing in New York City.  Join in if you&#8217;re in the area.\n\";s:12:\"atom_content\";s:187:\"<p><a href=\"http://en.blog.wordpress.com/2008/05/20/new-york-usability-testing/\">We&#8217;re doing some usability testing in New York City</a>.  Join in if you&#8217;re in the area.</p>\n\";}i:2;a:12:{s:5:\"title\";s:18:\"Upcoming WordCamps\";s:4:\"link\";s:60:\"http://wordpress.org/development/2008/04/upcoming-wordcamps/\";s:8:\"comments\";s:69:\"http://wordpress.org/development/2008/04/upcoming-wordcamps/#comments\";s:7:\"pubdate\";s:31:\"Wed, 30 Apr 2008 09:54:48 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:8:\"category\";s:6:\"Events\";s:4:\"guid\";s:39:\"http://wordpress.org/development/?p=240\";s:11:\"description\";s:343:\"WordCamps are my favorite events to go to because there&#8217;s something about the core WordPress community that attracts smart folks with good philosophies that are fun to hang out with. In this post I&#8217;ve collated the upcoming WordCamps we know about, including the one in San Francisco. Hopefully there will be one nearby so you [...]\";s:7:\"content\";a:1:{s:7:\"encoded\";s:1446:\"<p>WordCamps are my favorite events to go to because there&#8217;s something about the core WordPress community that attracts smart folks with good philosophies that are fun to hang out with. In this post I&#8217;ve collated the upcoming WordCamps we know about, including the one in San Francisco. Hopefully there will be one nearby so you can meet other WordPressers in your area.</p>\n<p><strong>WordCamp San Francisco</strong> will be August 16 at the Mission Bay Conference Center.</p>\n<p><a href=\"http://barcamp.org/WordCampParis\">WordCamp Paris will be on May 3rd</a>. <a href=\"http://wordcamp.fr/\">Here&#8217;s their official site</a>.</p>\n<p><a href=\"http://iwordpresscamp.wordpress.com/2008/02/20/european-wordcamp-beta/\">WordCamp Italy in Milan will be May 10th</a>. (And I believe I&#8217;ll be there.)</p>\n<p><a href=\"http://wiki.wordcampuk.tonyscott.org.uk/Main_Page\">WordCamp Birmingham UK</a> will be July 19-20.</p>\n<p><a href=\"http://wordcamptoronto.eventbrite.com/\">WordCamp Toronto will be October 4th</a>.</p>\n<p>There are people in the planning stages in <a href=\"http://australia.wordcamp.org/\">Australia</a>, Philippines, Beijing, Utah, Hawaii, UK, NYC, and possibly others, so if you live in one of those areas and would like to help set up a WordCamp in your area Google around or connect with bloggers in your area.</p>\n<p>You can always find out more at <a href=\"http://central.wordcamp.org/\">WordCamp Central</a>.</p>\n\";}s:3:\"wfw\";a:1:{s:10:\"commentrss\";s:65:\"http://wordpress.org/development/2008/04/upcoming-wordcamps/feed/\";}s:7:\"summary\";s:343:\"WordCamps are my favorite events to go to because there&#8217;s something about the core WordPress community that attracts smart folks with good philosophies that are fun to hang out with. In this post I&#8217;ve collated the upcoming WordCamps we know about, including the one in San Francisco. Hopefully there will be one nearby so you [...]\";s:12:\"atom_content\";s:1446:\"<p>WordCamps are my favorite events to go to because there&#8217;s something about the core WordPress community that attracts smart folks with good philosophies that are fun to hang out with. In this post I&#8217;ve collated the upcoming WordCamps we know about, including the one in San Francisco. Hopefully there will be one nearby so you can meet other WordPressers in your area.</p>\n<p><strong>WordCamp San Francisco</strong> will be August 16 at the Mission Bay Conference Center.</p>\n<p><a href=\"http://barcamp.org/WordCampParis\">WordCamp Paris will be on May 3rd</a>. <a href=\"http://wordcamp.fr/\">Here&#8217;s their official site</a>.</p>\n<p><a href=\"http://iwordpresscamp.wordpress.com/2008/02/20/european-wordcamp-beta/\">WordCamp Italy in Milan will be May 10th</a>. (And I believe I&#8217;ll be there.)</p>\n<p><a href=\"http://wiki.wordcampuk.tonyscott.org.uk/Main_Page\">WordCamp Birmingham UK</a> will be July 19-20.</p>\n<p><a href=\"http://wordcamptoronto.eventbrite.com/\">WordCamp Toronto will be October 4th</a>.</p>\n<p>There are people in the planning stages in <a href=\"http://australia.wordcamp.org/\">Australia</a>, Philippines, Beijing, Utah, Hawaii, UK, NYC, and possibly others, so if you live in one of those areas and would like to help set up a WordCamp in your area Google around or connect with bloggers in your area.</p>\n<p>You can always find out more at <a href=\"http://central.wordcamp.org/\">WordCamp Central</a>.</p>\n\";}i:3;a:12:{s:5:\"title\";s:15:\"WordPress 2.5.1\";s:4:\"link\";s:55:\"http://wordpress.org/development/2008/04/wordpress-251/\";s:8:\"comments\";s:64:\"http://wordpress.org/development/2008/04/wordpress-251/#comments\";s:7:\"pubdate\";s:31:\"Fri, 25 Apr 2008 15:54:19 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Ryan\";}s:8:\"category\";s:8:\"Releases\";s:4:\"guid\";s:39:\"http://wordpress.org/development/?p=231\";s:11:\"description\";s:360:\"Version 2.5.1 of WordPress is now available. It includes a number of bug fixes, performance enhancements, and one very important security fix. We recommend everyone update immediately, particularly if your blog has open registration. The vulnerability is not public but it will be shortly.\nIn addition to the security fix, 2.5.1 contains many bug fixes.  [...]\";s:7:\"content\";a:1:{s:7:\"encoded\";s:2251:\"<p>Version 2.5.1 of WordPress is now available. It includes a number of bug fixes, performance enhancements, and one very important security fix. We recommend everyone update immediately, particularly if your blog has open registration. The vulnerability is not public but it will be shortly.</p>\n<p>In addition to the security fix, 2.5.1 contains many bug fixes.  If you are interested only in the security fixes, you can download these corrected copies of <a href=\"http://trac.wordpress.org/browser/tags/2.5.1/wp-includes/pluggable.php?format=raw\">wp-includes/pluggable.php</a>, <a href=\"http://trac.wordpress.org/browser/tags/2.5.1/wp-admin/includes/media.php?format=raw\">wp-admin/includes/media.php</a>, and <a href=\"http://trac.wordpress.org/browser/tags/2.5.1/wp-admin/media.php?format=raw\">wp-admin/media.php</a>.  Replace your existing copies of these files with these new copies.</p>\n<p>If you download the entire 2.5.1 release, you will be getting <a href=\"http://trac.wordpress.org/query?status=closed&amp;milestone=2.5.1&amp;resolution=fixed&amp;order=priority\">over 70 other fixes</a>.  2.5.1 focuses on fixing the most annoying bugs and improving performance.  Here are some highlights:</p>\n<ul>\n<li>Performance improvements for the Dashboard, Write Post, and Edit Comments pages.</li>\n<li>Better performance for those who have many categories</li>\n<li>Media Uploader fixes</li>\n<li>An upgrade to TinyMCE 3.0.7</li>\n<li>Widget Administration fixes</li>\n<li>Various usability improvements</li>\n<li>Layout fixes for IE</li>\n</ul>\n<h3>Secret lives of blogs</h3>\n<p>Since 2.5 your <code>wp-config.php</code> file allows a new constant called <code>SECRET_KEY</code> which basically is meant to introduce a little permanent randomness into the cryptographic functions used for cookies in WordPress. You can <a href=\"http://api.wordpress.org/secret-key/1.0/\">visit this link we set up to get a unique secret key</a> for your config file. (It&#8217;s unique and random on every page load.) Having this line in your config file helps secure your blog.</p>\n<p>Many thanks to <a href=\"http://www.cl.cam.ac.uk/~sjm217/\">Steven Murdoch</a> for responsibly reporting the security issue (CVE-2008-1930) and Alex Concha for reporting an XSS issue.</p>\n\";}s:3:\"wfw\";a:1:{s:10:\"commentrss\";s:60:\"http://wordpress.org/development/2008/04/wordpress-251/feed/\";}s:7:\"summary\";s:360:\"Version 2.5.1 of WordPress is now available. It includes a number of bug fixes, performance enhancements, and one very important security fix. We recommend everyone update immediately, particularly if your blog has open registration. The vulnerability is not public but it will be shortly.\nIn addition to the security fix, 2.5.1 contains many bug fixes.  [...]\";s:12:\"atom_content\";s:2251:\"<p>Version 2.5.1 of WordPress is now available. It includes a number of bug fixes, performance enhancements, and one very important security fix. We recommend everyone update immediately, particularly if your blog has open registration. The vulnerability is not public but it will be shortly.</p>\n<p>In addition to the security fix, 2.5.1 contains many bug fixes.  If you are interested only in the security fixes, you can download these corrected copies of <a href=\"http://trac.wordpress.org/browser/tags/2.5.1/wp-includes/pluggable.php?format=raw\">wp-includes/pluggable.php</a>, <a href=\"http://trac.wordpress.org/browser/tags/2.5.1/wp-admin/includes/media.php?format=raw\">wp-admin/includes/media.php</a>, and <a href=\"http://trac.wordpress.org/browser/tags/2.5.1/wp-admin/media.php?format=raw\">wp-admin/media.php</a>.  Replace your existing copies of these files with these new copies.</p>\n<p>If you download the entire 2.5.1 release, you will be getting <a href=\"http://trac.wordpress.org/query?status=closed&amp;milestone=2.5.1&amp;resolution=fixed&amp;order=priority\">over 70 other fixes</a>.  2.5.1 focuses on fixing the most annoying bugs and improving performance.  Here are some highlights:</p>\n<ul>\n<li>Performance improvements for the Dashboard, Write Post, and Edit Comments pages.</li>\n<li>Better performance for those who have many categories</li>\n<li>Media Uploader fixes</li>\n<li>An upgrade to TinyMCE 3.0.7</li>\n<li>Widget Administration fixes</li>\n<li>Various usability improvements</li>\n<li>Layout fixes for IE</li>\n</ul>\n<h3>Secret lives of blogs</h3>\n<p>Since 2.5 your <code>wp-config.php</code> file allows a new constant called <code>SECRET_KEY</code> which basically is meant to introduce a little permanent randomness into the cryptographic functions used for cookies in WordPress. You can <a href=\"http://api.wordpress.org/secret-key/1.0/\">visit this link we set up to get a unique secret key</a> for your config file. (It&#8217;s unique and random on every page load.) Having this line in your config file helps secure your blog.</p>\n<p>Many thanks to <a href=\"http://www.cl.cam.ac.uk/~sjm217/\">Steven Murdoch</a> for responsibly reporting the security issue (CVE-2008-1930) and Alex Concha for reporting an XSS issue.</p>\n\";}i:4;a:12:{s:5:\"title\";s:23:\"An Event Apart Discount\";s:4:\"link\";s:65:\"http://wordpress.org/development/2008/04/an-event-apart-discount/\";s:8:\"comments\";s:74:\"http://wordpress.org/development/2008/04/an-event-apart-discount/#comments\";s:7:\"pubdate\";s:31:\"Tue, 22 Apr 2008 22:33:19 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:8:\"category\";s:36:\"Eventsan event aparthappy cogzeldman\";s:4:\"guid\";s:65:\"http://wordpress.org/development/2008/04/an-event-apart-discount/\";s:11:\"description\";s:321:\"An Event Apart is a web design and development conference which features some of the same fine folks who helped out with WordPress 2.5. (And many others.) I attended the one in Chicago a while back and was engaged the whole day in interesting talks on design, writing copy as interface, advanced CSS, and creativity [...]\";s:7:\"content\";a:1:{s:7:\"encoded\";s:1194:\"<p><a href=\"http://aneventapart.com/\">An Event Apart</a> is a web design and development conference which features some of the same fine folks who helped out with WordPress 2.5. (And many others.) I attended the one in Chicago a while back and was engaged the whole day in interesting talks on design, writing copy as interface, advanced CSS, and creativity &#8212; each topic presented by the leading folks in the field.</p>\n<p>The conference normally costs just under a thousand dollars to attend, which is well worth it, but because of our association with the folks they&#8217;ve set up a discount for WordPress users.</p>\n<p>If you enter AEAWP on checkout the price drops to $795, or $200 below the regular registration fee. The coupon is unlimited, and can be used for one ticket or five. They have events coming up in New Orleans, Boston, San Francisco, and Chicago. <a href=\"http://aneventapart.com/\">You can learn more and register on their site at aneventapart.com</a>.</p>\n<p>We&#8217;ll also have some final dates for WordCamp San Francisco coming up, I&#8217;ll be posting those soonish so people can start making vacation and travel plans. (Nothing like a blogging vacation.)</p>\n\";}s:3:\"wfw\";a:1:{s:10:\"commentrss\";s:70:\"http://wordpress.org/development/2008/04/an-event-apart-discount/feed/\";}s:7:\"summary\";s:321:\"An Event Apart is a web design and development conference which features some of the same fine folks who helped out with WordPress 2.5. (And many others.) I attended the one in Chicago a while back and was engaged the whole day in interesting talks on design, writing copy as interface, advanced CSS, and creativity [...]\";s:12:\"atom_content\";s:1194:\"<p><a href=\"http://aneventapart.com/\">An Event Apart</a> is a web design and development conference which features some of the same fine folks who helped out with WordPress 2.5. (And many others.) I attended the one in Chicago a while back and was engaged the whole day in interesting talks on design, writing copy as interface, advanced CSS, and creativity &#8212; each topic presented by the leading folks in the field.</p>\n<p>The conference normally costs just under a thousand dollars to attend, which is well worth it, but because of our association with the folks they&#8217;ve set up a discount for WordPress users.</p>\n<p>If you enter AEAWP on checkout the price drops to $795, or $200 below the regular registration fee. The coupon is unlimited, and can be used for one ticket or five. They have events coming up in New Orleans, Boston, San Francisco, and Chicago. <a href=\"http://aneventapart.com/\">You can learn more and register on their site at aneventapart.com</a>.</p>\n<p>We&#8217;ll also have some final dates for WordCamp San Francisco coming up, I&#8217;ll be posting those soonish so people can start making vacation and travel plans. (Nothing like a blogging vacation.)</p>\n\";}i:5;a:12:{s:5:\"title\";s:13:\"WordPress 2.5\";s:4:\"link\";s:62:\"http://wordpress.org/development/2008/03/wordpress-25-brecker/\";s:8:\"comments\";s:71:\"http://wordpress.org/development/2008/03/wordpress-25-brecker/#comments\";s:7:\"pubdate\";s:31:\"Sat, 29 Mar 2008 16:49:45 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:8:\"category\";s:8:\"Releases\";s:4:\"guid\";s:39:\"http://wordpress.org/development/?p=228\";s:11:\"description\";s:391:\"WordPress 2.5, the culmination of six months of work by the WordPress community, people just like you. The improvements in 2.5 are numerous, and almost entirely a result of your feedback: multi-file uploading, one-click plugin upgrades, built-in galleries, customizable dashboard, salted passwords and cookie encryption, media library, a WYSIWYG that doesn&#8217;t mess with your code, [...]\";s:7:\"content\";a:1:{s:7:\"encoded\";s:15112:\"<p>WordPress 2.5, the culmination of six months of work by the WordPress community, people just like you. The improvements in 2.5 are numerous, and almost entirely a result of your feedback: multi-file uploading, one-click plugin upgrades, built-in galleries, customizable dashboard, salted passwords and cookie encryption, media library, a WYSIWYG that doesn&#8217;t mess with your code, concurrent post editing protection, full-screen writing, and search that covers posts <em>and</em> pages.</p>\n<p>For a short overview of the features with screenshots, it&#8217;d be best to <a href=\"http://wordpress.org/development/2008/03/25-sneak-peek/\">visit our sneak peek announcement for RC1</a>. Or check out <a href=\"http://wordpress.org/development/2008/03/wordpress-25-rc2/\">a 4-minute screencast of the new interface in action</a>. If you just want to jump straight to the good stuff <a href=\"http://wordpress.org/download/\">here&#8217;s where you can find 2.5 upgrade and download information</a>.</p>\n<p>If you want to see everything I would grab a cup of coffee or a mojito, because this post is epic.</p>\n<h3>User Features</h3>\n<p><strong>Cleaner, faster, less cluttered dashboard</strong> &#8212; we&#8217;ve worked hard to take your feedback about what&#8217;s most important in the dashboard and organize things to allow you to focus on what&#8217;s important &#8212; your blog &#8212; and get out of your way. In collaboration with <a href=\"http://happycog.com/\">Happy Cog</a> and the community  we&#8217;ve taken the first major step forward in the WordPress interface since version 1.5.</p>\n<p><strong>Dashboard Widgets</strong> &#8212; the dashboard home page is now a series of widgets, including ones to show you fun stats about your posting, latest comments, people linking to you, new and popular plugins, and of course WordPress news. You can customize any of the dashboard widgets to show, for example, news from your local paper instead of WP news. Plugins can also hook in, for example the <a href=\"http://wordpress.org/extend/plugins/stats/\">WordPress.com stats widget</a> adds a handy double-wide stats box.</p>\n<p><img src=\"http://s.wordpress.org/screenshots/2.5/stats-widget.png\" alt=\"\" width=\"690\" /></p>\n<p><strong>Multi-file upload with progress bar</strong> &#8212; before when you would upload a large file you&#8217;d wait forever, never knowing how far along it was. And uploading more than one photo was an exercise in patience, as you could only do one at a time. Now you can select a whole of folder images or music or videos at once and it&#8217;ll show you the progress of each upload.</p>\n<p><strong>Bonus: EXIF extraction</strong> &#8212; if you upload JPEG files with EXIF metadata like camera make and model, aperture, shutter speed, ISO, et al. WordPress will extract all the data into custom fields you can use in your template. If you use the EXIF title fields or similar those will be put into their equivalent fields in WP. Most modern digital cameras generate EXIF data.</p>\n<p><strong>Search posts and pages</strong> &#8212; search used to cover just posts, now it includes pages too, a great boon for those using WordPress as a CMS. New themes can style or sort pages differently in results.</p>\n<p><strong>Tag management</strong> &#8212; you can now add, rename, delete, and do whatever else you like to tags from inside WordPress, no plugins needed.</p>\n<p><strong>Password strength meter</strong> &#8212; when you change your password on your profile it&#8217;ll tell you how strong your password is to help you pick a good one.</p>\n<p><strong>Concurrent editing protection</strong> &#8212; for those of you on multi-author blogs, have you ever opened a post while someone was already editing it, and your auto-saves kept overwriting each other, irrecoverably losing hours of work? I bet that added a few words to your vocabulary. Now if you open a post that someone else is editing, WordPress magically locks it and prevents you from saving until the other person is done. You&#8217;ll see a message like below.</p>\n<p><img src=\"http://s.wordpress.org/screenshots/2.5/write-currently-editing.png\" alt=\"\" /></p>\n<p><strong>Few-click plugin upgrades</strong> &#8212; if the plugins you use are part of the plugin directory since 2.3 we&#8217;ve told you when they have an update available. Now we take that to the next logical step &#8212; downloading and installing the upgrade for you. This is dependent a little bit on your host setup, and it may ask you for your FTP password much like OS X or Windows will ask you for a password, but it works well on majority of hosts we were able to test, your mileage may very, plugins in mirror may be larger than they appear.</p>\n<p><strong>Friendlier visual post editor</strong> &#8212; I&#8217;m not sure how to articulate this improvement except to say &#8220;it doesn&#8217;t mess with your code anymore.&#8221; We&#8217;re now using version 3.0 of TinyMCE, which means better compatibility with Safari, and we&#8217;ve paid particular attention this release to its integration and interaction with complex HTML. It also now has a &#8220;no-distractions&#8221; mode which is like Writeroom for your browser.</p>\n<p><strong>Built-in galleries</strong> &#8212; when you take advantage of multi-file upload to upload a bunch of photos, we have a new shortcode that lets you to easily embed galleries by just putting [ gallery] (without the space) in your post. It&#8217;ll display all your thumbnails and captions and each will link each to a page where people can comment on the individual photos. I&#8217;ve been using this feature on my blog and have already uploaded over <a href=\"http://ma.tt/category/gallery/\">1,200 pictures into 23 galleries</a>. The shortcode has some hidden options too, <a href=\"http://codex.wordpress.org/Using_the_gallery_shortcode\">check out this documentation</a>.</p>\n<h3>Developer Features</h3>\n<p>Now for the geeky stuff. While we&#8217;re excited about the above features, each one represents a new opportunity or API for other developers to take to another level. (The best of which we&#8217;ll someday integrate back into WP.)</p>\n<p><strong>Salted passwords</strong> &#8212; we now use the <a href=\"http://www.openwall.com/phpass/\">phpass</a> library to stretch and salt all passwords stored in the database, which makes brute-forcing them impractical. If you use something like <a href=\"http://modauthmysql.sourceforge.net/\">mod_auth_mysql</a> we&#8217;ve <a href=\"http://wordpress.org/extend/plugins/md5-password-hashes/\">created a plugin that will allow you to use legacy MD5 hashing</a>. (The hashing is completely pluggable.) Users will automatically switch to the more secure passwords next time they log in.</p>\n<p><strong>Secure cookies</strong> &#8212; cookies are now encrypted based on the protocol described <a href=\"http://www.cse.msu.edu/~alexliu/publications/Cookie/cookie.pdf\">in this PDF paper</a>. which is something like <code>user name|expiration time|HMAC( user name|expiration time, k)</code> where <code>k = HMAC(user name|expiration time, sk)</code> and where <code>sk</code> is a secret key, which you can define in your config.</p>\n<p><strong>Easy taxonomy and URL creation</strong> &#8212; probably best illustrated with an example: I can call <code>register_taxonomy()</code> with <a href=\"http://trac.wordpress.org/attachment/ticket/6357/taxes.php\">a few arguments</a> to register a &#8220;people&#8221; taxonomy and whenever I edit an image I&#8217;ll see a UI like tags has for identifying the people in a photo, and these will be URL addressable with <code>/person/firstname-lastname/</code>. All with a single function call.</p>\n<p><strong>Inline documentation</strong> &#8212; the vast majority of the new code going into WordPress include inline documentation that explains the functions and documents their arguments.</p>\n<p><strong>Database optimization</strong> &#8212; we haven&#8217;t changed the table layout in this release, which is one of the reasons so many plugins work fine with 2.5. We have added a few new indicies and made a few default fields more flexible based on some bottlenecks we found on WordPress.com, which now hosts 2.7 million WordPress blogs. It should be invisible to the application, just a bit faster on the database side.</p>\n<p><strong>$wpdb->prepare()</strong> &#8212; now almost all of the SQL in WordPress is prepared first, and the same functions are available to your plugins. This should prevent elementary SQL escaping issues.</p>\n<p><strong>Media buttons</strong> &#8212; the add media buttons above the post are both expandable, so you could have an &#8220;Add Google Map&#8221; button if you like, They can be overridden, so if you think you can do the video or audio tab better than we have you can replace the default.</p>\n<p><strong>Shortcode API</strong> &#8212; the new gallery functionality is powered by the new shortcode API. Shortcodes are little bracket-delineated strings that can be magically expanded at runtime to something more interesting. They give users a short, easy to type and copy/paste string they can move around their post without worrying about messing up complex HTML or embed codes. <a href=\"http://codex.wordpress.org/Shortcode_API\">The Shortcode API is fully documented</a>.</p>\n<p>Now you see why 2.5 took a little extra time. <img src=\'http://wordpress.org/development/wp-includes/images/smilies/icon_smile.gif\' alt=\':)\' class=\'wp-smiley\' /> </p>\n<h3>Upgrade Notes</h3>\n<p>2.5 does include security fixes so it is recommended for all users, the 2.3 branch will no longer be updated. <a href=\"http://codex.wordpress.org/Upgrading_WordPress\">The upgrade instructions for this version are pretty much the same as any other version</a>. The most important thing to check is your plugins, so if for example everything works except the new uploader, a legacy plugin might be causing a javascript error on the page and breaking it. If something goes wrong, the safest thing to do is turn your plugins off (we have a button to do them all at once, now) and turn them back on one-by-one, testing the problem along the way. This has solved almost everybody&#8217;s problems in testing, and it also lets you know which plugin author to show some love to so they&#8217;ll update their plugin, and which plugin authors already have so you can shower them with praises on your blog.</p>\n<p>One brief note about some of the new upload and plugin upgrade features, there are some edge-case hosting platforms, like versions of Lighttpd before 1.5 or over-agressive mod_security rules, which can break. If something isn&#8217;t working like it was looked in the screenshot, ask your host if there&#8217;s something on the server side which may be interfering. Hosts, feel free to join and post to <a href=\"http://lists.automattic.com/mailman/listinfo/wp-testers\">our wp-testers mailing list</a> if you have an environment that requires some extra code to work around. We&#8217;d be happy to include it in the next update.</p>\n<p><em>Quick tip:</em> in 2.5 you click the name of things to edit them, like your username to edit your profile or the title of a post to edit it.</p>\n<h3>The Community is Growing</h3>\n<p>More than growing, it&#8217;s on fire. We always talk about things like downloads, and the 2.3 branch has already had <strong>1.92 million downloads</strong> as I write this post, but this time we have some far more interesting information I&#8217;d like to share.</p>\n<p>There were over 1,200 commits to our repository since 2.3.0 and <strong>over 90 people were credited</strong> in them. This means in our core code, not plugins, there were at least 90 individual folks that contributed something high-quality enough that it made the cut to be part of the download you guys get today. I had no idea this group of people was so large.</p>\n<p>Outside of the <a href=\"http://wordpress.org/about/\">core commit team</a>, there was particular help from these people, in rough order of number of credits and tickets: <a href=\"http://blogwaffe.com/\">mdawaffe (Michael Adams)</a>, <a href=\"http://www.laptoptips.ca/\">azaozz (Andrew Ozz)</a>, <a href=\"http://nb.niichavo.org/\">nbachiyski (Nikolay Bachiyski)</a>, <a href=\"http://skeltoac.com/\">andy (Andy Skelton)</a>, <a href=\"http://iammattthomas.com/\">iammattthomas (Matt Thomas)</a>, <a href=\"http://thresholdstate.com/\">tellyworth (Alex Shiels)</a>, <a href=\"http://joseph.randomnetworks.com/\">josephscott (Joseph Scott)</a>, <a href=\"http://foolswisdom.com/\">lloydbudd (Lloyd Budd)</a>, <a href=\"http://dd32.id.au/\">DD32 (Dion)</a>, <a href=\"http://www.ilfilosofo.com/\">filosofo (Austin Matzko)</a>, <a href=\"http://engel.uk.to/\">hansengel (Hans Engel)</a>, <a href=\"http://www.freecharity.org.uk/\">pishmishy</a>, ffemtcj, <a href=\"http://www.viper007bond.com/\">Viper007Bond</a>, <a href=\"http://extralogical.net/\">ionfish (Benedict Eastaugh)</a>, <a href=\"http://www.poplarware.com/\">jhodgdon (Jennifer Hodgdon)</a>, <a href=\"http://ottodestruct.com/blog/\">Otto42</a>, <a href=\"http://www.cefm.ca/\">thee17 (Charles E. Free-Melvin)</a>, and <a href=\"http://www.buayacorp.com/\">xknown</a>. Also want to thank MichaelH and Lorelle on the documentation side, and moshu, Kafkaesqui, whooami, MichaelH, Otto42, and jeremyclark13 for helping with support.</p>\n<p>The 2.5 branch is nicknamed &#8220;Brecker&#8221; in honor of <a href=\"http://en.wikipedia.org/wiki/Michael_Brecker\">Michael Brecker</a>, an exceptionally talented saxophonist who could cross styles effortlessly and never stopped experimenting and pushing himself until he passed away last year.</p>\n<h3>New WordPress.org</h3>\n<p>All of this wasn&#8217;t enough, so in our copious spare time we decided to redesign WordPress.org to better match the aesthetics of the new dashboard and also to spruce up a few areas that needed lovin&#8217;. Some parts of the site, like the Codex, might show the old style for a day or two. We know, just give us a bit of time. Thanks to <a href=\"http://iammattthomas.com/\">Matt Thomas</a> for his epic effort in designing and coding the new site.</p>\n<h3>What&#8217;s Next</h3>\n<p>As always with WordPress, we don&#8217;t claim any of these features to be perfect, or to be better than everyone else in the world, but they are done by and for the people and the one thing we do promise is that with every release we listen and do our best to improve. </p>\n<p>2.5 is a major milestone for WordPress not because it added dozens of user-requested features, but because it reaffirms that we&#8217;re as passionate about blogging as the day we started. Our community is too fierce to rest on its laurels &#8212; contrary to what pundits claim, blogging is far from &#8220;finished&#8221; and every improvement just whets our appetite for more. And more is coming.</p>\n<p>It&#8217;s a good thing WordPress doesn&#8217;t limit the length of posts, because this one would have hit it. If you made it this far, thanks for sharing a bit of your day with us. I sincerely hope this new version of WordPress helps you do what you love to do.</p>\n\";}s:3:\"wfw\";a:1:{s:10:\"commentrss\";s:67:\"http://wordpress.org/development/2008/03/wordpress-25-brecker/feed/\";}s:7:\"summary\";s:391:\"WordPress 2.5, the culmination of six months of work by the WordPress community, people just like you. The improvements in 2.5 are numerous, and almost entirely a result of your feedback: multi-file uploading, one-click plugin upgrades, built-in galleries, customizable dashboard, salted passwords and cookie encryption, media library, a WYSIWYG that doesn&#8217;t mess with your code, [...]\";s:12:\"atom_content\";s:15112:\"<p>WordPress 2.5, the culmination of six months of work by the WordPress community, people just like you. The improvements in 2.5 are numerous, and almost entirely a result of your feedback: multi-file uploading, one-click plugin upgrades, built-in galleries, customizable dashboard, salted passwords and cookie encryption, media library, a WYSIWYG that doesn&#8217;t mess with your code, concurrent post editing protection, full-screen writing, and search that covers posts <em>and</em> pages.</p>\n<p>For a short overview of the features with screenshots, it&#8217;d be best to <a href=\"http://wordpress.org/development/2008/03/25-sneak-peek/\">visit our sneak peek announcement for RC1</a>. Or check out <a href=\"http://wordpress.org/development/2008/03/wordpress-25-rc2/\">a 4-minute screencast of the new interface in action</a>. If you just want to jump straight to the good stuff <a href=\"http://wordpress.org/download/\">here&#8217;s where you can find 2.5 upgrade and download information</a>.</p>\n<p>If you want to see everything I would grab a cup of coffee or a mojito, because this post is epic.</p>\n<h3>User Features</h3>\n<p><strong>Cleaner, faster, less cluttered dashboard</strong> &#8212; we&#8217;ve worked hard to take your feedback about what&#8217;s most important in the dashboard and organize things to allow you to focus on what&#8217;s important &#8212; your blog &#8212; and get out of your way. In collaboration with <a href=\"http://happycog.com/\">Happy Cog</a> and the community  we&#8217;ve taken the first major step forward in the WordPress interface since version 1.5.</p>\n<p><strong>Dashboard Widgets</strong> &#8212; the dashboard home page is now a series of widgets, including ones to show you fun stats about your posting, latest comments, people linking to you, new and popular plugins, and of course WordPress news. You can customize any of the dashboard widgets to show, for example, news from your local paper instead of WP news. Plugins can also hook in, for example the <a href=\"http://wordpress.org/extend/plugins/stats/\">WordPress.com stats widget</a> adds a handy double-wide stats box.</p>\n<p><img src=\"http://s.wordpress.org/screenshots/2.5/stats-widget.png\" alt=\"\" width=\"690\" /></p>\n<p><strong>Multi-file upload with progress bar</strong> &#8212; before when you would upload a large file you&#8217;d wait forever, never knowing how far along it was. And uploading more than one photo was an exercise in patience, as you could only do one at a time. Now you can select a whole of folder images or music or videos at once and it&#8217;ll show you the progress of each upload.</p>\n<p><strong>Bonus: EXIF extraction</strong> &#8212; if you upload JPEG files with EXIF metadata like camera make and model, aperture, shutter speed, ISO, et al. WordPress will extract all the data into custom fields you can use in your template. If you use the EXIF title fields or similar those will be put into their equivalent fields in WP. Most modern digital cameras generate EXIF data.</p>\n<p><strong>Search posts and pages</strong> &#8212; search used to cover just posts, now it includes pages too, a great boon for those using WordPress as a CMS. New themes can style or sort pages differently in results.</p>\n<p><strong>Tag management</strong> &#8212; you can now add, rename, delete, and do whatever else you like to tags from inside WordPress, no plugins needed.</p>\n<p><strong>Password strength meter</strong> &#8212; when you change your password on your profile it&#8217;ll tell you how strong your password is to help you pick a good one.</p>\n<p><strong>Concurrent editing protection</strong> &#8212; for those of you on multi-author blogs, have you ever opened a post while someone was already editing it, and your auto-saves kept overwriting each other, irrecoverably losing hours of work? I bet that added a few words to your vocabulary. Now if you open a post that someone else is editing, WordPress magically locks it and prevents you from saving until the other person is done. You&#8217;ll see a message like below.</p>\n<p><img src=\"http://s.wordpress.org/screenshots/2.5/write-currently-editing.png\" alt=\"\" /></p>\n<p><strong>Few-click plugin upgrades</strong> &#8212; if the plugins you use are part of the plugin directory since 2.3 we&#8217;ve told you when they have an update available. Now we take that to the next logical step &#8212; downloading and installing the upgrade for you. This is dependent a little bit on your host setup, and it may ask you for your FTP password much like OS X or Windows will ask you for a password, but it works well on majority of hosts we were able to test, your mileage may very, plugins in mirror may be larger than they appear.</p>\n<p><strong>Friendlier visual post editor</strong> &#8212; I&#8217;m not sure how to articulate this improvement except to say &#8220;it doesn&#8217;t mess with your code anymore.&#8221; We&#8217;re now using version 3.0 of TinyMCE, which means better compatibility with Safari, and we&#8217;ve paid particular attention this release to its integration and interaction with complex HTML. It also now has a &#8220;no-distractions&#8221; mode which is like Writeroom for your browser.</p>\n<p><strong>Built-in galleries</strong> &#8212; when you take advantage of multi-file upload to upload a bunch of photos, we have a new shortcode that lets you to easily embed galleries by just putting [ gallery] (without the space) in your post. It&#8217;ll display all your thumbnails and captions and each will link each to a page where people can comment on the individual photos. I&#8217;ve been using this feature on my blog and have already uploaded over <a href=\"http://ma.tt/category/gallery/\">1,200 pictures into 23 galleries</a>. The shortcode has some hidden options too, <a href=\"http://codex.wordpress.org/Using_the_gallery_shortcode\">check out this documentation</a>.</p>\n<h3>Developer Features</h3>\n<p>Now for the geeky stuff. While we&#8217;re excited about the above features, each one represents a new opportunity or API for other developers to take to another level. (The best of which we&#8217;ll someday integrate back into WP.)</p>\n<p><strong>Salted passwords</strong> &#8212; we now use the <a href=\"http://www.openwall.com/phpass/\">phpass</a> library to stretch and salt all passwords stored in the database, which makes brute-forcing them impractical. If you use something like <a href=\"http://modauthmysql.sourceforge.net/\">mod_auth_mysql</a> we&#8217;ve <a href=\"http://wordpress.org/extend/plugins/md5-password-hashes/\">created a plugin that will allow you to use legacy MD5 hashing</a>. (The hashing is completely pluggable.) Users will automatically switch to the more secure passwords next time they log in.</p>\n<p><strong>Secure cookies</strong> &#8212; cookies are now encrypted based on the protocol described <a href=\"http://www.cse.msu.edu/~alexliu/publications/Cookie/cookie.pdf\">in this PDF paper</a>. which is something like <code>user name|expiration time|HMAC( user name|expiration time, k)</code> where <code>k = HMAC(user name|expiration time, sk)</code> and where <code>sk</code> is a secret key, which you can define in your config.</p>\n<p><strong>Easy taxonomy and URL creation</strong> &#8212; probably best illustrated with an example: I can call <code>register_taxonomy()</code> with <a href=\"http://trac.wordpress.org/attachment/ticket/6357/taxes.php\">a few arguments</a> to register a &#8220;people&#8221; taxonomy and whenever I edit an image I&#8217;ll see a UI like tags has for identifying the people in a photo, and these will be URL addressable with <code>/person/firstname-lastname/</code>. All with a single function call.</p>\n<p><strong>Inline documentation</strong> &#8212; the vast majority of the new code going into WordPress include inline documentation that explains the functions and documents their arguments.</p>\n<p><strong>Database optimization</strong> &#8212; we haven&#8217;t changed the table layout in this release, which is one of the reasons so many plugins work fine with 2.5. We have added a few new indicies and made a few default fields more flexible based on some bottlenecks we found on WordPress.com, which now hosts 2.7 million WordPress blogs. It should be invisible to the application, just a bit faster on the database side.</p>\n<p><strong>$wpdb->prepare()</strong> &#8212; now almost all of the SQL in WordPress is prepared first, and the same functions are available to your plugins. This should prevent elementary SQL escaping issues.</p>\n<p><strong>Media buttons</strong> &#8212; the add media buttons above the post are both expandable, so you could have an &#8220;Add Google Map&#8221; button if you like, They can be overridden, so if you think you can do the video or audio tab better than we have you can replace the default.</p>\n<p><strong>Shortcode API</strong> &#8212; the new gallery functionality is powered by the new shortcode API. Shortcodes are little bracket-delineated strings that can be magically expanded at runtime to something more interesting. They give users a short, easy to type and copy/paste string they can move around their post without worrying about messing up complex HTML or embed codes. <a href=\"http://codex.wordpress.org/Shortcode_API\">The Shortcode API is fully documented</a>.</p>\n<p>Now you see why 2.5 took a little extra time. <img src=\'http://wordpress.org/development/wp-includes/images/smilies/icon_smile.gif\' alt=\':)\' class=\'wp-smiley\' /> </p>\n<h3>Upgrade Notes</h3>\n<p>2.5 does include security fixes so it is recommended for all users, the 2.3 branch will no longer be updated. <a href=\"http://codex.wordpress.org/Upgrading_WordPress\">The upgrade instructions for this version are pretty much the same as any other version</a>. The most important thing to check is your plugins, so if for example everything works except the new uploader, a legacy plugin might be causing a javascript error on the page and breaking it. If something goes wrong, the safest thing to do is turn your plugins off (we have a button to do them all at once, now) and turn them back on one-by-one, testing the problem along the way. This has solved almost everybody&#8217;s problems in testing, and it also lets you know which plugin author to show some love to so they&#8217;ll update their plugin, and which plugin authors already have so you can shower them with praises on your blog.</p>\n<p>One brief note about some of the new upload and plugin upgrade features, there are some edge-case hosting platforms, like versions of Lighttpd before 1.5 or over-agressive mod_security rules, which can break. If something isn&#8217;t working like it was looked in the screenshot, ask your host if there&#8217;s something on the server side which may be interfering. Hosts, feel free to join and post to <a href=\"http://lists.automattic.com/mailman/listinfo/wp-testers\">our wp-testers mailing list</a> if you have an environment that requires some extra code to work around. We&#8217;d be happy to include it in the next update.</p>\n<p><em>Quick tip:</em> in 2.5 you click the name of things to edit them, like your username to edit your profile or the title of a post to edit it.</p>\n<h3>The Community is Growing</h3>\n<p>More than growing, it&#8217;s on fire. We always talk about things like downloads, and the 2.3 branch has already had <strong>1.92 million downloads</strong> as I write this post, but this time we have some far more interesting information I&#8217;d like to share.</p>\n<p>There were over 1,200 commits to our repository since 2.3.0 and <strong>over 90 people were credited</strong> in them. This means in our core code, not plugins, there were at least 90 individual folks that contributed something high-quality enough that it made the cut to be part of the download you guys get today. I had no idea this group of people was so large.</p>\n<p>Outside of the <a href=\"http://wordpress.org/about/\">core commit team</a>, there was particular help from these people, in rough order of number of credits and tickets: <a href=\"http://blogwaffe.com/\">mdawaffe (Michael Adams)</a>, <a href=\"http://www.laptoptips.ca/\">azaozz (Andrew Ozz)</a>, <a href=\"http://nb.niichavo.org/\">nbachiyski (Nikolay Bachiyski)</a>, <a href=\"http://skeltoac.com/\">andy (Andy Skelton)</a>, <a href=\"http://iammattthomas.com/\">iammattthomas (Matt Thomas)</a>, <a href=\"http://thresholdstate.com/\">tellyworth (Alex Shiels)</a>, <a href=\"http://joseph.randomnetworks.com/\">josephscott (Joseph Scott)</a>, <a href=\"http://foolswisdom.com/\">lloydbudd (Lloyd Budd)</a>, <a href=\"http://dd32.id.au/\">DD32 (Dion)</a>, <a href=\"http://www.ilfilosofo.com/\">filosofo (Austin Matzko)</a>, <a href=\"http://engel.uk.to/\">hansengel (Hans Engel)</a>, <a href=\"http://www.freecharity.org.uk/\">pishmishy</a>, ffemtcj, <a href=\"http://www.viper007bond.com/\">Viper007Bond</a>, <a href=\"http://extralogical.net/\">ionfish (Benedict Eastaugh)</a>, <a href=\"http://www.poplarware.com/\">jhodgdon (Jennifer Hodgdon)</a>, <a href=\"http://ottodestruct.com/blog/\">Otto42</a>, <a href=\"http://www.cefm.ca/\">thee17 (Charles E. Free-Melvin)</a>, and <a href=\"http://www.buayacorp.com/\">xknown</a>. Also want to thank MichaelH and Lorelle on the documentation side, and moshu, Kafkaesqui, whooami, MichaelH, Otto42, and jeremyclark13 for helping with support.</p>\n<p>The 2.5 branch is nicknamed &#8220;Brecker&#8221; in honor of <a href=\"http://en.wikipedia.org/wiki/Michael_Brecker\">Michael Brecker</a>, an exceptionally talented saxophonist who could cross styles effortlessly and never stopped experimenting and pushing himself until he passed away last year.</p>\n<h3>New WordPress.org</h3>\n<p>All of this wasn&#8217;t enough, so in our copious spare time we decided to redesign WordPress.org to better match the aesthetics of the new dashboard and also to spruce up a few areas that needed lovin&#8217;. Some parts of the site, like the Codex, might show the old style for a day or two. We know, just give us a bit of time. Thanks to <a href=\"http://iammattthomas.com/\">Matt Thomas</a> for his epic effort in designing and coding the new site.</p>\n<h3>What&#8217;s Next</h3>\n<p>As always with WordPress, we don&#8217;t claim any of these features to be perfect, or to be better than everyone else in the world, but they are done by and for the people and the one thing we do promise is that with every release we listen and do our best to improve. </p>\n<p>2.5 is a major milestone for WordPress not because it added dozens of user-requested features, but because it reaffirms that we&#8217;re as passionate about blogging as the day we started. Our community is too fierce to rest on its laurels &#8212; contrary to what pundits claim, blogging is far from &#8220;finished&#8221; and every improvement just whets our appetite for more. And more is coming.</p>\n<p>It&#8217;s a good thing WordPress doesn&#8217;t limit the length of posts, because this one would have hit it. If you made it this far, thanks for sharing a bit of your day with us. I sincerely hope this new version of WordPress helps you do what you love to do.</p>\n\";}i:6;a:12:{s:5:\"title\";s:32:\"Screencast and WordPress 2.5 RC2\";s:4:\"link\";s:58:\"http://wordpress.org/development/2008/03/wordpress-25-rc2/\";s:8:\"comments\";s:67:\"http://wordpress.org/development/2008/03/wordpress-25-rc2/#comments\";s:7:\"pubdate\";s:31:\"Wed, 26 Mar 2008 11:36:25 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:8:\"category\";s:11:\"Development\";s:4:\"guid\";s:39:\"http://wordpress.org/development/?p=227\";s:11:\"description\";s:334:\"2.5 is coming along thanks to the fantastic feedback you guys provided on RC1 (over 580 pingbacks and counting), and we&#8217;re now ready to show you a bit more of a peek with a short screencast covering the new dashboard and uploader and Release Candidate 2. First here&#8217;s the screencast, which is also available embedded [...]\";s:7:\"content\";a:1:{s:7:\"encoded\";s:2991:\"<p>2.5 is coming along thanks to the <a href=\"http://wordpress.org/development/2008/03/25-sneak-peek/\">fantastic feedback you guys provided on RC1</a> (over 580 pingbacks and counting), and we&#8217;re now ready to show you a bit more of a peek with a short screencast covering the new dashboard and uploader and Release Candidate 2. First here&#8217;s the screencast, which is also available embedded below, <a href=\"http://s.wordpress.org/resources/2.5/dashboard-and-images.swf\">as a Flash movie</a>, or as a <a href=\"http://s.wordpress.org/resources/2.5/dashboard-and-images.avi\">17mb AVI download</a>:</p>\n<p>            <object id=\"csSWF\" classid=\"clsid:d27cdb6e-ae6d-11cf-96b8-444553540000\" width=\"640\" height=\"355\" codebase=\"http://active.macromedia.com/flash7/cabs/ swflash.cab#version=9,0,28,0\"><param name=\"src\" value=\"\"/><param name=\"bgcolor\" value=\"#1a1a1a\"/><param name=\"quality\" value=\"best\"/><param name=\"allowScriptAccess\" value=\"always\"/><param name=\"allowFullScreen\" value=\"true\"/><param name=\"scale\" value=\"showall\"/><param name=\"flashVars\" value=\"autostart=false\"/><embed name=\"csSWF\" src=\"http://s.wordpress.org/resources/2.5/dashboard-and-images.swf\" width=\"549\" height=\"304\" bgcolor=\"#1a1a1a\" quality=\"best\" allowScriptAccess=\"always\" allowFullScreen=\"true\" scale=\"showall\" flashVars=\"autostart=false\" pluginspage=\"http://www.macromedia.com/shockwave/download/index.cgi?P1_Prod_Version=ShockwaveFlash\"></embed></object></p>\n<p>I&#8217;ve <a href=\"http://ma.tt/category/gallery/\">uploaded more than a thousand photos already</a> into the new gallery system &#8212; it works.</p>\n<p>(This was my first screencast, but I hope we can have more on WordPress.org and our documentation in the future.)</p>\n<p>If you make frequent backups and you’re interested in helping us out with development by testing the very latest, <a href=\"http://wordpress.org/wordpress-2.5-RC2.zip\">download</a> and install <a href=\"http://wordpress.org/wordpress-2.5-RC2.zip\">Release Candidate 2</a> of WordPress 2.5, and <a href=\"http://lists.automattic.com/mailman/listinfo/wp-testers\">join our testers mailing list</a> to report any bugs you find in the code.</p>\n<p>Finally with regards to theme and plugin compatibility, we&#8217;ve had no reports of any broken themes in this upgrade, which makes sense because we didn&#8217;t really change anything core about themes, just added new optional capabilities like Gravatars. Plugins that work with the admin may require updating to take advantage of the new, cleaner UI in WordPress 2.5.</p>\n<p>The community has started to <a href=\"http://codex.wordpress.org/Plugins/Plugin_Compatibility/2.5\">keep a list here of which plugins work great and which don&#8217;t</a>. It&#8217;s worth looking at, or even better just deactivate your plugins before upgrading for 2.5 and let the built-in updater notify and give you one-click upgrades to plugins you have installed, assuming the developer is cool and has updated their code for 2.5 already.</p>\n\";}s:3:\"wfw\";a:1:{s:10:\"commentrss\";s:63:\"http://wordpress.org/development/2008/03/wordpress-25-rc2/feed/\";}s:7:\"summary\";s:334:\"2.5 is coming along thanks to the fantastic feedback you guys provided on RC1 (over 580 pingbacks and counting), and we&#8217;re now ready to show you a bit more of a peek with a short screencast covering the new dashboard and uploader and Release Candidate 2. First here&#8217;s the screencast, which is also available embedded [...]\";s:12:\"atom_content\";s:2991:\"<p>2.5 is coming along thanks to the <a href=\"http://wordpress.org/development/2008/03/25-sneak-peek/\">fantastic feedback you guys provided on RC1</a> (over 580 pingbacks and counting), and we&#8217;re now ready to show you a bit more of a peek with a short screencast covering the new dashboard and uploader and Release Candidate 2. First here&#8217;s the screencast, which is also available embedded below, <a href=\"http://s.wordpress.org/resources/2.5/dashboard-and-images.swf\">as a Flash movie</a>, or as a <a href=\"http://s.wordpress.org/resources/2.5/dashboard-and-images.avi\">17mb AVI download</a>:</p>\n<p>            <object id=\"csSWF\" classid=\"clsid:d27cdb6e-ae6d-11cf-96b8-444553540000\" width=\"640\" height=\"355\" codebase=\"http://active.macromedia.com/flash7/cabs/ swflash.cab#version=9,0,28,0\"><param name=\"src\" value=\"\"/><param name=\"bgcolor\" value=\"#1a1a1a\"/><param name=\"quality\" value=\"best\"/><param name=\"allowScriptAccess\" value=\"always\"/><param name=\"allowFullScreen\" value=\"true\"/><param name=\"scale\" value=\"showall\"/><param name=\"flashVars\" value=\"autostart=false\"/><embed name=\"csSWF\" src=\"http://s.wordpress.org/resources/2.5/dashboard-and-images.swf\" width=\"549\" height=\"304\" bgcolor=\"#1a1a1a\" quality=\"best\" allowScriptAccess=\"always\" allowFullScreen=\"true\" scale=\"showall\" flashVars=\"autostart=false\" pluginspage=\"http://www.macromedia.com/shockwave/download/index.cgi?P1_Prod_Version=ShockwaveFlash\"></embed></object></p>\n<p>I&#8217;ve <a href=\"http://ma.tt/category/gallery/\">uploaded more than a thousand photos already</a> into the new gallery system &#8212; it works.</p>\n<p>(This was my first screencast, but I hope we can have more on WordPress.org and our documentation in the future.)</p>\n<p>If you make frequent backups and you’re interested in helping us out with development by testing the very latest, <a href=\"http://wordpress.org/wordpress-2.5-RC2.zip\">download</a> and install <a href=\"http://wordpress.org/wordpress-2.5-RC2.zip\">Release Candidate 2</a> of WordPress 2.5, and <a href=\"http://lists.automattic.com/mailman/listinfo/wp-testers\">join our testers mailing list</a> to report any bugs you find in the code.</p>\n<p>Finally with regards to theme and plugin compatibility, we&#8217;ve had no reports of any broken themes in this upgrade, which makes sense because we didn&#8217;t really change anything core about themes, just added new optional capabilities like Gravatars. Plugins that work with the admin may require updating to take advantage of the new, cleaner UI in WordPress 2.5.</p>\n<p>The community has started to <a href=\"http://codex.wordpress.org/Plugins/Plugin_Compatibility/2.5\">keep a list here of which plugins work great and which don&#8217;t</a>. It&#8217;s worth looking at, or even better just deactivate your plugins before upgrading for 2.5 and let the built-in updater notify and give you one-click upgrades to plugins you have installed, assuming the developer is cool and has updated their code for 2.5 already.</p>\n\";}i:7;a:12:{s:5:\"title\";s:14:\"2.5 Sneak Peek\";s:4:\"link\";s:55:\"http://wordpress.org/development/2008/03/25-sneak-peek/\";s:8:\"comments\";s:64:\"http://wordpress.org/development/2008/03/25-sneak-peek/#comments\";s:7:\"pubdate\";s:31:\"Tue, 18 Mar 2008 07:08:57 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:8:\"category\";s:11:\"Development\";s:4:\"guid\";s:39:\"http://wordpress.org/development/?p=226\";s:11:\"description\";s:360:\"A customizable dashboard, multi-file upload, built-in galleries, one-click plugin upgrades, tag management, built-in Gravatars, full text feeds, and faster load times sound interesting? Then WordPress 2.5 might be the release for you. It&#8217;s been in the oven for a while, and we&#8217;re finally ready to open the doors a bit to give you a taste.\nFor [...]\";s:7:\"content\";a:1:{s:7:\"encoded\";s:5448:\"<p>A customizable dashboard, multi-file upload, built-in galleries, one-click plugin upgrades, tag management, built-in Gravatars, full text feeds, and faster load times sound interesting? Then WordPress 2.5 might be the release for you. It&#8217;s been in the oven for a while, and we&#8217;re finally ready to open the doors a bit to give you a taste.</p>\n<p>For the past few months, we’ve been working with our friends at Happy Cog &#8212; <a href=\"http://zeldman.com/\">Jeffrey Zeldman</a>, <a href=\"http://jasonsantamaria.com/\">Jason Santa Maria</a>, and <a href=\"http://bobulate.com/\">Liz Danzico</a> &#8212; to redesign WordPress from the ground-up. The result is a new way of interacting with WordPress that will remain familiar to seasoned users while improving the experience for everyone. This isn’t just a fresh coat of paint &#8212; we’ve re-thought the look of WordPress, as well as how it’s organized so that you can forget about the software and focus on your own creative pursuits.</p>\n<p>Here are a few vignettes of what&#8217;s in store.</p>\n<h3>The Dashboard</h3>\n<p class=\"tutscreen\"><img src=\"http://wpcom.wordpress.com/files/2008/03/dashboard-wide.png\" alt=\"dashboard-wide.png\" /></p>\n<p>The Dashboard’s most important role is to inform quickly and get you to where you’re headed in the admin. In interviewing users, we found that most of you ignore the Dashboard entirely &#8212; its useful information being mostly hidden in an overly complex design. The new Dashboard is focused on the most relevant tasks at hand: a quick summary of what’s published and scheduled for publication, the latest comments and incoming links, blog stats, and WordPress updates and news. You can add your own RSS feeds and edit the way information is presented so that the new Dashboard conforms to the way you use WordPress.</p>\n<h3>Navigation</h3>\n<p class=\"tutscreen\"><img src=\"http://wpcom.wordpress.com/files/2008/03/nav-wide.png\" alt=\"nav-wide.png\" /></p>\n<p>The WordPress navigation has confounded even sophisticated users. With the new design, we’ve cut the number of navigation options in half, separating the primary functions (writing, managing posts and pages, editing the blog’s design, and managing comments) from secondary functions. This presents information at a more comfortable pace, revealing only the information that’s necessary. Everything you need is still there &#8212; just better organized. (Especially for people new to WP.)</p>\n<h3>Write</h3>\n<p class=\"tutscreen\"><img src=\"http://wpcom.wordpress.com/files/2008/03/write-wide.png\" alt=\"write-wide.png\" /></p>\n<p><img class=\"right\" src=\"http://wpcom.wordpress.com/files/2008/03/write3.png\" alt=\"write2.png\" /></p>\n<p>By far, the most frequently accessed part of WordPress is the Write screen. It gets the job done, but its myriad options can be overwhelming. The new write screen only displays the information that you’ll use most often. It displays the most common fields in a way that makes posting incredibly easy. Additional options are hidden away until you need them. The new Write screen anticipates the natural flow of the way you write, and is smart enough to remember the way you left it so that your preferred writing environment is always quickly available. The new visual editor even has a handy full-screen mode to help block out distractions while composing your newest post. (My personal favorite new feature.)</p>\n<h3>Manage</h3>\n<p class=\"tutscreen\"><img src=\"http://wpcom.wordpress.com/files/2008/03/manage-wide.png\" alt=\"\" /></p>\n<p><img class=\"right\" src=\"http://wpcom.wordpress.com/files/2008/03/manage.png\" alt=\"\" /></p>\n<p>The Manage screens have been redesigned and unified so that now, managing your pages, posts, media, and comments all use similar, consistent interfaces. We’ve omitted superfluous information and made what’s important faster to find. We believe these changes will make you a faster, more proficient blogger.</p>\n<p>You might also notice there are some new colors, the dashboard feels much fresher and lighter. If you&#8217;re jonesing for the old look under your user options you can now select the &#8220;classic&#8221; colors and get those old blues back. (It&#8217;s also pluggable so people can easily add or share their own color schemes.)</p>\n<p>If you make frequent backups and you&#8217;re interested in helping us out with development by testing the new code, <a href=\"http://wordpress.org/wordpress-2.5-RC1.zip\">download</a> and install <a href=\"http://wordpress.org/wordpress-2.5-RC1.zip\">Release Candidate 1</a> of WordPress 2.5, and <a href=\"http://lists.automattic.com/mailman/listinfo/wp-testers\">join our testers mailing list</a> to report any bugs you find in the code.</p>\n<p>We&#8217;re also interested in feedback on the new interface and would love to hear your opinions, thoughts, rants, raves, and anything in between. We created a special email address just for the occasion: <a href=\"mailto:2.5-feedback@wordpress.org\">2.5-feedback@wordpress.org</a>.</p>\n<p>The software is basically done and stable, and could be released today, but we&#8217;d like to incorporate feedback from a wider audience before making it available to the general public. After a few days of your feedback we&#8217;ll set a final release date. Personally, I can&#8217;t wait. <img src=\'http://wordpress.org/development/wp-includes/images/smilies/icon_smile.gif\' alt=\':)\' class=\'wp-smiley\' /></p>\n\";}s:3:\"wfw\";a:1:{s:10:\"commentrss\";s:60:\"http://wordpress.org/development/2008/03/25-sneak-peek/feed/\";}s:7:\"summary\";s:360:\"A customizable dashboard, multi-file upload, built-in galleries, one-click plugin upgrades, tag management, built-in Gravatars, full text feeds, and faster load times sound interesting? Then WordPress 2.5 might be the release for you. It&#8217;s been in the oven for a while, and we&#8217;re finally ready to open the doors a bit to give you a taste.\nFor [...]\";s:12:\"atom_content\";s:5448:\"<p>A customizable dashboard, multi-file upload, built-in galleries, one-click plugin upgrades, tag management, built-in Gravatars, full text feeds, and faster load times sound interesting? Then WordPress 2.5 might be the release for you. It&#8217;s been in the oven for a while, and we&#8217;re finally ready to open the doors a bit to give you a taste.</p>\n<p>For the past few months, we’ve been working with our friends at Happy Cog &#8212; <a href=\"http://zeldman.com/\">Jeffrey Zeldman</a>, <a href=\"http://jasonsantamaria.com/\">Jason Santa Maria</a>, and <a href=\"http://bobulate.com/\">Liz Danzico</a> &#8212; to redesign WordPress from the ground-up. The result is a new way of interacting with WordPress that will remain familiar to seasoned users while improving the experience for everyone. This isn’t just a fresh coat of paint &#8212; we’ve re-thought the look of WordPress, as well as how it’s organized so that you can forget about the software and focus on your own creative pursuits.</p>\n<p>Here are a few vignettes of what&#8217;s in store.</p>\n<h3>The Dashboard</h3>\n<p class=\"tutscreen\"><img src=\"http://wpcom.wordpress.com/files/2008/03/dashboard-wide.png\" alt=\"dashboard-wide.png\" /></p>\n<p>The Dashboard’s most important role is to inform quickly and get you to where you’re headed in the admin. In interviewing users, we found that most of you ignore the Dashboard entirely &#8212; its useful information being mostly hidden in an overly complex design. The new Dashboard is focused on the most relevant tasks at hand: a quick summary of what’s published and scheduled for publication, the latest comments and incoming links, blog stats, and WordPress updates and news. You can add your own RSS feeds and edit the way information is presented so that the new Dashboard conforms to the way you use WordPress.</p>\n<h3>Navigation</h3>\n<p class=\"tutscreen\"><img src=\"http://wpcom.wordpress.com/files/2008/03/nav-wide.png\" alt=\"nav-wide.png\" /></p>\n<p>The WordPress navigation has confounded even sophisticated users. With the new design, we’ve cut the number of navigation options in half, separating the primary functions (writing, managing posts and pages, editing the blog’s design, and managing comments) from secondary functions. This presents information at a more comfortable pace, revealing only the information that’s necessary. Everything you need is still there &#8212; just better organized. (Especially for people new to WP.)</p>\n<h3>Write</h3>\n<p class=\"tutscreen\"><img src=\"http://wpcom.wordpress.com/files/2008/03/write-wide.png\" alt=\"write-wide.png\" /></p>\n<p><img class=\"right\" src=\"http://wpcom.wordpress.com/files/2008/03/write3.png\" alt=\"write2.png\" /></p>\n<p>By far, the most frequently accessed part of WordPress is the Write screen. It gets the job done, but its myriad options can be overwhelming. The new write screen only displays the information that you’ll use most often. It displays the most common fields in a way that makes posting incredibly easy. Additional options are hidden away until you need them. The new Write screen anticipates the natural flow of the way you write, and is smart enough to remember the way you left it so that your preferred writing environment is always quickly available. The new visual editor even has a handy full-screen mode to help block out distractions while composing your newest post. (My personal favorite new feature.)</p>\n<h3>Manage</h3>\n<p class=\"tutscreen\"><img src=\"http://wpcom.wordpress.com/files/2008/03/manage-wide.png\" alt=\"\" /></p>\n<p><img class=\"right\" src=\"http://wpcom.wordpress.com/files/2008/03/manage.png\" alt=\"\" /></p>\n<p>The Manage screens have been redesigned and unified so that now, managing your pages, posts, media, and comments all use similar, consistent interfaces. We’ve omitted superfluous information and made what’s important faster to find. We believe these changes will make you a faster, more proficient blogger.</p>\n<p>You might also notice there are some new colors, the dashboard feels much fresher and lighter. If you&#8217;re jonesing for the old look under your user options you can now select the &#8220;classic&#8221; colors and get those old blues back. (It&#8217;s also pluggable so people can easily add or share their own color schemes.)</p>\n<p>If you make frequent backups and you&#8217;re interested in helping us out with development by testing the new code, <a href=\"http://wordpress.org/wordpress-2.5-RC1.zip\">download</a> and install <a href=\"http://wordpress.org/wordpress-2.5-RC1.zip\">Release Candidate 1</a> of WordPress 2.5, and <a href=\"http://lists.automattic.com/mailman/listinfo/wp-testers\">join our testers mailing list</a> to report any bugs you find in the code.</p>\n<p>We&#8217;re also interested in feedback on the new interface and would love to hear your opinions, thoughts, rants, raves, and anything in between. We created a special email address just for the occasion: <a href=\"mailto:2.5-feedback@wordpress.org\">2.5-feedback@wordpress.org</a>.</p>\n<p>The software is basically done and stable, and could be released today, but we&#8217;d like to incorporate feedback from a wider audience before making it available to the general public. After a few days of your feedback we&#8217;ll set a final release date. Personally, I can&#8217;t wait. <img src=\'http://wordpress.org/development/wp-includes/images/smilies/icon_smile.gif\' alt=\':)\' class=\'wp-smiley\' /></p>\n\";}i:8;a:12:{s:5:\"title\";s:15:\"WordPress 2.3.3\";s:4:\"link\";s:55:\"http://wordpress.org/development/2008/02/wordpress-233/\";s:8:\"comments\";s:64:\"http://wordpress.org/development/2008/02/wordpress-233/#comments\";s:7:\"pubdate\";s:31:\"Tue, 05 Feb 2008 05:02:45 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Ryan\";}s:8:\"category\";s:11:\"Development\";s:4:\"guid\";s:39:\"http://wordpress.org/development/?p=225\";s:11:\"description\";s:307:\"WordPress 2.3.3 is an urgent security release. If you have registration enabled a flaw was found in the XML-RPC implementation such that a specially crafted request would allow a user to edit posts of other users on that blog.  In addition to fixing this security flaw,  2.3.3 fixes a few minor bugs.  [...]\";s:7:\"content\";a:1:{s:7:\"encoded\";s:1208:\"<p>WordPress 2.3.3 is an urgent security release. If you have registration enabled a flaw was found in the XML-RPC implementation such that a specially crafted request would allow a user to edit posts of other users on that blog.  In addition to fixing this security flaw,  2.3.3 fixes <a href=\"http://trac.wordpress.org/query?status=closed&amp;milestone=2.3.3\">a few minor bugs</a>.  If you are interested only in the security fix, download the <a href=\"http://trac.wordpress.org/browser/tags/2.3.3/xmlrpc.php?format=raw\">fixed version of <code>xmlrpc.php</code></a> and copy it over your existing <code>xmlrpc.php</code>.  Otherwise, you can get the entire release <a href=\"http://wordpress.org/download/\">here</a>.</p>\n<p>Also, there is <a href=\"http://weblogtoolscollection.com/archives/2008/01/21/wp-forum-plugin-security-bulletin/\">a vulnerability in the WP-Forum plugin</a> that is being actively exploited right now.  If you are using this plugin, please remove it until an update is available from its author.</p>\n<p>Since we are talking security, remember to use strong passwords and change them regularly.  While you&#8217;re updating WP and your plugins, consider refreshing your passwords.</p>\n\";}s:3:\"wfw\";a:1:{s:10:\"commentrss\";s:60:\"http://wordpress.org/development/2008/02/wordpress-233/feed/\";}s:7:\"summary\";s:307:\"WordPress 2.3.3 is an urgent security release. If you have registration enabled a flaw was found in the XML-RPC implementation such that a specially crafted request would allow a user to edit posts of other users on that blog.  In addition to fixing this security flaw,  2.3.3 fixes a few minor bugs.  [...]\";s:12:\"atom_content\";s:1208:\"<p>WordPress 2.3.3 is an urgent security release. If you have registration enabled a flaw was found in the XML-RPC implementation such that a specially crafted request would allow a user to edit posts of other users on that blog.  In addition to fixing this security flaw,  2.3.3 fixes <a href=\"http://trac.wordpress.org/query?status=closed&amp;milestone=2.3.3\">a few minor bugs</a>.  If you are interested only in the security fix, download the <a href=\"http://trac.wordpress.org/browser/tags/2.3.3/xmlrpc.php?format=raw\">fixed version of <code>xmlrpc.php</code></a> and copy it over your existing <code>xmlrpc.php</code>.  Otherwise, you can get the entire release <a href=\"http://wordpress.org/download/\">here</a>.</p>\n<p>Also, there is <a href=\"http://weblogtoolscollection.com/archives/2008/01/21/wp-forum-plugin-security-bulletin/\">a vulnerability in the WP-Forum plugin</a> that is being actively exploited right now.  If you are using this plugin, please remove it until an update is available from its author.</p>\n<p>Since we are talking security, remember to use strong passwords and change them regularly.  While you&#8217;re updating WP and your plugins, consider refreshing your passwords.</p>\n\";}i:9;a:12:{s:5:\"title\";s:15:\"WordPress 2.3.2\";s:4:\"link\";s:55:\"http://wordpress.org/development/2007/12/wordpress-232/\";s:8:\"comments\";s:64:\"http://wordpress.org/development/2007/12/wordpress-232/#comments\";s:7:\"pubdate\";s:31:\"Sat, 29 Dec 2007 22:44:09 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Ryan\";}s:8:\"category\";s:8:\"Releases\";s:4:\"guid\";s:55:\"http://wordpress.org/development/2007/12/wordpress-232/\";s:11:\"description\";s:325:\"WordPress 2.3.2 is an urgent security release that fixes a bug that can be used to expose your draft posts.  2.3.2 also suppresses some error messages that can give away information about your database table structure and limits and stops some information leaks in the XML-RPC and APP implementations.  Get 2.3.2 now to [...]\";s:7:\"content\";a:1:{s:7:\"encoded\";s:1243:\"<p>WordPress 2.3.2 is an urgent security release that fixes a bug that can be used to <a href=\"http://trac.wordpress.org/ticket/5487\">expose your draft posts</a>.  2.3.2 also <a href=\"http://trac.wordpress.org/ticket/5473\">suppresses some error messages</a> that can give away information about your database table structure and limits and stops some information leaks in the XML-RPC and APP implementations.  <a href=\"http://wordpress.org/download/\">Get 2.3.2 now</a> to protect your blog from these disclosures.</p>\n<p>As a little bonus,  2.3.2 allows you to define a custom DB error page. Place your custom template at wp-content/db-error.php.   If WP has a problem connecting to your database, this page will displayed rather than the default error message.</p>\n<p>For more detail on what&#8217;s new in 2.3.2, view the list of <a href=\"http://trac.wordpress.org/query?status=closed&amp;milestone=2.3.2&amp;resolution=fixed&amp;order=priority\">fixed bugs</a> and see the <a href=\"http://trac.wordpress.org/changeset?old_path=tags%2F2.3.1&amp;old=6528&amp;new_path=tags%2F2.3.2&amp;new=6528\">changes</a> between 2.3.1 and 2.3.2.</p>\n<p>Special thanks to <a href=\"http://www.buayacorp.com/\">Alex Concha</a> for his help on this release.</p>\n\";}s:3:\"wfw\";a:1:{s:10:\"commentrss\";s:60:\"http://wordpress.org/development/2007/12/wordpress-232/feed/\";}s:7:\"summary\";s:325:\"WordPress 2.3.2 is an urgent security release that fixes a bug that can be used to expose your draft posts.  2.3.2 also suppresses some error messages that can give away information about your database table structure and limits and stops some information leaks in the XML-RPC and APP implementations.  Get 2.3.2 now to [...]\";s:12:\"atom_content\";s:1243:\"<p>WordPress 2.3.2 is an urgent security release that fixes a bug that can be used to <a href=\"http://trac.wordpress.org/ticket/5487\">expose your draft posts</a>.  2.3.2 also <a href=\"http://trac.wordpress.org/ticket/5473\">suppresses some error messages</a> that can give away information about your database table structure and limits and stops some information leaks in the XML-RPC and APP implementations.  <a href=\"http://wordpress.org/download/\">Get 2.3.2 now</a> to protect your blog from these disclosures.</p>\n<p>As a little bonus,  2.3.2 allows you to define a custom DB error page. Place your custom template at wp-content/db-error.php.   If WP has a problem connecting to your database, this page will displayed rather than the default error message.</p>\n<p>For more detail on what&#8217;s new in 2.3.2, view the list of <a href=\"http://trac.wordpress.org/query?status=closed&amp;milestone=2.3.2&amp;resolution=fixed&amp;order=priority\">fixed bugs</a> and see the <a href=\"http://trac.wordpress.org/changeset?old_path=tags%2F2.3.1&amp;old=6528&amp;new_path=tags%2F2.3.2&amp;new=6528\">changes</a> between 2.3.1 and 2.3.2.</p>\n<p>Special thanks to <a href=\"http://www.buayacorp.com/\">Alex Concha</a> for his help on this release.</p>\n\";}}s:7:\"channel\";a:7:{s:5:\"title\";s:26:\"WordPress Development Blog\";s:4:\"link\";s:32:\"http://wordpress.org/development\";s:11:\"description\";s:33:\"WordPress development and updates\";s:7:\"pubdate\";s:31:\"Sun, 25 May 2008 17:27:47 +0000\";s:9:\"generator\";s:33:\"http://wordpress.org/?v=2.6-beta2\";s:8:\"language\";s:2:\"en\";s:7:\"tagline\";s:33:\"WordPress development and updates\";}s:9:\"textinput\";a:0:{}s:5:\"image\";a:0:{}s:9:\"feed_type\";s:3:\"RSS\";s:12:\"feed_version\";s:3:\"2.0\";s:5:\"stack\";a:0:{}s:9:\"inchannel\";b:0;s:6:\"initem\";b:0;s:9:\"incontent\";b:0;s:11:\"intextinput\";b:0;s:7:\"inimage\";b:0;s:13:\"current_field\";s:0:\"\";s:17:\"current_namespace\";b:0;s:19:\"_CONTENT_CONSTRUCTS\";a:6:{i:0;s:7:\"content\";i:1;s:7:\"summary\";i:2;s:4:\"info\";i:3;s:5:\"title\";i:4;s:7:\"tagline\";i:5;s:9:\"copyright\";}s:13:\"last_modified\";s:31:\"Sun, 25 May 2008 17:27:47 GMT\r\n\";s:4:\"etag\";s:36:\"\"bb875376d259c4b324d031ca47a27e5e\"\r\n\";}','no'),(82,0,'rss_0ff4b43bd116a9d8720d689c80e7dfd4_ts','1214839989','no'),(83,0,'rss_a5420c83891a9c88ad2a4f04584a5efc','O:9:\"MagpieRSS\":19:{s:6:\"parser\";i:0;s:12:\"current_item\";a:0:{}s:5:\"items\";a:15:{i:0;a:7:{s:5:\"title\";s:33:\"uberdose on \"All in One SEO Pack\"\";s:4:\"link\";s:65:\"http://wordpress.org/extend/plugins/all-in-one-seo-pack/#post-753\";s:7:\"pubdate\";s:31:\"Fri, 30 Mar 2007 20:08:18 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:8:\"uberdose\";}s:4:\"guid\";s:40:\"753@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:92:\"Automatically optimizes your Wordpress blog for Search Engines (Search Engine Optimization).\";s:7:\"summary\";s:92:\"Automatically optimizes your Wordpress blog for Search Engines (Search Engine Optimization).\";}i:1;a:7:{s:5:\"title\";s:17:\"matt on \"Akismet\"\";s:4:\"link\";s:52:\"http://wordpress.org/extend/plugins/akismet/#post-15\";s:7:\"pubdate\";s:31:\"Fri, 09 Mar 2007 22:11:30 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"matt\";}s:4:\"guid\";s:39:\"15@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:98:\"Akismet checks your comments against the Akismet web service to see if they look like spam or not.\";s:7:\"summary\";s:98:\"Akismet checks your comments against the Akismet web service to see if they look like spam or not.\";}i:2;a:7:{s:5:\"title\";s:30:\"arnee on \"Google XML Sitemaps\"\";s:4:\"link\";s:70:\"http://wordpress.org/extend/plugins/google-sitemap-generator/#post-132\";s:7:\"pubdate\";s:31:\"Fri, 09 Mar 2007 22:31:32 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:5:\"arnee\";}s:4:\"guid\";s:40:\"132@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:87:\"This plugin will create a Google sitemaps compliant XML-Sitemap of your WordPress blog.\";s:7:\"summary\";s:87:\"This plugin will create a Google sitemaps compliant XML-Sitemap of your WordPress blog.\";}i:3;a:7:{s:5:\"title\";s:29:\"andy on \"WordPress.com Stats\"\";s:4:\"link\";s:52:\"http://wordpress.org/extend/plugins/stats/#post-1355\";s:7:\"pubdate\";s:31:\"Sun, 06 May 2007 02:15:03 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"andy\";}s:4:\"guid\";s:41:\"1355@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:119:\"You can have simple, concise stats with no additional load on your server by plugging into WordPress.com\'s stat system.\";s:7:\"summary\";s:119:\"You can have simple, concise stats with no additional load on your server by plugging into WordPress.com\'s stat system.\";}i:4;a:7:{s:5:\"title\";s:36:\"olivers on \"cformsII - contact form\"\";s:4:\"link\";s:52:\"http://wordpress.org/extend/plugins/cforms/#post-925\";s:7:\"pubdate\";s:31:\"Mon, 09 Apr 2007 22:12:26 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:7:\"olivers\";}s:4:\"guid\";s:40:\"925@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:119:\"cforms II is the most customizable, flexible &#038; powerful ajax supporting contact form plugin (&#038; comment form)!\";s:7:\"summary\";s:119:\"cforms II is the most customizable, flexible &#038; powerful ajax supporting contact form plugin (&#038; comment form)!\";}i:5;a:7:{s:5:\"title\";s:29:\"alexrabe on \"NextGEN Gallery\"\";s:4:\"link\";s:62:\"http://wordpress.org/extend/plugins/nextgen-gallery/#post-1169\";s:7:\"pubdate\";s:31:\"Mon, 23 Apr 2007 20:08:06 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:8:\"alexrabe\";}s:4:\"guid\";s:41:\"1169@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:102:\"NextGEN Gallery is a full integrated Image Gallery plugin for WordPress with a Flash slideshow option.\";s:7:\"summary\";s:102:\"NextGEN Gallery is a full integrated Image Gallery plugin for WordPress with a Flash slideshow option.\";}i:6;a:7:{s:5:\"title\";s:27:\"danielelippi on \"StatPress\"\";s:4:\"link\";s:56:\"http://wordpress.org/extend/plugins/statpress/#post-2383\";s:7:\"pubdate\";s:31:\"Tue, 25 Sep 2007 16:00:08 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:12:\"danielelippi\";}s:4:\"guid\";s:41:\"2383@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:150:\"The real-time plugin dedicated to the management of statistics about blog visits. It collects information about visitors, spiders, search keywords, fe\";s:7:\"summary\";s:150:\"The real-time plugin dedicated to the management of statistics about blog visits. It collects information about visitors, spiders, search keywords, fe\";}i:7;a:7:{s:5:\"title\";s:29:\"snumb130 on \"Events Calendar\"\";s:4:\"link\";s:62:\"http://wordpress.org/extend/plugins/events-calendar/#post-2523\";s:7:\"pubdate\";s:31:\"Fri, 19 Oct 2007 04:10:03 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:8:\"snumb130\";}s:4:\"guid\";s:41:\"2523@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:149:\"Events-Calendar is a diverse replacement for the original calendar included with WordPress adding many useful functions to keep track of your events.\";s:7:\"summary\";s:149:\"Events-Calendar is a diverse replacement for the original calendar included with WordPress adding many useful functions to keep track of your events.\";}i:8;a:7:{s:5:\"title\";s:20:\"GamerZ on \"WP-Polls\"\";s:4:\"link\";s:54:\"http://wordpress.org/extend/plugins/wp-polls/#post-368\";s:7:\"pubdate\";s:31:\"Fri, 09 Mar 2007 23:18:28 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:6:\"GamerZ\";}s:4:\"guid\";s:40:\"368@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:117:\"Adds an AJAX poll system to your WordPress blog. You can also easily add a poll into your WordPress\'s blog post/page.\";s:7:\"summary\";s:117:\"Adds an AJAX poll system to your WordPress blog. You can also easily add a poll into your WordPress\'s blog post/page.\";}i:9;a:7:{s:5:\"title\";s:27:\"donncha on \"WP Super Cache\"\";s:4:\"link\";s:61:\"http://wordpress.org/extend/plugins/wp-super-cache/#post-2572\";s:7:\"pubdate\";s:31:\"Mon, 05 Nov 2007 11:40:04 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:7:\"donncha\";}s:4:\"guid\";s:41:\"2572@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:73:\"A very fast caching engine for WordPress that produces static html files.\";s:7:\"summary\";s:73:\"A very fast caching engine for WordPress that produces static html files.\";}i:10;a:7:{s:5:\"title\";s:20:\"seek3r on \"podPress\"\";s:4:\"link\";s:54:\"http://wordpress.org/extend/plugins/podpress/#post-566\";s:7:\"pubdate\";s:31:\"Tue, 20 Mar 2007 17:09:06 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:6:\"seek3r\";}s:4:\"guid\";s:40:\"566@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:47:\"A dream plugin for Podcasters using WordPress..\";s:7:\"summary\";s:47:\"A dream plugin for Podcasters using WordPress..\";}i:11;a:7:{s:5:\"title\";s:33:\"dancameron on \"Search Everything\"\";s:4:\"link\";s:63:\"http://wordpress.org/extend/plugins/search-everything/#post-269\";s:7:\"pubdate\";s:31:\"Fri, 09 Mar 2007 23:05:08 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:10:\"dancameron\";}s:4:\"guid\";s:40:\"269@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:150:\"Increases Wordpress\' default search functionality through an options panel. Options include searching pages, excerpts, attachments, drafts, comments a\";s:7:\"summary\";s:150:\"Increases Wordpress\' default search functionality through an options panel. Options include searching pages, excerpts, attachments, drafts, comments a\";}i:12;a:7:{s:5:\"title\";s:23:\"GamerZ on \"WP-PageNavi\"\";s:4:\"link\";s:57:\"http://wordpress.org/extend/plugins/wp-pagenavi/#post-363\";s:7:\"pubdate\";s:31:\"Fri, 09 Mar 2007 23:17:57 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:6:\"GamerZ\";}s:4:\"guid\";s:40:\"363@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:59:\"Adds a more advanced paging navigation your WordPress blog.\";s:7:\"summary\";s:59:\"Adds a more advanced paging navigation your WordPress blog.\";}i:13;a:7:{s:5:\"title\";s:33:\"RobMarsh on \"Post-Plugin Library\"\";s:4:\"link\";s:66:\"http://wordpress.org/extend/plugins/post-plugin-library/#post-4893\";s:7:\"pubdate\";s:31:\"Wed, 19 Mar 2008 10:33:16 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:8:\"RobMarsh\";}s:4:\"guid\";s:41:\"4893@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:90:\"The shared code library for Similar posts, Recent Posts, Random Posts and Recent Comments.\";s:7:\"summary\";s:90:\"The shared code library for Similar posts, Recent Posts, Random Posts and Recent Comments.\";}i:14;a:7:{s:5:\"title\";s:38:\"hallsofmontezuma on \"WP Security Scan\"\";s:4:\"link\";s:63:\"http://wordpress.org/extend/plugins/wp-security-scan/#post-4986\";s:7:\"pubdate\";s:31:\"Sat, 29 Mar 2008 20:05:44 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:16:\"hallsofmontezuma\";}s:4:\"guid\";s:41:\"4986@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:63:\"Scans your WordPress installation for security vulnerabilities.\";s:7:\"summary\";s:63:\"Scans your WordPress installation for security vulnerabilities.\";}}s:7:\"channel\";a:6:{s:5:\"title\";s:12:\"Most Popular\";s:4:\"link\";s:36:\"http://wordpress.org/extend/plugins/\";s:11:\"description\";s:12:\"Most Popular\";s:8:\"language\";s:2:\"en\";s:7:\"pubdate\";s:31:\"Mon, 30 Jun 2008 15:33:12 +0000\";s:7:\"tagline\";s:12:\"Most Popular\";}s:9:\"textinput\";a:0:{}s:5:\"image\";a:0:{}s:9:\"feed_type\";s:3:\"RSS\";s:12:\"feed_version\";s:3:\"2.0\";s:5:\"stack\";a:0:{}s:9:\"inchannel\";b:0;s:6:\"initem\";b:0;s:9:\"incontent\";b:0;s:11:\"intextinput\";b:0;s:7:\"inimage\";b:0;s:13:\"current_field\";s:0:\"\";s:17:\"current_namespace\";b:0;s:19:\"_CONTENT_CONSTRUCTS\";a:6:{i:0;s:7:\"content\";i:1;s:7:\"summary\";i:2;s:4:\"info\";i:3;s:5:\"title\";i:4;s:7:\"tagline\";i:5;s:9:\"copyright\";}s:13:\"last_modified\";s:21:\"2007-03-30 20:08:18\r\n\";s:4:\"etag\";s:36:\"\"a8894f3af502f182fd4e92b6405bade8\"\r\n\";}','no'),(84,0,'rss_a5420c83891a9c88ad2a4f04584a5efc_ts','1214839989','no'),(85,0,'rss_57bc725ad6568758915363af670fd8bc','O:9:\"MagpieRSS\":19:{s:6:\"parser\";i:0;s:12:\"current_item\";a:0:{}s:5:\"items\";a:15:{i:0;a:7:{s:5:\"title\";s:50:\"thamizhchelvan on \"Wordpress Related Posts Plugin\"\";s:4:\"link\";s:70:\"http://wordpress.org/extend/plugins/wordpress-related-posts/#post-6120\";s:7:\"pubdate\";s:31:\"Sat, 28 Jun 2008 14:10:12 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:14:\"thamizhchelvan\";}s:4:\"guid\";s:41:\"6120@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:71:\"Will add related links for posts &#038; pages with your own anchor text\";s:7:\"summary\";s:71:\"Will add related links for posts &#038; pages with your own anchor text\";}i:1;a:7:{s:5:\"title\";s:38:\"denishua on \"WordPress中文SEO插件\"\";s:4:\"link\";s:79:\"http://wordpress.org/extend/plugins/wordpress-seo-plugin-for-chinese/#post-6118\";s:7:\"pubdate\";s:31:\"Sat, 28 Jun 2008 09:26:05 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:8:\"denishua\";}s:4:\"guid\";s:41:\"6118@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:83:\"WordPress SEO Plugin for Chinese use Chinese Word Splitter to SEO Chinese SEO Blog.\";s:7:\"summary\";s:83:\"WordPress SEO Plugin for Chinese use Chinese Word Splitter to SEO Chinese SEO Blog.\";}i:2;a:7:{s:5:\"title\";s:34:\"chrisbliss18 on \"WP Easy Uploader\"\";s:4:\"link\";s:63:\"http://wordpress.org/extend/plugins/wp-easy-uploader/#post-6117\";s:7:\"pubdate\";s:31:\"Sat, 28 Jun 2008 04:21:40 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:12:\"chrisbliss18\";}s:4:\"guid\";s:41:\"6117@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:188:\"Easily upload any type of content without the need for <a href=\"http://FTP.\" rel=\"nofollow\">FTP.</a> You can even upload plugin and theme archives, and the files will be extracted for you.\";s:7:\"summary\";s:188:\"Easily upload any type of content without the need for <a href=\"http://FTP.\" rel=\"nofollow\">FTP.</a> You can even upload plugin and theme archives, and the files will be extracted for you.\";}i:3;a:7:{s:5:\"title\";s:31:\"DanCoulter on \"Datepicker i18n\"\";s:4:\"link\";s:62:\"http://wordpress.org/extend/plugins/datepicker-i18n/#post-6111\";s:7:\"pubdate\";s:31:\"Fri, 27 Jun 2008 16:05:35 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:10:\"DanCoulter\";}s:4:\"guid\";s:41:\"6111@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:55:\"Globally translate your jQuery UI Datepicker instances.\";s:7:\"summary\";s:55:\"Globally translate your jQuery UI Datepicker instances.\";}i:4;a:7:{s:5:\"title\";s:61:\"kurt_penang on \"Waktu Solat Malaysia / Malaysia Prayer Times\"\";s:4:\"link\";s:105:\"http://wordpress.org/extend/plugins/wp-prayer-times-waktu-solat-malaysia-malaysia-prayer-times/#post-6107\";s:7:\"pubdate\";s:31:\"Fri, 27 Jun 2008 10:20:51 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:11:\"kurt_penang\";}s:4:\"guid\";s:41:\"6107@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:30:\"Display Malaysia Prayer Times.\";s:7:\"summary\";s:30:\"Display Malaysia Prayer Times.\";}i:5;a:7:{s:5:\"title\";s:34:\"Trouts on \"Rot13 Email Protection\"\";s:4:\"link\";s:69:\"http://wordpress.org/extend/plugins/rot13-email-protection/#post-6103\";s:7:\"pubdate\";s:31:\"Thu, 26 Jun 2008 19:35:02 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:6:\"Trouts\";}s:4:\"guid\";s:41:\"6103@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:122:\"Protects all email addresses and mailto: links on your blog from email harvesters using rot13 encryption and/or text mode.\";s:7:\"summary\";s:122:\"Protects all email addresses and mailto: links on your blog from email harvesters using rot13 encryption and/or text mode.\";}i:6;a:7:{s:5:\"title\";s:38:\"donncha on \"WordPress Exploit Scanner\"\";s:4:\"link\";s:62:\"http://wordpress.org/extend/plugins/exploit-scanner/#post-6099\";s:7:\"pubdate\";s:31:\"Thu, 26 Jun 2008 14:19:03 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:7:\"donncha\";}s:4:\"guid\";s:41:\"6099@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:108:\"Search the files and database of your WordPress install for malicious code or spammy links left by a hacker.\";s:7:\"summary\";s:108:\"Search the files and database of your WordPress install for malicious code or spammy links left by a hacker.\";}i:7;a:7:{s:5:\"title\";s:38:\"microkid on \"Microkid\'s Related Posts\"\";s:4:\"link\";s:70:\"http://wordpress.org/extend/plugins/microkids-related-posts/#post-6098\";s:7:\"pubdate\";s:31:\"Thu, 26 Jun 2008 11:34:02 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:8:\"microkid\";}s:4:\"guid\";s:41:\"6098@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:113:\"Microkid\'s Related Posts plugin lets you manually select related posts using a nifty AJAX powered search utility.\";s:7:\"summary\";s:113:\"Microkid\'s Related Posts plugin lets you manually select related posts using a nifty AJAX powered search utility.\";}i:8;a:7:{s:5:\"title\";s:42:\"collinsinternet on \"Feed Subscriber Stats\"\";s:4:\"link\";s:68:\"http://wordpress.org/extend/plugins/feed-subscriber-stats/#post-6097\";s:7:\"pubdate\";s:31:\"Thu, 26 Jun 2008 04:21:22 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:15:\"collinsinternet\";}s:4:\"guid\";s:41:\"6097@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:72:\"This plugin will show the number of subscribers according to FeedBurner.\";s:7:\"summary\";s:72:\"This plugin will show the number of subscribers according to FeedBurner.\";}i:9;a:7:{s:5:\"title\";s:30:\"datafeedr.com on \"Plugin Name\"\";s:4:\"link\";s:57:\"http://wordpress.org/extend/plugins/random-ads/#post-6095\";s:7:\"pubdate\";s:31:\"Wed, 25 Jun 2008 19:33:50 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:13:\"datafeedr.com\";}s:4:\"guid\";s:41:\"6095@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:139:\"The Datafeedr Random Ads plugin allows you to simply and easily show random, rotating ads anywhere in your template files or using widgets.\";s:7:\"summary\";s:139:\"The Datafeedr Random Ads plugin allows you to simply and easily show random, rotating ads anywhere in your template files or using widgets.\";}i:10;a:7:{s:5:\"title\";s:32:\"pomspot on \"[GWA] AutoResponder\"\";s:4:\"link\";s:64:\"http://wordpress.org/extend/plugins/autoresponder-gwa/#post-6094\";s:7:\"pubdate\";s:31:\"Wed, 25 Jun 2008 15:30:07 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:7:\"pomspot\";}s:4:\"guid\";s:41:\"6094@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:77:\"A plugin to add an AutoResponder and Opt-in Mail List Builder to your weblog.\";s:7:\"summary\";s:77:\"A plugin to add an AutoResponder and Opt-in Mail List Builder to your weblog.\";}i:11;a:7:{s:5:\"title\";s:28:\"zambesc on \"UTF-8 Convertor\"\";s:4:\"link\";s:62:\"http://wordpress.org/extend/plugins/utf-8-convertor/#post-6091\";s:7:\"pubdate\";s:31:\"Wed, 25 Jun 2008 14:16:38 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:7:\"zambesc\";}s:4:\"guid\";s:41:\"6091@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:109:\"This plugin is design to convert all the characters that appear on your blog posts to UTF-8 English text only\";s:7:\"summary\";s:109:\"This plugin is design to convert all the characters that appear on your blog posts to UTF-8 English text only\";}i:12;a:7:{s:5:\"title\";s:25:\"76design on \"friendsroll\"\";s:4:\"link\";s:58:\"http://wordpress.org/extend/plugins/friendsroll/#post-6081\";s:7:\"pubdate\";s:31:\"Tue, 24 Jun 2008 14:18:54 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:8:\"76design\";}s:4:\"guid\";s:41:\"6081@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:102:\"Let your readers show they are part of your community. Brought to you by Thornley Fallis and 76design.\";s:7:\"summary\";s:102:\"Let your readers show they are part of your community. Brought to you by Thornley Fallis and 76design.\";}i:13;a:7:{s:5:\"title\";s:22:\"76design on \"toplinks\"\";s:4:\"link\";s:55:\"http://wordpress.org/extend/plugins/toplinks/#post-6080\";s:7:\"pubdate\";s:31:\"Tue, 24 Jun 2008 14:17:41 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:8:\"76design\";}s:4:\"guid\";s:41:\"6080@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:102:\"Let your readers show they are part of your community. Brought to you by Thornley Fallis and 76design.\";s:7:\"summary\";s:102:\"Let your readers show they are part of your community. Brought to you by Thornley Fallis and 76design.\";}i:14;a:7:{s:5:\"title\";s:26:\"bcallaars on \"WP-AlertBox\"\";s:4:\"link\";s:58:\"http://wordpress.org/extend/plugins/wp-alertbox/#post-6079\";s:7:\"pubdate\";s:31:\"Tue, 24 Jun 2008 11:22:38 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:9:\"bcallaars\";}s:4:\"guid\";s:41:\"6079@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:100:\"A simple alert box configureable by the admin. User get\'s a overlay div with the alert message once.\";s:7:\"summary\";s:100:\"A simple alert box configureable by the admin. User get\'s a overlay div with the alert message once.\";}}s:7:\"channel\";a:6:{s:5:\"title\";s:6:\"Newest\";s:4:\"link\";s:36:\"http://wordpress.org/extend/plugins/\";s:11:\"description\";s:6:\"Newest\";s:8:\"language\";s:2:\"en\";s:7:\"pubdate\";s:31:\"Mon, 30 Jun 2008 15:33:12 +0000\";s:7:\"tagline\";s:6:\"Newest\";}s:9:\"textinput\";a:0:{}s:5:\"image\";a:0:{}s:9:\"feed_type\";s:3:\"RSS\";s:12:\"feed_version\";s:3:\"2.0\";s:5:\"stack\";a:0:{}s:9:\"inchannel\";b:0;s:6:\"initem\";b:0;s:9:\"incontent\";b:0;s:11:\"intextinput\";b:0;s:7:\"inimage\";b:0;s:13:\"current_field\";s:0:\"\";s:17:\"current_namespace\";b:0;s:19:\"_CONTENT_CONSTRUCTS\";a:6:{i:0;s:7:\"content\";i:1;s:7:\"summary\";i:2;s:4:\"info\";i:3;s:5:\"title\";i:4;s:7:\"tagline\";i:5;s:9:\"copyright\";}s:13:\"last_modified\";s:21:\"2008-06-28 14:10:12\r\n\";s:4:\"etag\";s:36:\"\"30b602a1b78c9b1415df62c44a26151c\"\r\n\";}','no'),(86,0,'rss_57bc725ad6568758915363af670fd8bc_ts','1214839989','no'),(87,0,'rss_867bd5c64f85878d03a060509cd2f92c','O:9:\"MagpieRSS\":19:{s:6:\"parser\";i:0;s:12:\"current_item\";a:0:{}s:5:\"items\";a:50:{i:0;a:7:{s:5:\"title\";s:46:\"Dougal Campbell: Update on WordPress blog APIs\";s:4:\"guid\";s:32:\"http://dougal.gunters.org/?p=991\";s:4:\"link\";s:71:\"http://dougal.gunters.org/blog/2008/06/30/update-on-wordpress-blog-apis\";s:11:\"description\";s:2662:\"<p><a href=\"http://dougal.gunters.org/blog/2008/06/24/wordpress-26-beta-1\">I mentioned previously</a> that the <acronym title=\"eXtensible Markup Language\"><span class=\"caps\">XML</span></acronym>-<acronym title=\"Remote Procedure Call\"><span class=\"caps\">RPC</span></acronym> and Atom blog APIs would be disabled by default when <a href=\"http://wordpress.org/\">WordPress</a> 2.6 is released. This was a matter of some debate within the community, and there has been some clarification:</p>\n<ul>\n<li>The APIs will <strong>not</strong> be automatically disabled for sites <em>upgrading</em> from older versions. Since the APIs have previously been &#8216;on&#8217; by default, they will continue to function.</li>\n<li>For <em>new installs</em> of WordPress 2.6 and later, there will be an option presented at install-time to enable the APIs.</li>\n<li>There will be options in the <code>Write</code> settings to enable or disable <acronym title=\"eXtensible Markup Language\"><span class=\"caps\">XML</span></acronym>-<acronym title=\"Remote Procedure Call\"><span class=\"caps\">RPC</span></acronym> posting and Atom <acronym title=\"Application Interface\"><span class=\"caps\">API</span></acronym> posting individually.</li>\n</ul>\n<p>This sounds like the most reasonable path to make this change without causing disruption for those who have been using client tools like <a href=\"http://infinite-sushi.com/software/ecto/\">Ecto</a>, <a href=\"http://www.red-sweater.com/marsedit/\">MarsEdit</a>, or <a href=\"http://windowslivewriter.spaces.live.com/\">Windows Live Writer</a> (or third-party web services which can post to blogs, like <a href=\"http://flickr.com/\">Flickr</a> or <a href=\"http://del.icio.us/\">Delicious</a>) to post to their blogs.</p>\n<p>Also, though this change is being made under the moniker of a security improvement, that is not to imply that the current <acronym title=\"Application Interface\"><span class=\"caps\">API</span></acronym> code is not secure. It is simply a pretty standard practice to turn off services that are not used, just as when building a dedicated email server, you wouldn&#8217;t turn on FTP unless you absolutely needed it. <a href=\"http://wordpress.com/stats/posting/\">Stats</a> from WordPress.com have shown that only about 5% of its users utilize the client APIs, so it doesn&#8217;t make sense to automatically turn it on for the 95% who <em>aren&#8217;t</em> using them.</p>\n<p><a href=\"http://sharethis.com/item?&wp=2.6-beta1&amp;publisher=06a70a77-1fc0-46a9-81d1-6a696e6ed23f&amp;title=Update+on+WordPress+blog+APIs&amp;url=http%3A%2F%2Fdougal.gunters.org%2Fblog%2F2008%2F06%2F30%2Fupdate-on-wordpress-blog-apis\">ShareThis</a></p>\";s:7:\"pubdate\";s:31:\"Mon, 30 Jun 2008 13:45:35 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:6:\"Dougal\";}s:7:\"summary\";s:2662:\"<p><a href=\"http://dougal.gunters.org/blog/2008/06/24/wordpress-26-beta-1\">I mentioned previously</a> that the <acronym title=\"eXtensible Markup Language\"><span class=\"caps\">XML</span></acronym>-<acronym title=\"Remote Procedure Call\"><span class=\"caps\">RPC</span></acronym> and Atom blog APIs would be disabled by default when <a href=\"http://wordpress.org/\">WordPress</a> 2.6 is released. This was a matter of some debate within the community, and there has been some clarification:</p>\n<ul>\n<li>The APIs will <strong>not</strong> be automatically disabled for sites <em>upgrading</em> from older versions. Since the APIs have previously been &#8216;on&#8217; by default, they will continue to function.</li>\n<li>For <em>new installs</em> of WordPress 2.6 and later, there will be an option presented at install-time to enable the APIs.</li>\n<li>There will be options in the <code>Write</code> settings to enable or disable <acronym title=\"eXtensible Markup Language\"><span class=\"caps\">XML</span></acronym>-<acronym title=\"Remote Procedure Call\"><span class=\"caps\">RPC</span></acronym> posting and Atom <acronym title=\"Application Interface\"><span class=\"caps\">API</span></acronym> posting individually.</li>\n</ul>\n<p>This sounds like the most reasonable path to make this change without causing disruption for those who have been using client tools like <a href=\"http://infinite-sushi.com/software/ecto/\">Ecto</a>, <a href=\"http://www.red-sweater.com/marsedit/\">MarsEdit</a>, or <a href=\"http://windowslivewriter.spaces.live.com/\">Windows Live Writer</a> (or third-party web services which can post to blogs, like <a href=\"http://flickr.com/\">Flickr</a> or <a href=\"http://del.icio.us/\">Delicious</a>) to post to their blogs.</p>\n<p>Also, though this change is being made under the moniker of a security improvement, that is not to imply that the current <acronym title=\"Application Interface\"><span class=\"caps\">API</span></acronym> code is not secure. It is simply a pretty standard practice to turn off services that are not used, just as when building a dedicated email server, you wouldn&#8217;t turn on FTP unless you absolutely needed it. <a href=\"http://wordpress.com/stats/posting/\">Stats</a> from WordPress.com have shown that only about 5% of its users utilize the client APIs, so it doesn&#8217;t make sense to automatically turn it on for the 95% who <em>aren&#8217;t</em> using them.</p>\n<p><a href=\"http://sharethis.com/item?&wp=2.6-beta1&amp;publisher=06a70a77-1fc0-46a9-81d1-6a696e6ed23f&amp;title=Update+on+WordPress+blog+APIs&amp;url=http%3A%2F%2Fdougal.gunters.org%2Fblog%2F2008%2F06%2F30%2Fupdate-on-wordpress-blog-apis\">ShareThis</a></p>\";}i:1;a:7:{s:5:\"title\";s:59:\"Weblog Tools Collection: WordPress Plugin Releases for 6/30\";s:4:\"guid\";s:40:\"http://weblogtoolscollection.com/?p=3697\";s:4:\"link\";s:89:\"http://weblogtoolscollection.com/archives/2008/06/30/wordpress-plugin-releases-for-630-2/\";s:11:\"description\";s:2356:\"<p><a href=\"http://www.asiden.dk/semi-manual-breadcrumb-navigation-plugin-for-wordpress/\">Semi-manual Breadcrumb Navigation</a></p>\n<p>This plugin allows you to add breadcrumb navigation to your Wordpress installation. Unlike other Wordpress breadcrumb plugins, this one allows you to manually control the breadcrumb structure. At the same time it takes care of a few things automatically, hence the name.</p>\n<p><a href=\"http://ocaoimh.ie/exploit-scanner/\">WordPress Exploit Scanner</a></p>\n<p>This plugin searches the files and database of your website for signs of suspicious activity. It will not stop someone hacking into your site, but it may help you find any uploaded or compromised files left by the hacker. When a website is compromised, hackers leave behind scripts and modified content that can be found by manually searching through all the files on a site. Some of the methods used to hide their code or spam links are obvious, like using CSS to hide text, and we can search for those strings.</p>\n<p><a href=\"http://wordpress.org/extend/plugins/drop-caps/\">Drop Caps</a></p>\n<p>Adds mark-up to your posts and pages which allows you to create wicked drop cap.</p>\n<p><a href=\"http://adrien.gibrat.free.fr/wp-menus/\">WP-Menus</a></p>\n<p>Allows to include user menus via sidebar widgets or in themes.</p>\n<p><a href=\"http://www.microkid.net/wordpress/related-posts/\">Microkid&#8217;s Related Posts</a></p>\n<p>This Wordpress plugin lets you manually select related posts using a nifty AJAX powered search utility.</p>\n<p><a href=\"http://www.phpbits.info/wordpress-plugins/wp-related-post-plugin/\">Related Posts WordPress Plugin</a></p>\n<p>The ‘Related Posts’ wordpress plug-in can be used to link other relevant posts with our own anchor text.</p>\n<p><a href=\"http://blog.collinsinternet.com/20/feed-subscriber-stats-wordpress-plugin/\">Feed Subscriber Stats</a></p>\n<p>The plugin uses FeedBurner’s Awareness API to get your feed subscriber stats directly from FeedBurner.</p>\n<p><a href=\"http://broken-watch.info/?p=42\">Inline Google Docs</a></p>\n<p>This plugin allows the user to embed Google Document/Spreadsheet content in posts and pages using shortcode.</p>\n<p><a href=\"http://www.sajithmr.com/wordpress-announcement-plugin/\">WordPress Announcement</a></p>\n<p>Allows you to inform your visitors about any new updates to your site.</p>\";s:7:\"pubdate\";s:31:\"Mon, 30 Jun 2008 06:51:13 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Ajay\";}s:7:\"summary\";s:2356:\"<p><a href=\"http://www.asiden.dk/semi-manual-breadcrumb-navigation-plugin-for-wordpress/\">Semi-manual Breadcrumb Navigation</a></p>\n<p>This plugin allows you to add breadcrumb navigation to your Wordpress installation. Unlike other Wordpress breadcrumb plugins, this one allows you to manually control the breadcrumb structure. At the same time it takes care of a few things automatically, hence the name.</p>\n<p><a href=\"http://ocaoimh.ie/exploit-scanner/\">WordPress Exploit Scanner</a></p>\n<p>This plugin searches the files and database of your website for signs of suspicious activity. It will not stop someone hacking into your site, but it may help you find any uploaded or compromised files left by the hacker. When a website is compromised, hackers leave behind scripts and modified content that can be found by manually searching through all the files on a site. Some of the methods used to hide their code or spam links are obvious, like using CSS to hide text, and we can search for those strings.</p>\n<p><a href=\"http://wordpress.org/extend/plugins/drop-caps/\">Drop Caps</a></p>\n<p>Adds mark-up to your posts and pages which allows you to create wicked drop cap.</p>\n<p><a href=\"http://adrien.gibrat.free.fr/wp-menus/\">WP-Menus</a></p>\n<p>Allows to include user menus via sidebar widgets or in themes.</p>\n<p><a href=\"http://www.microkid.net/wordpress/related-posts/\">Microkid&#8217;s Related Posts</a></p>\n<p>This Wordpress plugin lets you manually select related posts using a nifty AJAX powered search utility.</p>\n<p><a href=\"http://www.phpbits.info/wordpress-plugins/wp-related-post-plugin/\">Related Posts WordPress Plugin</a></p>\n<p>The ‘Related Posts’ wordpress plug-in can be used to link other relevant posts with our own anchor text.</p>\n<p><a href=\"http://blog.collinsinternet.com/20/feed-subscriber-stats-wordpress-plugin/\">Feed Subscriber Stats</a></p>\n<p>The plugin uses FeedBurner’s Awareness API to get your feed subscriber stats directly from FeedBurner.</p>\n<p><a href=\"http://broken-watch.info/?p=42\">Inline Google Docs</a></p>\n<p>This plugin allows the user to embed Google Document/Spreadsheet content in posts and pages using shortcode.</p>\n<p><a href=\"http://www.sajithmr.com/wordpress-announcement-plugin/\">WordPress Announcement</a></p>\n<p>Allows you to inform your visitors about any new updates to your site.</p>\";}i:2;a:7:{s:5:\"title\";s:46:\"Weblog Tools Collection: 3 Under Rated Plugins\";s:4:\"guid\";s:40:\"http://weblogtoolscollection.com/?p=3695\";s:4:\"link\";s:75:\"http://weblogtoolscollection.com/archives/2008/06/29/3-under-rated-plugins/\";s:11:\"description\";s:2520:\"<p>There appears to be a WordPress meme going around (started by <a href=\"http://wordpress.jdwebdev.com/blog/underrated-wordpress-plugins/\" target=\"_blank\">John at WordPress Expert</a>) where members throughout the community are tagging other blogs along with mentioning three plugins which they feel are under rated. Although I don&#8217;t have access to the plugins used here on WLTC, I would like to bring about three under rated plugins I use on my own blog.</p>\n<ol>\n<li><a href=\"http://www.aviransplace.com/index.php/digg-this-wordpress-plugin/\" target=\"_blank\">DiggThis</a> - This plugin was written by <cite><a title=\"Visit author homepage\" href=\"http://www.aviransplace.com/\">Aviran Mordod</a></cite> This by far my favorite DiggThis plugin because it only displays on posts which have been submitted to Digg. My opinion, which is shared by many others is that, it&#8217;s better to show a DiggThis button when the article has been submitted to Digg rather than showing a Digg button next to every article. Unless you are ReadWriteWeb or some other mainstream blog, those buttons will show 0 diggs and will possibly hint to the content not being up to snuff. I suppose its one of those psychological things.</li>\n<li><a href=\"http://rmarsh.com/plugins/highlight-comments/\" target=\"_blank\">Highlight Author Comments</a> - This plugin written by <cite><a title=\"Visit author homepage\" href=\"http://rmarsh.com/\">Rob Marsh, SJ</a>.</cite> makes following comments written by the blog author a cinch. After installing and then activating the plugin, you can style what blog authors comments will look like, giving you the ability to distinguish your comments versus all other comments.</li>\n<li><a href=\"http://blogwaffe.com/2006/10/04/421/\" target=\"_blank\">No Self Pings</a> - No one has been able to tell me what the purpose is behind WordPress sending pings and tackbacks to itself if you link to an article located in your archive. I&#8217;ve found myself always deleting these pingbacks. Thanks this this plugin written by <cite><a title=\"Visit author homepage\" href=\"http://blogwaffe.com/\">Michael D. Adams</a>.</cite> this is no longer an issue for me. Now, I can easily link to myself without having to deal with those pingbacks giving me that much more free time to do other things.</li>\n</ol>\n<p>These are three plugins which I feel are under rated that I use on my own blog. I invite you to partake in this meme and highlight three plugins you are using on your own blog that you feel are under rated.</p>\";s:7:\"pubdate\";s:31:\"Sun, 29 Jun 2008 22:44:53 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:13:\"Jeff Chandler\";}s:7:\"summary\";s:2520:\"<p>There appears to be a WordPress meme going around (started by <a href=\"http://wordpress.jdwebdev.com/blog/underrated-wordpress-plugins/\" target=\"_blank\">John at WordPress Expert</a>) where members throughout the community are tagging other blogs along with mentioning three plugins which they feel are under rated. Although I don&#8217;t have access to the plugins used here on WLTC, I would like to bring about three under rated plugins I use on my own blog.</p>\n<ol>\n<li><a href=\"http://www.aviransplace.com/index.php/digg-this-wordpress-plugin/\" target=\"_blank\">DiggThis</a> - This plugin was written by <cite><a title=\"Visit author homepage\" href=\"http://www.aviransplace.com/\">Aviran Mordod</a></cite> This by far my favorite DiggThis plugin because it only displays on posts which have been submitted to Digg. My opinion, which is shared by many others is that, it&#8217;s better to show a DiggThis button when the article has been submitted to Digg rather than showing a Digg button next to every article. Unless you are ReadWriteWeb or some other mainstream blog, those buttons will show 0 diggs and will possibly hint to the content not being up to snuff. I suppose its one of those psychological things.</li>\n<li><a href=\"http://rmarsh.com/plugins/highlight-comments/\" target=\"_blank\">Highlight Author Comments</a> - This plugin written by <cite><a title=\"Visit author homepage\" href=\"http://rmarsh.com/\">Rob Marsh, SJ</a>.</cite> makes following comments written by the blog author a cinch. After installing and then activating the plugin, you can style what blog authors comments will look like, giving you the ability to distinguish your comments versus all other comments.</li>\n<li><a href=\"http://blogwaffe.com/2006/10/04/421/\" target=\"_blank\">No Self Pings</a> - No one has been able to tell me what the purpose is behind WordPress sending pings and tackbacks to itself if you link to an article located in your archive. I&#8217;ve found myself always deleting these pingbacks. Thanks this this plugin written by <cite><a title=\"Visit author homepage\" href=\"http://blogwaffe.com/\">Michael D. Adams</a>.</cite> this is no longer an issue for me. Now, I can easily link to myself without having to deal with those pingbacks giving me that much more free time to do other things.</li>\n</ol>\n<p>These are three plugins which I feel are under rated that I use on my own blog. I invite you to partake in this meme and highlight three plugins you are using on your own blog that you feel are under rated.</p>\";}i:3;a:7:{s:5:\"title\";s:51:\"Weblog Tools Collection: Google Trends for Websites\";s:4:\"guid\";s:40:\"http://weblogtoolscollection.com/?p=3694\";s:4:\"link\";s:80:\"http://weblogtoolscollection.com/archives/2008/06/28/google-trends-for-websites/\";s:11:\"description\";s:635:\"<p><a href=\"http://trends.google.com/websites?q=weblogtoolscollection.com\">Google Trends: weblogtoolscollection.com</a> provides (slightly) interesting facts on your blog. Though the product is free, signing in is required to see the number estimates. Not all sites seem to have a lot of data in Google Trends but it was fun to browse the various links. Most interesting to me were the &#8220;also visited&#8221; section and the &#8220;also searched&#8221; section. <a href=\"http://www.mattcutts.com/blog/google-trends-for-websites/\">Matt Cutts had an interesting writeup on Google Trends for websites</a> a little over a week ago.</p>\";s:7:\"pubdate\";s:31:\"Sat, 28 Jun 2008 23:57:02 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:10:\"Mark Ghosh\";}s:7:\"summary\";s:635:\"<p><a href=\"http://trends.google.com/websites?q=weblogtoolscollection.com\">Google Trends: weblogtoolscollection.com</a> provides (slightly) interesting facts on your blog. Though the product is free, signing in is required to see the number estimates. Not all sites seem to have a lot of data in Google Trends but it was fun to browse the various links. Most interesting to me were the &#8220;also visited&#8221; section and the &#8220;also searched&#8221; section. <a href=\"http://www.mattcutts.com/blog/google-trends-for-websites/\">Matt Cutts had an interesting writeup on Google Trends for websites</a> a little over a week ago.</p>\";}i:4;a:7:{s:5:\"title\";s:26:\"Alex King: PhoneFactor 1.0\";s:4:\"guid\";s:50:\"http://alexking.org/blog/2008/06/27/phonefactor-10\";s:4:\"link\";s:50:\"http://alexking.org/blog/2008/06/27/phonefactor-10\";s:11:\"description\";s:2446:\"<p>PhoneFactor is a <a href=\"http://wordpress.org/\" rel=\"external\">WordPress</a> plugin that provides an additional layer of security when logging in to your WordPress site.</p>\n<p class=\"center\"><a href=\"http://www.flickr.com/photos/alexkingorg/sets/72157605848461623/\"><img src=\"http://farm4.static.flickr.com/3149/2616262594_5cf244979b_m.jpg\" width=\"240\" height=\"177\" alt=\"Login\" /></a></p>\n<p>It&#8217;s a pretty cool system. When you log in to your WordPress blog with PhoneFactor enabled, you receive a phone call asking you to press # to authorize. When you do so, you are logged in. If you do not, then the login attempt fails.</p>\n<p>Basically, this means someone needs to get your username, password <em>and</em> phone in order to log in to your WordPress site as you.</p>\n<p><a href=\"http://crowdfavorite.com\">Crowd Favorite</a> worked with the <a href=\"http://phonefactor.com/\" rel=\"external\">PhoneFactor</a> team to build this integration on their API layer. This was a fun project for us - it&#8217;s always nice to enhance WordPress by integrating features from another service.</p>\n<p>Besides building the integration with the API to authenticate via phone, we also built the PhoneFactor registration step right into the WordPress admin so that the user wouldn&#8217;t have to leave the WordPress admin when activating the plugin and getting it set up. I think it&#8217;s a much better user experience.</p>\n<p class=\"center\"><a href=\"http://www.flickr.com/photos/alexkingorg/2615434383/\" title=\"Registration by alexkingorg, on Flickr\"><img src=\"http://farm4.static.flickr.com/3119/2615434383_9647b143f0_m.jpg\" width=\"240\" height=\"178\" alt=\"Registration\" /></a></p>\n<p>We integrated some instructional banners into the WordPress admin pages to let people know the next steps to getting things set up after activating the plugin. We also built in some more advanced user features like whitelisting IP addresses so that you don&#8217;t have to use PhoneFactor authentication if you&#8217;re logging in from your home machine.</p>\n<p>The download and more information are available at the <a href=\"http://wordpress.org/extend/plugins/phonefactor/\">WordPress Plugin repository</a> on wordpress.org.</p>\n<p>Enjoy!</p>\n<p><a href=\"http://sharethis.com/item?&wp=2.2.3&amp;publisher=06654962-d77d-102a-861d-00161729a8a2&amp;title=PhoneFactor+1.0&amp;url=http%3A%2F%2Falexking.org%2Fblog%2F2008%2F06%2F27%2Fphonefactor-10\">ShareThis</a></p>\";s:7:\"pubdate\";s:31:\"Fri, 27 Jun 2008 23:13:22 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Alex\";}s:7:\"summary\";s:2446:\"<p>PhoneFactor is a <a href=\"http://wordpress.org/\" rel=\"external\">WordPress</a> plugin that provides an additional layer of security when logging in to your WordPress site.</p>\n<p class=\"center\"><a href=\"http://www.flickr.com/photos/alexkingorg/sets/72157605848461623/\"><img src=\"http://farm4.static.flickr.com/3149/2616262594_5cf244979b_m.jpg\" width=\"240\" height=\"177\" alt=\"Login\" /></a></p>\n<p>It&#8217;s a pretty cool system. When you log in to your WordPress blog with PhoneFactor enabled, you receive a phone call asking you to press # to authorize. When you do so, you are logged in. If you do not, then the login attempt fails.</p>\n<p>Basically, this means someone needs to get your username, password <em>and</em> phone in order to log in to your WordPress site as you.</p>\n<p><a href=\"http://crowdfavorite.com\">Crowd Favorite</a> worked with the <a href=\"http://phonefactor.com/\" rel=\"external\">PhoneFactor</a> team to build this integration on their API layer. This was a fun project for us - it&#8217;s always nice to enhance WordPress by integrating features from another service.</p>\n<p>Besides building the integration with the API to authenticate via phone, we also built the PhoneFactor registration step right into the WordPress admin so that the user wouldn&#8217;t have to leave the WordPress admin when activating the plugin and getting it set up. I think it&#8217;s a much better user experience.</p>\n<p class=\"center\"><a href=\"http://www.flickr.com/photos/alexkingorg/2615434383/\" title=\"Registration by alexkingorg, on Flickr\"><img src=\"http://farm4.static.flickr.com/3119/2615434383_9647b143f0_m.jpg\" width=\"240\" height=\"178\" alt=\"Registration\" /></a></p>\n<p>We integrated some instructional banners into the WordPress admin pages to let people know the next steps to getting things set up after activating the plugin. We also built in some more advanced user features like whitelisting IP addresses so that you don&#8217;t have to use PhoneFactor authentication if you&#8217;re logging in from your home machine.</p>\n<p>The download and more information are available at the <a href=\"http://wordpress.org/extend/plugins/phonefactor/\">WordPress Plugin repository</a> on wordpress.org.</p>\n<p>Enjoy!</p>\n<p><a href=\"http://sharethis.com/item?&wp=2.2.3&amp;publisher=06654962-d77d-102a-861d-00161729a8a2&amp;title=PhoneFactor+1.0&amp;url=http%3A%2F%2Falexking.org%2Fblog%2F2008%2F06%2F27%2Fphonefactor-10\">ShareThis</a></p>\";}i:5;a:7:{s:5:\"title\";s:35:\"Gravatar: Add a Face to any Mailto:\";s:4:\"guid\";s:35:\"http://gravatar.wordpress.com/?p=59\";s:4:\"link\";s:61:\"http://blog.gravatar.com/2008/06/27/add-a-face-to-any-mailto/\";s:11:\"description\";s:1784:\"<div class=\"snap_preview\"><br /><p>Recently I came across a <a title=\"greasemonkey script for adding gravatars to mailto links\" href=\"http://userscripts.org/scripts/review/29070\">Greasemonkey userscript which adds a gravatar image to every mailto</a>: link on a web page. Clever use of <a title=\"greasemonkey\" href=\"https://addons.mozilla.org/en-US/firefox/addon/748\">interesting technologies</a> by <a href=\"http://userscripts.org/users/57156\">docwhat</a>.  Very cool!</p>\n<img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/categories/gravatar.wordpress.com/59/\" /> <img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/tags/gravatar.wordpress.com/59/\" /> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gocomments/gravatar.wordpress.com/59/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/comments/gravatar.wordpress.com/59/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godelicious/gravatar.wordpress.com/59/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/delicious/gravatar.wordpress.com/59/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gostumble/gravatar.wordpress.com/59/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/stumble/gravatar.wordpress.com/59/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godigg/gravatar.wordpress.com/59/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/digg/gravatar.wordpress.com/59/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/goreddit/gravatar.wordpress.com/59/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/reddit/gravatar.wordpress.com/59/\" /></a> <img alt=\"\" border=\"0\" src=\"http://stats.wordpress.com/b.gif?host=blog.gravatar.com&blog=1886259&post=59&subd=gravatar&ref=&feed=1\" /></div>\";s:7:\"pubdate\";s:31:\"Fri, 27 Jun 2008 18:43:37 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:11:\"apokalyptik\";}s:7:\"summary\";s:1784:\"<div class=\"snap_preview\"><br /><p>Recently I came across a <a title=\"greasemonkey script for adding gravatars to mailto links\" href=\"http://userscripts.org/scripts/review/29070\">Greasemonkey userscript which adds a gravatar image to every mailto</a>: link on a web page. Clever use of <a title=\"greasemonkey\" href=\"https://addons.mozilla.org/en-US/firefox/addon/748\">interesting technologies</a> by <a href=\"http://userscripts.org/users/57156\">docwhat</a>.  Very cool!</p>\n<img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/categories/gravatar.wordpress.com/59/\" /> <img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/tags/gravatar.wordpress.com/59/\" /> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gocomments/gravatar.wordpress.com/59/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/comments/gravatar.wordpress.com/59/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godelicious/gravatar.wordpress.com/59/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/delicious/gravatar.wordpress.com/59/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gostumble/gravatar.wordpress.com/59/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/stumble/gravatar.wordpress.com/59/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godigg/gravatar.wordpress.com/59/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/digg/gravatar.wordpress.com/59/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/goreddit/gravatar.wordpress.com/59/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/reddit/gravatar.wordpress.com/59/\" /></a> <img alt=\"\" border=\"0\" src=\"http://stats.wordpress.com/b.gif?host=blog.gravatar.com&blog=1886259&post=59&subd=gravatar&ref=&feed=1\" /></div>\";}i:6;a:7:{s:5:\"title\";s:58:\"Weblog Tools Collection: WordPress Theme Releases For 6/26\";s:4:\"guid\";s:86:\"http://weblogtoolscollection.com/archives/2008/06/26/wordpress-theme-releases-for-626/\";s:4:\"link\";s:86:\"http://weblogtoolscollection.com/archives/2008/06/26/wordpress-theme-releases-for-626/\";s:11:\"description\";s:2772:\"<h3>One Column Theme</h3>\n<p><a href=\"http://templates.arcsin.se/grunge-superstar-wordpress-theme/\" target=\"_blank\"><strong>Grunge Superstar</strong></a></p>\n<p><img title=\"grunge-superstar-thumbnail\" height=\"161\" alt=\"grunge-superstar-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/grungesuperstarthumbnail.png\" width=\"200\" border=\"0\" /> </p>\n<p>Grunge Superstar is a simple one column theme. The theme uses dark colors and you can add in additional information in the footer area.</p>\n<p><a href=\"http://redworks.sk/linquist/\" target=\"_blank\"><strong>Linquist</strong></a></p>\n<p><img title=\"linquist-thumbnail\" height=\"145\" alt=\"linquist-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/linquistthumbnail.png\" width=\"200\" border=\"0\" /> </p>\n<p>Linquist is a simple one column theme which can be used to display portfolios. The theme comes in light and dark styles and has built-in lightbox and gravatar support.</p>\n<h3>Two Column Themes</h3>\n<p><a href=\"http://www.dailywp.com/brightness-wordpress-theme/\" target=\"_blank\"><strong>Brightness</strong></a></p>\n<p><img title=\"brightness-thumbnail\" height=\"151\" alt=\"brightness-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/brightnessthumbnail.png\" width=\"200\" border=\"0\" /> </p>\n<p>Brightness is a magazine style 2 column widget-ready theme. You can control the layout of the theme using settings in the Admin Panel.</p>\n<p><a href=\"http://www.wp-magazine.se/free-theme-wp-pixels/\" target=\"_blank\"><strong>WP Pixels</strong></a></p>\n<p><img title=\"wp-pixels-thumbnail\" height=\"159\" alt=\"wp-pixels-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/wppixelsthumbnail.png\" width=\"229\" border=\"0\" /> </p>\n<p>WP Pixels is a two column widget-ready theme. The home page has no sidebars and uses loops for three different categories.</p>\n<h3>Three Column Themes</h3>\n<p><a href=\"http://www.rvoodoo.com/2008/06/voodoo-empire-theme-available/\" target=\"_blank\"><strong>Voodoo Empire</strong></a></p>\n<p><img title=\"rev-vodoo-thumbnail\" height=\"125\" alt=\"rev-vodoo-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/revvodoothumbnail.png\" width=\"199\" border=\"0\" /> </p>\n<p>Voodoo Empire is a three column widget ready theme. The theme makes use of dark colors. Both the sidebars are to the right of the content.</p>\n<p><a href=\"http://www.alibiproductions.com/html/elements-of-seo.html\" target=\"_blank\"><strong>Elements of SEO</strong></a></p>\n<p><img title=\"elements-of-seo-thumbnail\" height=\"125\" alt=\"elements-of-seo-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/elementsofseothumbnail.png\" width=\"200\" border=\"0\" /> </p>\n<p>Elements of SEO is a 3 column widget-ready theme. The theme has two sidebars to the right of the content.</p>\";s:7:\"pubdate\";s:31:\"Fri, 27 Jun 2008 03:58:00 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:12:\"Keith Dsouza\";}s:7:\"summary\";s:2772:\"<h3>One Column Theme</h3>\n<p><a href=\"http://templates.arcsin.se/grunge-superstar-wordpress-theme/\" target=\"_blank\"><strong>Grunge Superstar</strong></a></p>\n<p><img title=\"grunge-superstar-thumbnail\" height=\"161\" alt=\"grunge-superstar-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/grungesuperstarthumbnail.png\" width=\"200\" border=\"0\" /> </p>\n<p>Grunge Superstar is a simple one column theme. The theme uses dark colors and you can add in additional information in the footer area.</p>\n<p><a href=\"http://redworks.sk/linquist/\" target=\"_blank\"><strong>Linquist</strong></a></p>\n<p><img title=\"linquist-thumbnail\" height=\"145\" alt=\"linquist-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/linquistthumbnail.png\" width=\"200\" border=\"0\" /> </p>\n<p>Linquist is a simple one column theme which can be used to display portfolios. The theme comes in light and dark styles and has built-in lightbox and gravatar support.</p>\n<h3>Two Column Themes</h3>\n<p><a href=\"http://www.dailywp.com/brightness-wordpress-theme/\" target=\"_blank\"><strong>Brightness</strong></a></p>\n<p><img title=\"brightness-thumbnail\" height=\"151\" alt=\"brightness-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/brightnessthumbnail.png\" width=\"200\" border=\"0\" /> </p>\n<p>Brightness is a magazine style 2 column widget-ready theme. You can control the layout of the theme using settings in the Admin Panel.</p>\n<p><a href=\"http://www.wp-magazine.se/free-theme-wp-pixels/\" target=\"_blank\"><strong>WP Pixels</strong></a></p>\n<p><img title=\"wp-pixels-thumbnail\" height=\"159\" alt=\"wp-pixels-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/wppixelsthumbnail.png\" width=\"229\" border=\"0\" /> </p>\n<p>WP Pixels is a two column widget-ready theme. The home page has no sidebars and uses loops for three different categories.</p>\n<h3>Three Column Themes</h3>\n<p><a href=\"http://www.rvoodoo.com/2008/06/voodoo-empire-theme-available/\" target=\"_blank\"><strong>Voodoo Empire</strong></a></p>\n<p><img title=\"rev-vodoo-thumbnail\" height=\"125\" alt=\"rev-vodoo-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/revvodoothumbnail.png\" width=\"199\" border=\"0\" /> </p>\n<p>Voodoo Empire is a three column widget ready theme. The theme makes use of dark colors. Both the sidebars are to the right of the content.</p>\n<p><a href=\"http://www.alibiproductions.com/html/elements-of-seo.html\" target=\"_blank\"><strong>Elements of SEO</strong></a></p>\n<p><img title=\"elements-of-seo-thumbnail\" height=\"125\" alt=\"elements-of-seo-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/elementsofseothumbnail.png\" width=\"200\" border=\"0\" /> </p>\n<p>Elements of SEO is a 3 column widget-ready theme. The theme has two sidebars to the right of the content.</p>\";}i:7;a:7:{s:5:\"title\";s:38:\"Donncha: WordPress Exploit Scanner 0.1\";s:4:\"guid\";s:29:\"http://ocaoimh.ie/?p=89493777\";s:4:\"link\";s:58:\"http://ocaoimh.ie/2008/06/26/wordpress-exploit-scanner-01/\";s:11:\"description\";s:2241:\"<p>My previous post about <a href=\"http://ocaoimh.ie/2008/06/08/did-your-wordpress-site-get-hacked/\">hacked WordPress sites</a> caused <a href=\"http://www.donnacha.com/\">Donnacha</a> to <a href=\"http://ocaoimh.ie/2008/06/16/catch-website-file-changes-with-aide/comment-page-1/#comment-651462\">ask</a>,<br />\n<blockquote>After your last post on this subject, I was thinking that it would be a good idea for Automattic to create a plugin that carries out all the checks you suggested people do to find out if they’ve been hacked&#8230;</p></blockquote>\n<p>At the time I wasn&#8217;t too optimistic about it but after thinking about the idea for a few days I came up with the <a href=\"http://ocaoimh.ie/exploit-scanner/\">WordPress Exploit Scanner</a> which does most of what Donnacha wanted.</p>\n<p>This WordPress plugin searches the files on your site for a few known strings sometimes used by hackers, and lists them with code fragments taken from the files. It also makes a few checks of the database, looking at the active_plugins blog option, the comments table, and the posts table.<br />\nIt also allows the blog owner to search for whatever string they like which could come in handy when new exploit code is used in a hack.</p>\n<p>You must be running WordPress 2.5.1 or higher to use this plugin. There&#8217;s not much point in finding exploited files if you&#8217;re running an old version of the software that can be broken into again.</p>\n<p>Download the plugin from here: <a href=\"http://ocaoimh.ie/exploit-scanner/\">WordPress Exploit Scanner</a></p>\n<p>Thanks to those who tested the plugin, especially <a href=\"http://cgarvey.com/\">Cathal Garvey</a> who provided some great feedback!</p>\n<p><img src=\"http://ocaoimh.ie/?voyeur=1\" /></p><p><strong>Related Posts</strong><ul><li><a href=\"http://ocaoimh.ie/exploit-scanner/\" rel=\"bookmark\" title=\"Permanent Link: WordPress Exploit Scanner\">WordPress Exploit Scanner</a></li><li><a href=\"http://ocaoimh.ie/wordpress-plugins/\" rel=\"bookmark\" title=\"Permanent Link: WordPress Plugins\">WordPress Plugins</a></li><li><a href=\"http://ocaoimh.ie/2003/11/25/linux-desktop-surprises/\" rel=\"bookmark\" title=\"Permanent Link: Linux Desktop Surprises\">Linux Desktop Surprises</a></li></ul></p>\";s:7:\"pubdate\";s:31:\"Thu, 26 Jun 2008 18:17:55 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:7:\"Donncha\";}s:7:\"summary\";s:2241:\"<p>My previous post about <a href=\"http://ocaoimh.ie/2008/06/08/did-your-wordpress-site-get-hacked/\">hacked WordPress sites</a> caused <a href=\"http://www.donnacha.com/\">Donnacha</a> to <a href=\"http://ocaoimh.ie/2008/06/16/catch-website-file-changes-with-aide/comment-page-1/#comment-651462\">ask</a>,<br />\n<blockquote>After your last post on this subject, I was thinking that it would be a good idea for Automattic to create a plugin that carries out all the checks you suggested people do to find out if they’ve been hacked&#8230;</p></blockquote>\n<p>At the time I wasn&#8217;t too optimistic about it but after thinking about the idea for a few days I came up with the <a href=\"http://ocaoimh.ie/exploit-scanner/\">WordPress Exploit Scanner</a> which does most of what Donnacha wanted.</p>\n<p>This WordPress plugin searches the files on your site for a few known strings sometimes used by hackers, and lists them with code fragments taken from the files. It also makes a few checks of the database, looking at the active_plugins blog option, the comments table, and the posts table.<br />\nIt also allows the blog owner to search for whatever string they like which could come in handy when new exploit code is used in a hack.</p>\n<p>You must be running WordPress 2.5.1 or higher to use this plugin. There&#8217;s not much point in finding exploited files if you&#8217;re running an old version of the software that can be broken into again.</p>\n<p>Download the plugin from here: <a href=\"http://ocaoimh.ie/exploit-scanner/\">WordPress Exploit Scanner</a></p>\n<p>Thanks to those who tested the plugin, especially <a href=\"http://cgarvey.com/\">Cathal Garvey</a> who provided some great feedback!</p>\n<p><img src=\"http://ocaoimh.ie/?voyeur=1\" /></p><p><strong>Related Posts</strong><ul><li><a href=\"http://ocaoimh.ie/exploit-scanner/\" rel=\"bookmark\" title=\"Permanent Link: WordPress Exploit Scanner\">WordPress Exploit Scanner</a></li><li><a href=\"http://ocaoimh.ie/wordpress-plugins/\" rel=\"bookmark\" title=\"Permanent Link: WordPress Plugins\">WordPress Plugins</a></li><li><a href=\"http://ocaoimh.ie/2003/11/25/linux-desktop-surprises/\" rel=\"bookmark\" title=\"Permanent Link: Linux Desktop Surprises\">Linux Desktop Surprises</a></li></ul></p>\";}i:8;a:7:{s:5:\"title\";s:100:\"WordPress Podcast: Episode 43: Out of date blogs hacked, All-In-One SEO gets dropped, then picked up\";s:4:\"guid\";s:29:\"http://wp-community.org/?p=79\";s:4:\"link\";s:46:\"http://wp-community.org/2008/06/26/episode-43/\";s:11:\"description\";s:3269:\"<ol>\n<li><a href=\"http://www.thecodecave.com/\" title=\"The Code Cave\" target=\"_blank\" class=\"extlink\">Brian Layman</a>, of <a href=\"http://www.b5media.com\" title=\"b5media homepage\" target=\"_self\" class=\"extlink\">b5media</a>, joins Charles as co-host, filling in for <a href=\"http://www.plagiarismtoday.com\" title=\"Plagiarism Today\" target=\"_blank\" class=\"extlink\">Jonathan Bailey</a> who&#8217;s in Newcastle-upon-tyne, United Kingdom speaking at the <a href=\"http://www.plagiarismconference.co.uk\" title=\"3rd International Plagiarism Conference\" target=\"_blank\" class=\"extlink\">3rd International Plagiarism Conference</a>.</li>\n<li>Some blogs running older versions of WordPress <a href=\"http://ocaoimh.ie/2008/06/08/did-your-wordpress-site-get-hacked/\" title=\"Holy Shmoly!: Did your WordPress site get hacked?\" target=\"_blank\" class=\"extlink\">were hacked</a>. Peter offers a way to <a href=\"http://www.theblog.ca/file-change-notifications\" title=\"Peter\'s Useless Crap: File change notifications for your WordPress blog on Linux\" target=\"_blank\" class=\"extlink\">have your site notify you</a> when things are changed.</li>\n<li>Support for the popular plugin All in One SEO Pack was <a href=\"http://weblogtoolscollection.com/archives/2008/06/17/all-in-one-seo-lives-on/\" title=\"Weblog Tools Collection: All In One SEO Lives On\" target=\"_blank\" class=\"extlink\">dropped by its author</a>, only to be picked up a few days later by a new author.</li>\n<li><a href=\"http://www.jeffro2pt0.com/ready-to-ride-the-crazy-horse\" title=\"Jeffro2pt0: Ready to ride the Crazy Horse\" target=\"_blank\" class=\"extlink\">Crazy Horse</a> is a <a href=\"http://svn.automattic.com/wordpress/branches/crazyhorse/\" title=\"Crazy Horse on subversion\" target=\"_blank\" class=\"extlink\">development branch</a> of WordPress to further the admin overhaul begun by WordPress 2.5. (<a href=\"http://ma.tt/dropbox/2008/06/wordpress-prototype-1.1.pdf\" title=\"Crazy Horse prototype PDF\" target=\"_blank\" class=\"extlink\">PDF document</a>)</li>\n<li>Automattic Dev Andy Skelton introduced <a href=\"http://andy.wordpress.com/2008/06/22/batcache-for-wordpress/\" title=\"Andy Skelton: Batcache for WordPress\" target=\"_blank\" class=\"extlink\">Batcache</a>, a non file-based caching solution based on the one WordPress.com uses.</li>\n<li><a href=\"http://codex.wordpress.org/HyperDB\" title=\"WordPress Codex: HyperDB\" target=\"_blank\" class=\"extlink\">HyperDB</a> <a href=\"http://comox.textdrive.com/pipermail/hyperdb/2008-June/000093.html\" title=\"HyperDB update\" target=\"_blank\" class=\"extlink\">updated for compatibility</a> with WordPress trunk at revision 8094.</li>\n<li>Get well soon, <a href=\"http://lorelle.wordpress.com/\" title=\"Lorelle on WordPress\" target=\"_blank\" class=\"extlink\">Lorelle</a>!</li>\n<li>Plugin: <a href=\"http://dev.wp-plugins.org/wiki/AuthorHighlight\" title=\"Author Highlight plugin\" target=\"_blank\" class=\"extlink\">Author Highlight</a> allows you to style the comments made by you or other authors differently than regular commenters.</li>\n<li>Plugin: <a href=\"http://semperfiwebdesign.com/plugins/wp-security-scan/\" title=\"WP Security Scan\" target=\"_blank\" class=\"extlink\">WP Security Scan </a>scans your WordPress installation for security vulnerabilities and suggests corrective actions.</li>\n</ol>\";s:7:\"pubdate\";s:31:\"Thu, 26 Jun 2008 17:58:55 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:17:\"Charles Stricklin\";}s:7:\"summary\";s:3269:\"<ol>\n<li><a href=\"http://www.thecodecave.com/\" title=\"The Code Cave\" target=\"_blank\" class=\"extlink\">Brian Layman</a>, of <a href=\"http://www.b5media.com\" title=\"b5media homepage\" target=\"_self\" class=\"extlink\">b5media</a>, joins Charles as co-host, filling in for <a href=\"http://www.plagiarismtoday.com\" title=\"Plagiarism Today\" target=\"_blank\" class=\"extlink\">Jonathan Bailey</a> who&#8217;s in Newcastle-upon-tyne, United Kingdom speaking at the <a href=\"http://www.plagiarismconference.co.uk\" title=\"3rd International Plagiarism Conference\" target=\"_blank\" class=\"extlink\">3rd International Plagiarism Conference</a>.</li>\n<li>Some blogs running older versions of WordPress <a href=\"http://ocaoimh.ie/2008/06/08/did-your-wordpress-site-get-hacked/\" title=\"Holy Shmoly!: Did your WordPress site get hacked?\" target=\"_blank\" class=\"extlink\">were hacked</a>. Peter offers a way to <a href=\"http://www.theblog.ca/file-change-notifications\" title=\"Peter\'s Useless Crap: File change notifications for your WordPress blog on Linux\" target=\"_blank\" class=\"extlink\">have your site notify you</a> when things are changed.</li>\n<li>Support for the popular plugin All in One SEO Pack was <a href=\"http://weblogtoolscollection.com/archives/2008/06/17/all-in-one-seo-lives-on/\" title=\"Weblog Tools Collection: All In One SEO Lives On\" target=\"_blank\" class=\"extlink\">dropped by its author</a>, only to be picked up a few days later by a new author.</li>\n<li><a href=\"http://www.jeffro2pt0.com/ready-to-ride-the-crazy-horse\" title=\"Jeffro2pt0: Ready to ride the Crazy Horse\" target=\"_blank\" class=\"extlink\">Crazy Horse</a> is a <a href=\"http://svn.automattic.com/wordpress/branches/crazyhorse/\" title=\"Crazy Horse on subversion\" target=\"_blank\" class=\"extlink\">development branch</a> of WordPress to further the admin overhaul begun by WordPress 2.5. (<a href=\"http://ma.tt/dropbox/2008/06/wordpress-prototype-1.1.pdf\" title=\"Crazy Horse prototype PDF\" target=\"_blank\" class=\"extlink\">PDF document</a>)</li>\n<li>Automattic Dev Andy Skelton introduced <a href=\"http://andy.wordpress.com/2008/06/22/batcache-for-wordpress/\" title=\"Andy Skelton: Batcache for WordPress\" target=\"_blank\" class=\"extlink\">Batcache</a>, a non file-based caching solution based on the one WordPress.com uses.</li>\n<li><a href=\"http://codex.wordpress.org/HyperDB\" title=\"WordPress Codex: HyperDB\" target=\"_blank\" class=\"extlink\">HyperDB</a> <a href=\"http://comox.textdrive.com/pipermail/hyperdb/2008-June/000093.html\" title=\"HyperDB update\" target=\"_blank\" class=\"extlink\">updated for compatibility</a> with WordPress trunk at revision 8094.</li>\n<li>Get well soon, <a href=\"http://lorelle.wordpress.com/\" title=\"Lorelle on WordPress\" target=\"_blank\" class=\"extlink\">Lorelle</a>!</li>\n<li>Plugin: <a href=\"http://dev.wp-plugins.org/wiki/AuthorHighlight\" title=\"Author Highlight plugin\" target=\"_blank\" class=\"extlink\">Author Highlight</a> allows you to style the comments made by you or other authors differently than regular commenters.</li>\n<li>Plugin: <a href=\"http://semperfiwebdesign.com/plugins/wp-security-scan/\" title=\"WP Security Scan\" target=\"_blank\" class=\"extlink\">WP Security Scan </a>scans your WordPress installation for security vulnerabilities and suggests corrective actions.</li>\n</ol>\";}i:9;a:7:{s:5:\"title\";s:44:\"Weblog Tools Collection: For The Love Of 2.3\";s:4:\"guid\";s:40:\"http://weblogtoolscollection.com/?p=3685\";s:4:\"link\";s:72:\"http://weblogtoolscollection.com/archives/2008/06/26/for-the-love-of-23/\";s:11:\"description\";s:1244:\"<p>James Dimick, author of the <a href=\"http://www.jamesdimick.com/creations/easy-admin-color-schemes/\">Easy Admin Color Schemes</a> plugin for WordPress has released a color scheme based exclusively on WordPress 2.3. Now, when you install this plugin, the default color scheme selected is called &#8220;<strong>For the Love of 2.3</strong>&#8221; which will take the new WordPress and make it look like the old. Something many WordPressers have wanted since the release of 2.5. However, James mentions that:</p>\n<blockquote><p>It isn’t a completely faithful recreation but I think it’s pretty darn close.</p></blockquote>\n<p><a href=\"http://weblogtoolscollection.com/b2-img/2008/06/for-the-love-of-23-big.gif\"><img src=\"http://weblogtoolscollection.com/b2-img/2008/06/for-the-love-of-23-big-262x300.gif\" alt=\"2.3 Old Style Color Scheme\" title=\"2.3 Old Style Color Scheme\" class=\"alignnone size-medium wp-image-3686 aligncenter\" height=\"300\" width=\"262\" /></a></p>\n<p>The scheme was created purely from CSS with no modifications to the core code. So while you might not be able to make WordPress 2.5 look similar to the way WordPress 2.3 did in terms of the user interface, you can at least slap on old faithful for that nostalgic look.</p>\";s:7:\"pubdate\";s:31:\"Thu, 26 Jun 2008 11:58:06 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:13:\"Jeff Chandler\";}s:7:\"summary\";s:1244:\"<p>James Dimick, author of the <a href=\"http://www.jamesdimick.com/creations/easy-admin-color-schemes/\">Easy Admin Color Schemes</a> plugin for WordPress has released a color scheme based exclusively on WordPress 2.3. Now, when you install this plugin, the default color scheme selected is called &#8220;<strong>For the Love of 2.3</strong>&#8221; which will take the new WordPress and make it look like the old. Something many WordPressers have wanted since the release of 2.5. However, James mentions that:</p>\n<blockquote><p>It isn’t a completely faithful recreation but I think it’s pretty darn close.</p></blockquote>\n<p><a href=\"http://weblogtoolscollection.com/b2-img/2008/06/for-the-love-of-23-big.gif\"><img src=\"http://weblogtoolscollection.com/b2-img/2008/06/for-the-love-of-23-big-262x300.gif\" alt=\"2.3 Old Style Color Scheme\" title=\"2.3 Old Style Color Scheme\" class=\"alignnone size-medium wp-image-3686 aligncenter\" height=\"300\" width=\"262\" /></a></p>\n<p>The scheme was created purely from CSS with no modifications to the core code. So while you might not be able to make WordPress 2.5 look similar to the way WordPress 2.3 did in terms of the user interface, you can at least slap on old faithful for that nostalgic look.</p>\";}i:10;a:7:{s:5:\"title\";s:46:\"Gravatar: DotNetNuke Shows Some Gravatar Love!\";s:4:\"guid\";s:35:\"http://gravatar.wordpress.com/?p=58\";s:4:\"link\";s:72:\"http://blog.gravatar.com/2008/06/25/dotnetnuke-shows-some-gravatar-love/\";s:11:\"description\";s:1714:\"<div class=\"snap_preview\"><br /><p>Seems the guys working on DotNetNuke have <a title=\"DotNetNuke Gravatar Support\" href=\"http://www.dotnetnuke.com/Default.aspx?tabid=825&amp;EntryID=1868\">added support for Gravatar</a>!  Rip Rowan brought this to our attention in a recent comment, and we&#8217;re looking forward to helping to provide avatars for all those DNN sites.  Keep up the good work guys!</p>\n<img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/categories/gravatar.wordpress.com/58/\" /> <img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/tags/gravatar.wordpress.com/58/\" /> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gocomments/gravatar.wordpress.com/58/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/comments/gravatar.wordpress.com/58/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godelicious/gravatar.wordpress.com/58/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/delicious/gravatar.wordpress.com/58/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gostumble/gravatar.wordpress.com/58/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/stumble/gravatar.wordpress.com/58/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godigg/gravatar.wordpress.com/58/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/digg/gravatar.wordpress.com/58/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/goreddit/gravatar.wordpress.com/58/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/reddit/gravatar.wordpress.com/58/\" /></a> <img alt=\"\" border=\"0\" src=\"http://stats.wordpress.com/b.gif?host=blog.gravatar.com&blog=1886259&post=58&subd=gravatar&ref=&feed=1\" /></div>\";s:7:\"pubdate\";s:31:\"Wed, 25 Jun 2008 17:55:59 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:11:\"apokalyptik\";}s:7:\"summary\";s:1714:\"<div class=\"snap_preview\"><br /><p>Seems the guys working on DotNetNuke have <a title=\"DotNetNuke Gravatar Support\" href=\"http://www.dotnetnuke.com/Default.aspx?tabid=825&amp;EntryID=1868\">added support for Gravatar</a>!  Rip Rowan brought this to our attention in a recent comment, and we&#8217;re looking forward to helping to provide avatars for all those DNN sites.  Keep up the good work guys!</p>\n<img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/categories/gravatar.wordpress.com/58/\" /> <img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/tags/gravatar.wordpress.com/58/\" /> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gocomments/gravatar.wordpress.com/58/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/comments/gravatar.wordpress.com/58/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godelicious/gravatar.wordpress.com/58/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/delicious/gravatar.wordpress.com/58/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gostumble/gravatar.wordpress.com/58/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/stumble/gravatar.wordpress.com/58/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godigg/gravatar.wordpress.com/58/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/digg/gravatar.wordpress.com/58/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/goreddit/gravatar.wordpress.com/58/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/reddit/gravatar.wordpress.com/58/\" /></a> <img alt=\"\" border=\"0\" src=\"http://stats.wordpress.com/b.gif?host=blog.gravatar.com&blog=1886259&post=58&subd=gravatar&ref=&feed=1\" /></div>\";}i:11;a:7:{s:5:\"title\";s:48:\"Weblog Tools Collection: How Much Would You Pay?\";s:4:\"guid\";s:40:\"http://weblogtoolscollection.com/?p=3617\";s:4:\"link\";s:76:\"http://weblogtoolscollection.com/archives/2008/06/25/how-much-would-you-pay/\";s:11:\"description\";s:2031:\"<p>So how much would you pay for an installation of WordPress? This question was raised by Damien Mulley of mully.net. In his post (<a title=\"http://www.mulley.net/2008/06/04/wanted-people-to-install-wordpress-without-robbing-others-blind/\" href=\"http://www.mulley.net/2008/06/04/wanted-people-to-install-wordpress-without-robbing-others-blind/\" target=\"_blank\">Wanted: People to install WordPress without robbing others blind</a>), Damien mentions that he is building a list of people with reasonable prices who will perform default WordPress installations. According to Damien, there are a few people who are charging anywhere from €550 to €1500 (Euros). This equates to $857.00 to  $2,338.33. I can&#8217;t imagine anyone in their right mind who would pay that amount to have WordPress installed for them. However, when I stop to think about those who are brand new to blogging or to WordPress in general, those are the ones that when offered these prices, may believe this to be normal when in fact, it&#8217;s not.</p>\n<p>The conversation about this topic was handled pretty well within the comments. Most people commented that they would charge $50.00. Others said they wouldn&#8217;t charge a dime while others wouldn&#8217;t mind having a beer purchased for them.</p>\n<p>You know, if someone wants you to install WordPress and turn it into a CMS or do some sort of customization that doesn&#8217;t exist out of the box, I can understand the price. However, for default installations, there is no reason why the install should be over $50.00 maximum.</p>\n<p>For those of you wanting to install WordPress for free, be sure to read this article in the Codex entitled, <a title=\"http://codex.wordpress.org/Installing_WordPress#WordPress_Installed_for_Free\" href=\"http://codex.wordpress.org/Installing_WordPress#WordPress_Installed_for_Free\" target=\"_blank\">Installing WordPress for Free</a>. Or read the other aspects of the article to figure out how you can install WordPress on your own without emptying your pockets.</p>\";s:7:\"pubdate\";s:31:\"Wed, 25 Jun 2008 14:41:31 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:13:\"Jeff Chandler\";}s:7:\"summary\";s:2031:\"<p>So how much would you pay for an installation of WordPress? This question was raised by Damien Mulley of mully.net. In his post (<a title=\"http://www.mulley.net/2008/06/04/wanted-people-to-install-wordpress-without-robbing-others-blind/\" href=\"http://www.mulley.net/2008/06/04/wanted-people-to-install-wordpress-without-robbing-others-blind/\" target=\"_blank\">Wanted: People to install WordPress without robbing others blind</a>), Damien mentions that he is building a list of people with reasonable prices who will perform default WordPress installations. According to Damien, there are a few people who are charging anywhere from €550 to €1500 (Euros). This equates to $857.00 to  $2,338.33. I can&#8217;t imagine anyone in their right mind who would pay that amount to have WordPress installed for them. However, when I stop to think about those who are brand new to blogging or to WordPress in general, those are the ones that when offered these prices, may believe this to be normal when in fact, it&#8217;s not.</p>\n<p>The conversation about this topic was handled pretty well within the comments. Most people commented that they would charge $50.00. Others said they wouldn&#8217;t charge a dime while others wouldn&#8217;t mind having a beer purchased for them.</p>\n<p>You know, if someone wants you to install WordPress and turn it into a CMS or do some sort of customization that doesn&#8217;t exist out of the box, I can understand the price. However, for default installations, there is no reason why the install should be over $50.00 maximum.</p>\n<p>For those of you wanting to install WordPress for free, be sure to read this article in the Codex entitled, <a title=\"http://codex.wordpress.org/Installing_WordPress#WordPress_Installed_for_Free\" href=\"http://codex.wordpress.org/Installing_WordPress#WordPress_Installed_for_Free\" target=\"_blank\">Installing WordPress for Free</a>. Or read the other aspects of the article to figure out how you can install WordPress on your own without emptying your pockets.</p>\";}i:12;a:7:{s:5:\"title\";s:42:\"Mike Little: Wordcamp UK - Change of Venue\";s:4:\"guid\";s:76:\"http://zed1.com/journalized/archives/2008/06/24/wordcamp-uk-change-of-venue/\";s:4:\"link\";s:76:\"http://zed1.com/journalized/archives/2008/06/24/wordcamp-uk-change-of-venue/\";s:11:\"description\";s:771:\"<p>After some last minute changes (and upsetting a few people) there is now a new venue for the <a href=\"http://uk.wordcamp.org/\">WordCamp UK 2008</a> event.</p>\n<p>The new venue is The Studio, it is still in Birmingham, and in fact quite close to the previously planned venue. The Studio has a <a href=\"http://www.studiovenues.co.uk/gettinghere.htm\">directions map</a> and their <a href=\"http://www.orangestudio.co.uk/\">main web site</a> lists all the features. The event will be hosted in  &#8216;Atrium Bar&#8217; and the adjoining &#8216;Open&#8217; room. </p>\n<p>Someone kindly set up a <a href=\"http://tinyurl.com/4km3p2\">Google Maps directions page</a>.</p>\n<p>For full details go to the <a href=\"http://wiki.wordcampuk.tonyscott.org.uk/\">WordCamp UK wiki</a>.</p>\";s:7:\"pubdate\";s:31:\"Tue, 24 Jun 2008 20:44:34 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"mike\";}s:7:\"summary\";s:771:\"<p>After some last minute changes (and upsetting a few people) there is now a new venue for the <a href=\"http://uk.wordcamp.org/\">WordCamp UK 2008</a> event.</p>\n<p>The new venue is The Studio, it is still in Birmingham, and in fact quite close to the previously planned venue. The Studio has a <a href=\"http://www.studiovenues.co.uk/gettinghere.htm\">directions map</a> and their <a href=\"http://www.orangestudio.co.uk/\">main web site</a> lists all the features. The event will be hosted in  &#8216;Atrium Bar&#8217; and the adjoining &#8216;Open&#8217; room. </p>\n<p>Someone kindly set up a <a href=\"http://tinyurl.com/4km3p2\">Google Maps directions page</a>.</p>\n<p>For full details go to the <a href=\"http://wiki.wordcampuk.tonyscott.org.uk/\">WordCamp UK wiki</a>.</p>\";}i:13;a:7:{s:5:\"title\";s:37:\"Dougal Campbell: WordPress 2.6 Beta 1\";s:4:\"guid\";s:32:\"http://dougal.gunters.org/?p=983\";s:4:\"link\";s:61:\"http://dougal.gunters.org/blog/2008/06/24/wordpress-26-beta-1\";s:11:\"description\";s:3660:\"<p>Last night Ryan Boren announced the release of <a href=\"http://boren.nu/archives/2008/06/23/wordpress-26-beta-1/\">WordPress 2.6 Beta 1</a>. At first glance, you probably wouldn&#8217;t notice much difference between versions 2.5 and 2.6. The two biggest new features are <a href=\"http://mdawaffe.wordpress.com/2008/05/22/post-revisions-in-wordpress-26/\">Post Revisions</a>, which saves a history of edits made to each post, and support for <a href=\"http://gears.google.com/\">Gears</a>, which greatly speeds up working with the admin screens.</p>\n<p>Other changes of note:</p>\n<ul>\n<li>Support for the <acronym title=\"eXtensible Markup Language\"><span class=\"caps\">XML</span></acronym>-<acronym title=\"Remote Procedure Call\"><span class=\"caps\">RPC</span></acronym> and Atom client APIs is turned off by default now. If you use an external client for posting, you&#8217;ll need to specifically turn these features on, under <code>Settings/Writing</code>.</li>\n<li>Impoved support for SSL access to the admin screens.</li>\n<li>When you are selecting themes, you now get to view a preview of how your site looks with the new theme before commiting to the change.</li>\n<li>The return of the &#8220;Press This&#8221; bookmarklet (find it in the sidebar of the <code>Write</code> page). Drag the link to your browser&#8217;s bookmarks toolbar, and use it to quickly post about sites you visit.</li>\n<li>More avatar options. The <a title=\"Global Avatars\" href=\"http://gravatar.com/\">Gravatar</a> service now supports displaying <a href=\"http://en.blog.wordpress.com/2008/05/04/default-avatars/\">MonsterID, Wavatar, or Identicon icons</a> in place of the default icon for those who don&#8217;t have a personalized Gravatar. These options are selectable under <code>Settings/Discussion</code>. My <a title=\"Add Gravatars to WordPress themes that don\'t have built-in support\" href=\"http://wordpress.org/extend/plugins/easygravatars/\">Easy Gravatars plugin</a> can support this as well, though I&#8217;ll probably update it to use the new built-in settings, rather than its own. I&#8217;ll write that up later.</li>\n<li>Improved plugin management. Active and inactive plugins are now listed separately. You now have the abiliity to &#8220;bulk activate&#8221; or &#8220;bulk deactivate&#8221; plugins. You can also delete unused plugins at the click of a button. (Now all we need is the ability to search and install plugins from the  Extend directory right from our admin! Maybe in 2.7?)</li>\n</ul>\n<p>And there are other assorted improvements, such as updates to the latest <a href=\"http://jquery.com/\">jQuery</a> and <a href=\"http://ui.jquery.com/\">jQuery UI</a> libraries, updates to the latest TinyMCE editor, improvements to Page and Category management, and as always, general performance enhancements.</p>\n<p>It <em>is</em> a beta, so there are still quirks being hammered out (personally, I&#8217;ve observed issues with the theme preview, and with deleting plugins). So don&#8217;t use this on a &#8220;critical&#8221; production web site yet. I&#8217;m running it here, but obviously, my personal blog isn&#8217;t critical, and I&#8217;ve always got backups available for when I screw something up. <img src=\"http://dougal.gunters.org/wordpress/wp-includes/images/smilies/icon_smile.gif\" alt=\":)\" class=\"wp-smiley\" /> </p>\n<p><a href=\"http://wordpress.org/wordpress-2.6-beta1.zip\">Download WordPress 2.6 Beta 1</a></p>\n<p><a href=\"http://sharethis.com/item?&wp=2.6-beta1&amp;publisher=06a70a77-1fc0-46a9-81d1-6a696e6ed23f&amp;title=WordPress+2.6+Beta+1&amp;url=http%3A%2F%2Fdougal.gunters.org%2Fblog%2F2008%2F06%2F24%2Fwordpress-26-beta-1\">ShareThis</a></p>\";s:7:\"pubdate\";s:31:\"Tue, 24 Jun 2008 16:30:21 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:6:\"Dougal\";}s:7:\"summary\";s:3660:\"<p>Last night Ryan Boren announced the release of <a href=\"http://boren.nu/archives/2008/06/23/wordpress-26-beta-1/\">WordPress 2.6 Beta 1</a>. At first glance, you probably wouldn&#8217;t notice much difference between versions 2.5 and 2.6. The two biggest new features are <a href=\"http://mdawaffe.wordpress.com/2008/05/22/post-revisions-in-wordpress-26/\">Post Revisions</a>, which saves a history of edits made to each post, and support for <a href=\"http://gears.google.com/\">Gears</a>, which greatly speeds up working with the admin screens.</p>\n<p>Other changes of note:</p>\n<ul>\n<li>Support for the <acronym title=\"eXtensible Markup Language\"><span class=\"caps\">XML</span></acronym>-<acronym title=\"Remote Procedure Call\"><span class=\"caps\">RPC</span></acronym> and Atom client APIs is turned off by default now. If you use an external client for posting, you&#8217;ll need to specifically turn these features on, under <code>Settings/Writing</code>.</li>\n<li>Impoved support for SSL access to the admin screens.</li>\n<li>When you are selecting themes, you now get to view a preview of how your site looks with the new theme before commiting to the change.</li>\n<li>The return of the &#8220;Press This&#8221; bookmarklet (find it in the sidebar of the <code>Write</code> page). Drag the link to your browser&#8217;s bookmarks toolbar, and use it to quickly post about sites you visit.</li>\n<li>More avatar options. The <a title=\"Global Avatars\" href=\"http://gravatar.com/\">Gravatar</a> service now supports displaying <a href=\"http://en.blog.wordpress.com/2008/05/04/default-avatars/\">MonsterID, Wavatar, or Identicon icons</a> in place of the default icon for those who don&#8217;t have a personalized Gravatar. These options are selectable under <code>Settings/Discussion</code>. My <a title=\"Add Gravatars to WordPress themes that don\'t have built-in support\" href=\"http://wordpress.org/extend/plugins/easygravatars/\">Easy Gravatars plugin</a> can support this as well, though I&#8217;ll probably update it to use the new built-in settings, rather than its own. I&#8217;ll write that up later.</li>\n<li>Improved plugin management. Active and inactive plugins are now listed separately. You now have the abiliity to &#8220;bulk activate&#8221; or &#8220;bulk deactivate&#8221; plugins. You can also delete unused plugins at the click of a button. (Now all we need is the ability to search and install plugins from the  Extend directory right from our admin! Maybe in 2.7?)</li>\n</ul>\n<p>And there are other assorted improvements, such as updates to the latest <a href=\"http://jquery.com/\">jQuery</a> and <a href=\"http://ui.jquery.com/\">jQuery UI</a> libraries, updates to the latest TinyMCE editor, improvements to Page and Category management, and as always, general performance enhancements.</p>\n<p>It <em>is</em> a beta, so there are still quirks being hammered out (personally, I&#8217;ve observed issues with the theme preview, and with deleting plugins). So don&#8217;t use this on a &#8220;critical&#8221; production web site yet. I&#8217;m running it here, but obviously, my personal blog isn&#8217;t critical, and I&#8217;ve always got backups available for when I screw something up. <img src=\"http://dougal.gunters.org/wordpress/wp-includes/images/smilies/icon_smile.gif\" alt=\":)\" class=\"wp-smiley\" /> </p>\n<p><a href=\"http://wordpress.org/wordpress-2.6-beta1.zip\">Download WordPress 2.6 Beta 1</a></p>\n<p><a href=\"http://sharethis.com/item?&wp=2.6-beta1&amp;publisher=06a70a77-1fc0-46a9-81d1-6a696e6ed23f&amp;title=WordPress+2.6+Beta+1&amp;url=http%3A%2F%2Fdougal.gunters.org%2Fblog%2F2008%2F06%2F24%2Fwordpress-26-beta-1\">ShareThis</a></p>\";}i:14;a:7:{s:5:\"title\";s:65:\"Weblog Tools Collection: Uber unique Prize for Plugin Competition\";s:4:\"guid\";s:40:\"http://weblogtoolscollection.com/?p=3683\";s:4:\"link\";s:94:\"http://weblogtoolscollection.com/archives/2008/06/24/uber-unique-prize-for-plugin-competition/\";s:11:\"description\";s:749:\"<p><a href=\"http://www.kymhuynh.com/\">Kym Huynh</a> is a songwriter and is writing a song about WordPress (just for fun I have been told) and has offered to write a song and record it professionally in a studio for the winner of the WordPress Plugin Competition! What an amazing prize! I love it when WordPress Samaritans offer their thanks in their own way. It has been added to the <a href=\"http://weblogtoolscollection.com/plugin-competition-prizes/\">growing list of prizes for the competition</a>. Thank you Kym!</p>\n<p>For those that have not noticed, we have<a href=\"http://weblogtoolscollection.com/pluginblog/\"> eleven plugins entered/announced for the plugin competition</a> already, with many more in the works. Have you signed up yet?</p>\";s:7:\"pubdate\";s:31:\"Tue, 24 Jun 2008 14:57:20 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:10:\"Mark Ghosh\";}s:7:\"summary\";s:749:\"<p><a href=\"http://www.kymhuynh.com/\">Kym Huynh</a> is a songwriter and is writing a song about WordPress (just for fun I have been told) and has offered to write a song and record it professionally in a studio for the winner of the WordPress Plugin Competition! What an amazing prize! I love it when WordPress Samaritans offer their thanks in their own way. It has been added to the <a href=\"http://weblogtoolscollection.com/plugin-competition-prizes/\">growing list of prizes for the competition</a>. Thank you Kym!</p>\n<p>For those that have not noticed, we have<a href=\"http://weblogtoolscollection.com/pluginblog/\"> eleven plugins entered/announced for the plugin competition</a> already, with many more in the works. Have you signed up yet?</p>\";}i:15;a:7:{s:5:\"title\";s:32:\"Ryan Boren: WordPress 2.6 Beta 1\";s:4:\"guid\";s:23:\"http://boren.nu/?p=1554\";s:4:\"link\";s:56:\"http://boren.nu/archives/2008/06/23/wordpress-26-beta-1/\";s:11:\"description\";s:2192:\"<p>The features we slated for 2.6 have been in the repository for awhile now, so we&#8217;re going to start our beta cycle in preparation for a mid-July release.  <a href=\"http://wordpress.org/wordpress-2.6-beta1.zip\">Get Beta 1 here</a>.  2.6 isn&#8217;t a big release like 2.5 was, but it has a few new features for you.</p>\n<ul>\n<li><a href=\"http://mdawaffe.wordpress.com/2008/05/22/post-revisions-in-wordpress-26/\">Post Revisioning</a></li>\n<li>A &#8220;Press This&#8221; bookmarklet for tumblelog style quick posting</li>\n<li><a href=\"http://www.geniosity.co.za/musings/wordpress/wordpress-26-and-google-gears/\">Caching of static files</a> with <a href=\"http://gears.google.com/\">Gears</a> for faster Admin page loads</li>\n<li>A new and improved image editing dialog that offers lots of control over the images in your posts</li>\n<li>Theme previewing as seen on <a href=\"http://wordpress.com/\">WordPress.com</a></li>\n<li>Built-in word counting in the post editor</li>\n<li>The ability to <a href=\"http://westi.wordpress.com/2008/06/20/making-the-default-install-more-secure/\">disable remote publishing</a> for the security conscious</li>\n<li>XML-RPC API for changing blog options</li>\n<li>Better SSL support for the Admin</li>\n<li>Hierarchy-aware paging in the Manage Pages and Manage Categories admin pages.</li>\n<li>The ability to relocate your content directory</li>\n<li>The ability to move wp-config.php out of your web root</li>\n<li>Drag-and-drop sortable galleries</li>\n<li><a href=\"http://en.blog.wordpress.com/2008/05/04/default-avatars/\">Customizable default avatars</a></li>\n<li>Bulk delete, activate, and deactivate for plugins</li>\n<li>Check box range selection with shift-click</li>\n<li><a href=\"http://tinymce.moxiecode.com/punbb/viewtopic.php?id=12007\">TinyMCE 3.1.0.1</a> with lots of bug fixes</li>\n<li><a href=\"http://jquery.com/blog/2008/06/04/jquery-126-events-100-faster/\">jQuery 1.2.6</a> with some nice performance improvements</li>\n<li><a href=\"http://jquery.com/blog/2008/06/09/jquery-ui-v15-released-focus-on-consistent-api-and-effects/\">jQuery UI 1.5</a></li>\n<li>Lots of bug fixes and performance improvements</li>\n</ul>\n<p>Enjoy, and thanks for testing.</p>\";s:7:\"pubdate\";s:31:\"Tue, 24 Jun 2008 05:04:09 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Ryan\";}s:7:\"summary\";s:2192:\"<p>The features we slated for 2.6 have been in the repository for awhile now, so we&#8217;re going to start our beta cycle in preparation for a mid-July release.  <a href=\"http://wordpress.org/wordpress-2.6-beta1.zip\">Get Beta 1 here</a>.  2.6 isn&#8217;t a big release like 2.5 was, but it has a few new features for you.</p>\n<ul>\n<li><a href=\"http://mdawaffe.wordpress.com/2008/05/22/post-revisions-in-wordpress-26/\">Post Revisioning</a></li>\n<li>A &#8220;Press This&#8221; bookmarklet for tumblelog style quick posting</li>\n<li><a href=\"http://www.geniosity.co.za/musings/wordpress/wordpress-26-and-google-gears/\">Caching of static files</a> with <a href=\"http://gears.google.com/\">Gears</a> for faster Admin page loads</li>\n<li>A new and improved image editing dialog that offers lots of control over the images in your posts</li>\n<li>Theme previewing as seen on <a href=\"http://wordpress.com/\">WordPress.com</a></li>\n<li>Built-in word counting in the post editor</li>\n<li>The ability to <a href=\"http://westi.wordpress.com/2008/06/20/making-the-default-install-more-secure/\">disable remote publishing</a> for the security conscious</li>\n<li>XML-RPC API for changing blog options</li>\n<li>Better SSL support for the Admin</li>\n<li>Hierarchy-aware paging in the Manage Pages and Manage Categories admin pages.</li>\n<li>The ability to relocate your content directory</li>\n<li>The ability to move wp-config.php out of your web root</li>\n<li>Drag-and-drop sortable galleries</li>\n<li><a href=\"http://en.blog.wordpress.com/2008/05/04/default-avatars/\">Customizable default avatars</a></li>\n<li>Bulk delete, activate, and deactivate for plugins</li>\n<li>Check box range selection with shift-click</li>\n<li><a href=\"http://tinymce.moxiecode.com/punbb/viewtopic.php?id=12007\">TinyMCE 3.1.0.1</a> with lots of bug fixes</li>\n<li><a href=\"http://jquery.com/blog/2008/06/04/jquery-126-events-100-faster/\">jQuery 1.2.6</a> with some nice performance improvements</li>\n<li><a href=\"http://jquery.com/blog/2008/06/09/jquery-ui-v15-released-focus-on-consistent-api-and-effects/\">jQuery UI 1.5</a></li>\n<li>Lots of bug fixes and performance improvements</li>\n</ul>\n<p>Enjoy, and thanks for testing.</p>\";}i:16;a:7:{s:5:\"title\";s:59:\"Weblog Tools Collection: WordPress Plugin releases for 6/22\";s:4:\"guid\";s:89:\"http://weblogtoolscollection.com/archives/2008/06/24/wordpress-plugin-releases-for-622-2/\";s:4:\"link\";s:89:\"http://weblogtoolscollection.com/archives/2008/06/24/wordpress-plugin-releases-for-622-2/\";s:11:\"description\";s:1886:\"<p><a href=\"http://www.preblogging.com/increase-sociability/\" target=\"_blank\"><strong>Increase Sociability</strong></a></p>\n<p>Increase Sociability shows customized messages to users who visit your site through StumbleUpon, Digg. The messages enhances your chances to gain better traffic from social networking sites.</p>\n<p><a href=\"http://blog.clearskys.net/2008/06/17/hoard-hunter-fundraising-using-wordpress/\" target=\"_blank\"><strong>Hoard Hunter</strong></a></p>\n<p>Hoard Hunter is an online treasure map game designed and developed for people who would like an easy (and fun) method of fund raising online.</p>\n<p><a href=\"http://www.themiak.com/wp-plugins/nofollow-links-in-posts/\" target=\"_blank\"><strong>Nofollow Links in Posts</strong></a></p>\n<p>Nofollow Links in Posts plugin enables you to add the nofollow attribute into links in a post within a specified category.</p>\n<p><a href=\"http://epicalex.com/newspaper-style-posts/\" target=\"_blank\"><strong>Newspaper Style Posts</strong></a></p>\n<p>Newspaper Style Posts is a simple plugin that splits your post into two columns, as you find in newspaper articles.</p>\n<p><a href=\"http://www.prelovac.com/vladimir/wordpress-plugins/seo-smart-links\" target=\"_blank\"><strong>SEO Smart Links</strong></a></p>\n<p>SEO Smart Links can automatically link keywords and phrases in your posts and comments with corresponding posts, pages, categories and tags.</p>\n<p><a href=\"http://techblissonline.com/platinum-seo-pack/\" target=\"_blank\"><strong>Platinum SEO</strong></a></p>\n<p>Platinum SEO offers all the features of All in One SEO plugin plus more. You can import your settings from All in One SEO to Platinum SEO.</p>\n<p><a href=\"http://www.prelovac.com/vladimir/wordpress-plugins/category-search\" target=\"_blank\"><strong>Category Search</strong></a></p>\n<p>Category Search allows you to add category names to your search results.</p>\";s:7:\"pubdate\";s:31:\"Tue, 24 Jun 2008 04:22:36 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:12:\"Keith Dsouza\";}s:7:\"summary\";s:1886:\"<p><a href=\"http://www.preblogging.com/increase-sociability/\" target=\"_blank\"><strong>Increase Sociability</strong></a></p>\n<p>Increase Sociability shows customized messages to users who visit your site through StumbleUpon, Digg. The messages enhances your chances to gain better traffic from social networking sites.</p>\n<p><a href=\"http://blog.clearskys.net/2008/06/17/hoard-hunter-fundraising-using-wordpress/\" target=\"_blank\"><strong>Hoard Hunter</strong></a></p>\n<p>Hoard Hunter is an online treasure map game designed and developed for people who would like an easy (and fun) method of fund raising online.</p>\n<p><a href=\"http://www.themiak.com/wp-plugins/nofollow-links-in-posts/\" target=\"_blank\"><strong>Nofollow Links in Posts</strong></a></p>\n<p>Nofollow Links in Posts plugin enables you to add the nofollow attribute into links in a post within a specified category.</p>\n<p><a href=\"http://epicalex.com/newspaper-style-posts/\" target=\"_blank\"><strong>Newspaper Style Posts</strong></a></p>\n<p>Newspaper Style Posts is a simple plugin that splits your post into two columns, as you find in newspaper articles.</p>\n<p><a href=\"http://www.prelovac.com/vladimir/wordpress-plugins/seo-smart-links\" target=\"_blank\"><strong>SEO Smart Links</strong></a></p>\n<p>SEO Smart Links can automatically link keywords and phrases in your posts and comments with corresponding posts, pages, categories and tags.</p>\n<p><a href=\"http://techblissonline.com/platinum-seo-pack/\" target=\"_blank\"><strong>Platinum SEO</strong></a></p>\n<p>Platinum SEO offers all the features of All in One SEO plugin plus more. You can import your settings from All in One SEO to Platinum SEO.</p>\n<p><a href=\"http://www.prelovac.com/vladimir/wordpress-plugins/category-search\" target=\"_blank\"><strong>Category Search</strong></a></p>\n<p>Category Search allows you to add category names to your search results.</p>\";}i:17;a:7:{s:5:\"title\";s:50:\"Weblog Tools Collection: Comment License Made Easy\";s:4:\"guid\";s:40:\"http://weblogtoolscollection.com/?p=3680\";s:4:\"link\";s:79:\"http://weblogtoolscollection.com/archives/2008/06/23/comment-license-made-easy/\";s:11:\"description\";s:2354:\"<p>Not too long ago, I <a title=\"http://weblogtoolscollection.com/archives/2008/06/14/the-ownership-of-comments/\" href=\"http://weblogtoolscollection.com/archives/2008/06/14/the-ownership-of-comments/\" target=\"_self\">published a post</a> which dove into the controversial subject of comment ownership and I must say, you guys and gals came up with quite a bit of discussion points. In that post, here is how I described my position on the matter:</p>\n<blockquote><p>I don’t believe a commenting bill of rights needs to be created in which all blogs should follow. However, I do think that each blogger should create and make publicly accessible a commenting policy. This policy should clearly explain what you as the blog author will do with comments posted on your site, who retains ownership of those comments and explain circumstances which would require you to edit an end user’s comment.</p></blockquote>\n<p>For those of you that agree, I&#8217;d like to draw your attention to Alex King&#8217;s <a title=\"http://alexking.org/projects/wordpress\" href=\"http://alexking.org/projects/wordpress\" target=\"_blank\">Comment License</a> WordPress plugin. This plugin makes it as easy as 1 2 3 to add a commenting policy to your blog.</p>\n<p>After downloading, uploading, then activating the plugin, browse to <strong>ADMIN-SETTINGS-COMMENT LICENSE</strong>. You should see something similar to this:</p>\n<p><a href=\"http://weblogtoolscollection.com/b2-img/2008/06/commentlicense.png\"><img src=\"http://weblogtoolscollection.com/b2-img/2008/06/commentlicense.png\" alt=\"How To Configure Comment License\" title=\"How To Configure Comment License\" class=\"alignnone size-full wp-image-3681 aligncenter\" height=\"276\" width=\"389\" /></a></p>\n<p>Click anywhere inside of the text field and then create your own comment license. Once you click on the <strong>UPDATE COMMENT LICENSE</strong> button, the text will be saved and displayed just above your commenting form on your blog.</p>\n<p>If you notice that the license is not being displayed on your theme, it may be because your comments.php template is missing &#8216;<strong>comment_form</strong>&#8216;. In order to add this functionality, read the readme.txt file included with the plugin download package as that contains the line of code that will need to be placed within your comments.php template file.</p>\";s:7:\"pubdate\";s:31:\"Mon, 23 Jun 2008 11:15:44 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:13:\"Jeff Chandler\";}s:7:\"summary\";s:2354:\"<p>Not too long ago, I <a title=\"http://weblogtoolscollection.com/archives/2008/06/14/the-ownership-of-comments/\" href=\"http://weblogtoolscollection.com/archives/2008/06/14/the-ownership-of-comments/\" target=\"_self\">published a post</a> which dove into the controversial subject of comment ownership and I must say, you guys and gals came up with quite a bit of discussion points. In that post, here is how I described my position on the matter:</p>\n<blockquote><p>I don’t believe a commenting bill of rights needs to be created in which all blogs should follow. However, I do think that each blogger should create and make publicly accessible a commenting policy. This policy should clearly explain what you as the blog author will do with comments posted on your site, who retains ownership of those comments and explain circumstances which would require you to edit an end user’s comment.</p></blockquote>\n<p>For those of you that agree, I&#8217;d like to draw your attention to Alex King&#8217;s <a title=\"http://alexking.org/projects/wordpress\" href=\"http://alexking.org/projects/wordpress\" target=\"_blank\">Comment License</a> WordPress plugin. This plugin makes it as easy as 1 2 3 to add a commenting policy to your blog.</p>\n<p>After downloading, uploading, then activating the plugin, browse to <strong>ADMIN-SETTINGS-COMMENT LICENSE</strong>. You should see something similar to this:</p>\n<p><a href=\"http://weblogtoolscollection.com/b2-img/2008/06/commentlicense.png\"><img src=\"http://weblogtoolscollection.com/b2-img/2008/06/commentlicense.png\" alt=\"How To Configure Comment License\" title=\"How To Configure Comment License\" class=\"alignnone size-full wp-image-3681 aligncenter\" height=\"276\" width=\"389\" /></a></p>\n<p>Click anywhere inside of the text field and then create your own comment license. Once you click on the <strong>UPDATE COMMENT LICENSE</strong> button, the text will be saved and displayed just above your commenting form on your blog.</p>\n<p>If you notice that the license is not being displayed on your theme, it may be because your comments.php template is missing &#8216;<strong>comment_form</strong>&#8216;. In order to add this functionality, read the readme.txt file included with the plugin download package as that contains the line of code that will need to be placed within your comments.php template file.</p>\";}i:18;a:7:{s:5:\"title\";s:27:\"Matt: Top Gear on WordPress\";s:4:\"guid\";s:20:\"http://ma.tt/?p=6558\";s:4:\"link\";s:43:\"http://ma.tt/2008/06/top-gear-on-wordpress/\";s:11:\"description\";s:141:\"<p>For the Top Gear fans out there, the <a href=\"http://blogs.topgear.com/\">new Top Gear blog is the latest WordPress.com VIP</a>. Vroom!</p>\";s:7:\"pubdate\";s:31:\"Mon, 23 Jun 2008 02:06:03 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:7:\"summary\";s:141:\"<p>For the Top Gear fans out there, the <a href=\"http://blogs.topgear.com/\">new Top Gear blog is the latest WordPress.com VIP</a>. Vroom!</p>\";}i:19;a:7:{s:5:\"title\";s:36:\"Andy Skelton: Batcache for WordPress\";s:4:\"guid\";s:32:\"http://andy.wordpress.com/?p=180\";s:4:\"link\";s:60:\"http://andy.wordpress.com/2008/06/22/batcache-for-wordpress/\";s:11:\"description\";s:7374:\"<div class=\"snap_preview\"><br /><p>[I meant to publicize this after a period of quiet testing and feedback but the watchdogs at WLTC <a href=\"http://weblogtoolscollection.com/archives/2008/06/22/andy-skelton-introduces-batcache/\">upended the kitten bag</a> and forced my hand. <a href=\"http://wordpress.org/extend/plugins/batcache/\">Batcache</a> comes with all the usual disclaimers. If you try it on a production server expect the moon to fall on your head.]</p>\n<p>People say WordPress can&#8217;t perform under pressure. The way most people set it up, that&#8217;s true. For those who host their blog for $7.99 a month (do they also run Vista on an 8086?) the best bet is to serve static pages rather than dynamic pages. <a href=\"http://wordpress.org/extend/plugins/wp-super-cache/\">Donncha&#8217;s WP-Super-Cache</a> does that brilliantly. I&#8217;ve seen it raise a server&#8217;s capacity for blog traffic by <em>one hundred times or more</em>. It&#8217;s a cheapskate&#8217;s dream.</p>\n<p>WP-Super-Cache is good for anyone with a single web server with a writable <code>wp-content/cache</code> directory. To them, the majority, I say use WP-Super-Cache. What about enterprises with multiple servers that don&#8217;t share disk space? If you can&#8217;t or won&#8217;t use file-based caching, I have something for you. It&#8217;s based on what WordPress.com uses. It&#8217;s Batcache.</p>\n<h3>Batcache will protect you</h3>\n<p>Batcache implements a very simplistic caching model that shields your database and web servers from traffic spikes: after a document has been requested X times in Y seconds, the document is cached for Z seconds and all new users are served the cached copy.</p>\n<p>New users are defined as anybody who hasn&#8217;t interacted with your domain&#8212;once they&#8217;ve left a comment or logged in, their cookies will ensure they get fresh pages. People arriving from Digg won&#8217;t notice that the comments are a minute or two behind but they&#8217;ll appreciate your site being up.</p>\n<p>You don&#8217;t need PHP skills to install Batcache but you do have to get Memcached working first. That can be easy or hard. We use Memcached because it&#8217;s <em>awesome</em>. Once you know how to install it you can create the same kind of distributed, persistent cache that underpin web giants like WordPress.com and Facebook.</p>\n<h3>What Batcache does</h3>\n<p>The first thing Batcache does is decide whether the visitor is eligible to receive cached documents. If their cookies don&#8217;t show evidence of previous interaction on that domain they are eligible. Next it decides whether the request is eligible for caching. For example, Batcache won&#8217;t interfere when a comment is being posted.</p>\n<p>If the visitor and the request are eligible, Batcache enters its traffic metering routine. By default it looks for URLs that receive more than two hits from unrecognized users in two minutes. When a URL&#8217;s traffic crosses that threshold, Batcache caches the document for five minutes. You can configure these numbers any way you like, or turn off traffic metering and send documents right to the cache.</p>\n<p>Once a document has been cached, it is served to eligible visitors until it expires. This is one place where Batcache is different. Most other caches delete cached documents as soon as the underlying data changes. Batcache doesn&#8217;t care if it&#8217;s serving old data because &#8220;old&#8221; is relative (and configurable).</p>\n<h3>What Batcache doesn&#8217;t do</h3>\n<p>It doesn&#8217;t guarantee a current document. I repeat this because reliable cache invalidation is a typical feature that was purposefully omitted from Batcache. There is a routine in the included plugin that tries to trigger regeneration of updated and commented posts but in some situations a document will still live in the cache until it expires. This routine will be improved over time but it is only an afterthought.</p>\n<p>Batcache doesn&#8217;t automatically know the difference between document variants. Variants exist when two requests for the same URL can yield two different documents. Common examples are user agent-dependent variants formatted for mobile devices and referrer-dependent variants with Google search terms highlighted. In these cases you MUST take extra steps to inform Batcache about variants to avoid serving a variant to the wrong audience. The source code includes examples of how to turn off caching of uncommon variants (search term highlighting) or cache common variants separately (mobile versions).</p>\n<h3>Where Batcache is going</h3>\n<p>I want to make Batcache easier to configure by adding a configuration page and storing the main settings in memcached as well as the database. This way you won&#8217;t have to deploy a code change to update the configuration. However, conditional configurations (e.g. &#8220;never cache URLs matching some pattern&#8221;) and variant detection will probably always live in PHP.</p>\n<p>I want to have Batcache serve correct headers more reliably. On some servers it can detect the headers that were sent with a newly generated page and serve them again from the cache. But when that doesn&#8217;t work you will have to take extra steps to serve certain headers. For example you must specify the Content-Encoding header in the Batcache configuration or add it to <code>php.ini</code>. I want this sort of thing to be done automatically for all server setups.</p>\n<p>I know that Batcache is not ideal for most WordPress installations. It saves us a lot of headaches and expense at WordPress.com, so maybe it can help other large installations. If you try it, I want to hear from you whether it worked and how well. I am also keen to see what new configurations and modifications you use.</p>\n<p>As always, this software is provided without claims or warrantees. It&#8217;s so experimental that it doesn&#8217;t even have a version number! Until the project grows to need its own blog, keep an eye on the <a href=\"http://dev.wp-plugins.org/browser/batcache/trunk\">Trac browser</a> for updates.</p>\n<img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/categories/andy.wordpress.com/180/\" /> <img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/tags/andy.wordpress.com/180/\" /> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gocomments/andy.wordpress.com/180/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/comments/andy.wordpress.com/180/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godelicious/andy.wordpress.com/180/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/delicious/andy.wordpress.com/180/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gostumble/andy.wordpress.com/180/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/stumble/andy.wordpress.com/180/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godigg/andy.wordpress.com/180/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/digg/andy.wordpress.com/180/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/goreddit/andy.wordpress.com/180/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/reddit/andy.wordpress.com/180/\" /></a> <img alt=\"\" border=\"0\" src=\"http://stats.wordpress.com/b.gif?host=andy.wordpress.com&blog=155&post=180&subd=andy&ref=&feed=1\" /></div>\";s:7:\"pubdate\";s:31:\"Sun, 22 Jun 2008 18:47:40 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Andy\";}s:7:\"summary\";s:7374:\"<div class=\"snap_preview\"><br /><p>[I meant to publicize this after a period of quiet testing and feedback but the watchdogs at WLTC <a href=\"http://weblogtoolscollection.com/archives/2008/06/22/andy-skelton-introduces-batcache/\">upended the kitten bag</a> and forced my hand. <a href=\"http://wordpress.org/extend/plugins/batcache/\">Batcache</a> comes with all the usual disclaimers. If you try it on a production server expect the moon to fall on your head.]</p>\n<p>People say WordPress can&#8217;t perform under pressure. The way most people set it up, that&#8217;s true. For those who host their blog for $7.99 a month (do they also run Vista on an 8086?) the best bet is to serve static pages rather than dynamic pages. <a href=\"http://wordpress.org/extend/plugins/wp-super-cache/\">Donncha&#8217;s WP-Super-Cache</a> does that brilliantly. I&#8217;ve seen it raise a server&#8217;s capacity for blog traffic by <em>one hundred times or more</em>. It&#8217;s a cheapskate&#8217;s dream.</p>\n<p>WP-Super-Cache is good for anyone with a single web server with a writable <code>wp-content/cache</code> directory. To them, the majority, I say use WP-Super-Cache. What about enterprises with multiple servers that don&#8217;t share disk space? If you can&#8217;t or won&#8217;t use file-based caching, I have something for you. It&#8217;s based on what WordPress.com uses. It&#8217;s Batcache.</p>\n<h3>Batcache will protect you</h3>\n<p>Batcache implements a very simplistic caching model that shields your database and web servers from traffic spikes: after a document has been requested X times in Y seconds, the document is cached for Z seconds and all new users are served the cached copy.</p>\n<p>New users are defined as anybody who hasn&#8217;t interacted with your domain&#8212;once they&#8217;ve left a comment or logged in, their cookies will ensure they get fresh pages. People arriving from Digg won&#8217;t notice that the comments are a minute or two behind but they&#8217;ll appreciate your site being up.</p>\n<p>You don&#8217;t need PHP skills to install Batcache but you do have to get Memcached working first. That can be easy or hard. We use Memcached because it&#8217;s <em>awesome</em>. Once you know how to install it you can create the same kind of distributed, persistent cache that underpin web giants like WordPress.com and Facebook.</p>\n<h3>What Batcache does</h3>\n<p>The first thing Batcache does is decide whether the visitor is eligible to receive cached documents. If their cookies don&#8217;t show evidence of previous interaction on that domain they are eligible. Next it decides whether the request is eligible for caching. For example, Batcache won&#8217;t interfere when a comment is being posted.</p>\n<p>If the visitor and the request are eligible, Batcache enters its traffic metering routine. By default it looks for URLs that receive more than two hits from unrecognized users in two minutes. When a URL&#8217;s traffic crosses that threshold, Batcache caches the document for five minutes. You can configure these numbers any way you like, or turn off traffic metering and send documents right to the cache.</p>\n<p>Once a document has been cached, it is served to eligible visitors until it expires. This is one place where Batcache is different. Most other caches delete cached documents as soon as the underlying data changes. Batcache doesn&#8217;t care if it&#8217;s serving old data because &#8220;old&#8221; is relative (and configurable).</p>\n<h3>What Batcache doesn&#8217;t do</h3>\n<p>It doesn&#8217;t guarantee a current document. I repeat this because reliable cache invalidation is a typical feature that was purposefully omitted from Batcache. There is a routine in the included plugin that tries to trigger regeneration of updated and commented posts but in some situations a document will still live in the cache until it expires. This routine will be improved over time but it is only an afterthought.</p>\n<p>Batcache doesn&#8217;t automatically know the difference between document variants. Variants exist when two requests for the same URL can yield two different documents. Common examples are user agent-dependent variants formatted for mobile devices and referrer-dependent variants with Google search terms highlighted. In these cases you MUST take extra steps to inform Batcache about variants to avoid serving a variant to the wrong audience. The source code includes examples of how to turn off caching of uncommon variants (search term highlighting) or cache common variants separately (mobile versions).</p>\n<h3>Where Batcache is going</h3>\n<p>I want to make Batcache easier to configure by adding a configuration page and storing the main settings in memcached as well as the database. This way you won&#8217;t have to deploy a code change to update the configuration. However, conditional configurations (e.g. &#8220;never cache URLs matching some pattern&#8221;) and variant detection will probably always live in PHP.</p>\n<p>I want to have Batcache serve correct headers more reliably. On some servers it can detect the headers that were sent with a newly generated page and serve them again from the cache. But when that doesn&#8217;t work you will have to take extra steps to serve certain headers. For example you must specify the Content-Encoding header in the Batcache configuration or add it to <code>php.ini</code>. I want this sort of thing to be done automatically for all server setups.</p>\n<p>I know that Batcache is not ideal for most WordPress installations. It saves us a lot of headaches and expense at WordPress.com, so maybe it can help other large installations. If you try it, I want to hear from you whether it worked and how well. I am also keen to see what new configurations and modifications you use.</p>\n<p>As always, this software is provided without claims or warrantees. It&#8217;s so experimental that it doesn&#8217;t even have a version number! Until the project grows to need its own blog, keep an eye on the <a href=\"http://dev.wp-plugins.org/browser/batcache/trunk\">Trac browser</a> for updates.</p>\n<img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/categories/andy.wordpress.com/180/\" /> <img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/tags/andy.wordpress.com/180/\" /> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gocomments/andy.wordpress.com/180/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/comments/andy.wordpress.com/180/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godelicious/andy.wordpress.com/180/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/delicious/andy.wordpress.com/180/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gostumble/andy.wordpress.com/180/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/stumble/andy.wordpress.com/180/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godigg/andy.wordpress.com/180/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/digg/andy.wordpress.com/180/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/goreddit/andy.wordpress.com/180/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/reddit/andy.wordpress.com/180/\" /></a> <img alt=\"\" border=\"0\" src=\"http://stats.wordpress.com/b.gif?host=andy.wordpress.com&blog=155&post=180&subd=andy&ref=&feed=1\" /></div>\";}i:20;a:7:{s:5:\"title\";s:25:\"Matt: Star Wars Dance-Off\";s:4:\"guid\";s:41:\"http://ma.tt/2008/06/star-wars-dance-off/\";s:4:\"link\";s:41:\"http://ma.tt/2008/06/star-wars-dance-off/\";s:11:\"description\";s:43:\"<p></p>\n<p>The internet has everything.</p>\";s:7:\"pubdate\";s:31:\"Sun, 22 Jun 2008 18:04:23 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:7:\"summary\";s:43:\"<p></p>\n<p>The internet has everything.</p>\";}i:21;a:7:{s:5:\"title\";s:57:\"Weblog Tools Collection: Andy Skelton Introduces Batcache\";s:4:\"guid\";s:40:\"http://weblogtoolscollection.com/?p=3679\";s:4:\"link\";s:86:\"http://weblogtoolscollection.com/archives/2008/06/22/andy-skelton-introduces-batcache/\";s:11:\"description\";s:2102:\"<p>So far, there are two excellent performance enhancing caching solutions available for WordPress, WP-Cache and WP-SuperCache. Both do an awesome job of caching posts/pages to help keep your website from falling off the map in the middle of a Digg effect. Now, there is a new entrant in the caching arena called <a href=\"http://wordpress.org/extend/plugins/batcache/\" target=\"_blank\">Batcache</a> which was created by <a href=\"http://wordpress.org/extend/plugins/profile/andy\" target=\"_blank\">Andy Skelton</a>.</p>\n<p>This plugin is meant to be used in situations where file-based caching is either not practical, or is not desired. According to Andy, development testing showed a 40x reduction in the time it took to generate pages. Pages that were generated in 200ms ended up being served from the cache in only 5ms.</p>\n<blockquote><p>Batcache uses Memcached to store and serve rendered pages. It&#8217;s not as fast as Donncha&#8217;s WP-Super-Cache but it can be used where file-based caching is not practical or not desired.</p></blockquote>\n<p>The basic premise of the plugin works similar to the other caching plugins that are available:</p>\n<blockquote><p>Batcache is aimed at preventing a flood of traffic from breaking your site. It does this by serving old pages to new users. This reduces the demand on the web server CPU and the database. It also means some people may see a page that is a few minutes old. However this only applies to people who have not interacted with your web site before. Once they have logged in or left a comment they will always get fresh pages.</p></blockquote>\n<p>Andy has also noted that the following features could be implemented in future versions of the plugin:</p>\n<ul>\n<li><strong> Comments, edits, and new posts will trigger cache regeneration</strong></li>\n<li><strong> Online installation assistance</strong></li>\n<li><strong> Configuration page</strong></li>\n<li><strong> Stats</strong></li>\n</ul>\n<p>So if WP-Cache or WP-SuperCache is not helping your situation, give Batcache a try and please, let us know how it works for you in the comments.</p>\";s:7:\"pubdate\";s:31:\"Sun, 22 Jun 2008 07:59:24 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:13:\"Jeff Chandler\";}s:7:\"summary\";s:2102:\"<p>So far, there are two excellent performance enhancing caching solutions available for WordPress, WP-Cache and WP-SuperCache. Both do an awesome job of caching posts/pages to help keep your website from falling off the map in the middle of a Digg effect. Now, there is a new entrant in the caching arena called <a href=\"http://wordpress.org/extend/plugins/batcache/\" target=\"_blank\">Batcache</a> which was created by <a href=\"http://wordpress.org/extend/plugins/profile/andy\" target=\"_blank\">Andy Skelton</a>.</p>\n<p>This plugin is meant to be used in situations where file-based caching is either not practical, or is not desired. According to Andy, development testing showed a 40x reduction in the time it took to generate pages. Pages that were generated in 200ms ended up being served from the cache in only 5ms.</p>\n<blockquote><p>Batcache uses Memcached to store and serve rendered pages. It&#8217;s not as fast as Donncha&#8217;s WP-Super-Cache but it can be used where file-based caching is not practical or not desired.</p></blockquote>\n<p>The basic premise of the plugin works similar to the other caching plugins that are available:</p>\n<blockquote><p>Batcache is aimed at preventing a flood of traffic from breaking your site. It does this by serving old pages to new users. This reduces the demand on the web server CPU and the database. It also means some people may see a page that is a few minutes old. However this only applies to people who have not interacted with your web site before. Once they have logged in or left a comment they will always get fresh pages.</p></blockquote>\n<p>Andy has also noted that the following features could be implemented in future versions of the plugin:</p>\n<ul>\n<li><strong> Comments, edits, and new posts will trigger cache regeneration</strong></li>\n<li><strong> Online installation assistance</strong></li>\n<li><strong> Configuration page</strong></li>\n<li><strong> Stats</strong></li>\n</ul>\n<p>So if WP-Cache or WP-SuperCache is not helping your situation, give Batcache a try and please, let us know how it works for you in the comments.</p>\";}i:22;a:7:{s:5:\"title\";s:49:\"Weblog Tools Collection: WordPress: Did you Know?\";s:4:\"guid\";s:40:\"http://weblogtoolscollection.com/?p=3653\";s:4:\"link\";s:76:\"http://weblogtoolscollection.com/archives/2008/06/21/wordpress-did-you-know/\";s:11:\"description\";s:212:\"<p>That if you change your URI after a post has been published, <a href=\"http://foolswisdom.com/typepad-seo-blows/#comment-157436\">the old URI continues to work while the links are changed to the new one?</a></p>\";s:7:\"pubdate\";s:31:\"Sat, 21 Jun 2008 23:11:03 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:10:\"Mark Ghosh\";}s:7:\"summary\";s:212:\"<p>That if you change your URI after a post has been published, <a href=\"http://foolswisdom.com/typepad-seo-blows/#comment-157436\">the old URI continues to work while the links are changed to the new one?</a></p>\";}i:23;a:7:{s:5:\"title\";s:58:\"Weblog Tools Collection: WordPress Theme Releases for 6/20\";s:4:\"guid\";s:88:\"http://weblogtoolscollection.com/archives/2008/06/20/wordpress-theme-releases-for-620-2/\";s:4:\"link\";s:88:\"http://weblogtoolscollection.com/archives/2008/06/20/wordpress-theme-releases-for-620-2/\";s:11:\"description\";s:3276:\"<h3>Two Column Themes</h3>\n<p><a href=\"http://xfep.com/wordpress/wordpress-theme-natural-power/\" target=\"_blank\"><strong>Natural Power</strong></a></p>\n<p><img title=\"natural-power-thumbnail\" height=\"126\" alt=\"natural-power-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/naturalpowerthumbnail.png\" width=\"199\" border=\"0\" /> </p>\n<p>Natural Power is a two column theme, widget ready, that comes with four different colored header graphics that can easily be changed in the options panel included with the theme.</p>\n<p><a href=\"http://www.wpthemedesigner.com/2008/06/04/blue-block-theme/\" target=\"_blank\"><strong>Blue Block</strong></a></p>\n<p><img title=\"blue-block-thumbnail\" height=\"139\" alt=\"blue-block-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/blueblockthumbnail.png\" width=\"200\" border=\"0\" /> </p>\n<p>Blue Block is a two column widget-ready theme. The theme comes build in with banner ad integration. The content are is quite smaller compared to other two column themes.</p>\n<p><a href=\"http://curtishenson.com/checkmate-a-free-wordpress-theme/\" target=\"_blank\"><strong>CheckMate</strong></a></p>\n<p><img title=\"checkmate-thumbnail\" height=\"120\" alt=\"checkmate-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/checkmatethumbnail.png\" width=\"200\" border=\"0\" /> </p>\n<p>CheckMate is a two column theme with 10 widgetized areas. The themes comes with 7 color options.</p>\n<p><a href=\"http://www.themelab.com/2008/06/01/out-of-the-box-free-wordpress-theme/\" target=\"_blank\"><strong>Out of the Box</strong></a></p>\n<p><img title=\"out-of-the-box\" height=\"121\" alt=\"out-of-the-box\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/outofthebox.png\" width=\"200\" border=\"0\" /> </p>\n<p>Out of the Box is a two column widget ready theme. The theme has a widgetized sidebar and footer area along with Gravatar support for WordPress 2.5 and above.</p>\n<p><a href=\"http://www.themelab.com/2008/06/04/techjunkie-free-wordpress-theme/\" target=\"_blank\"><strong>TechJunkie</strong></a></p>\n<p><img title=\"techie-junkie-thumbnail\" height=\"121\" alt=\"techie-junkie-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/techiejunkiethumbnail.png\" width=\"199\" border=\"0\" /> </p>\n<p>TechJunkie is a two column widget-ready theme with 4 widgetized areas including 3 in the footer. The theme comes built in with Gravatar support.</p>\n<p><a href=\"http://blog.butterflymedia.ro/wordpress-theme-whiskey-air/\" target=\"_blank\"><strong>Whisky Air</strong></a></p>\n<p><img title=\"whisky-air\" height=\"150\" alt=\"whisky-air\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/whiskyair.png\" width=\"141\" border=\"0\" /> </p>\n<p>Whisky Air is a two column widget-ready theme which does not make use of any images. The them uses simple dotted grey lines, justified paragraphs, easy-for-the-eye headings and titles.</p>\n<h3>Three Column Themes</h3>\n<p><a href=\"http://layout.nu/?d=13\" target=\"_blank\"><strong>Wolf</strong></a></p>\n<p><img title=\"wolf-thumbnail\" height=\"119\" alt=\"wolf-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/wolfthumbnail.png\" width=\"200\" border=\"0\" /> </p>\n</p>\n<p>Wolf is a 3 column black and gray theme featuring wolf photos in header and footer. The theme is widget-ready and supports Gravatars.</p>\";s:7:\"pubdate\";s:31:\"Sat, 21 Jun 2008 03:58:00 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:12:\"Keith Dsouza\";}s:7:\"summary\";s:3276:\"<h3>Two Column Themes</h3>\n<p><a href=\"http://xfep.com/wordpress/wordpress-theme-natural-power/\" target=\"_blank\"><strong>Natural Power</strong></a></p>\n<p><img title=\"natural-power-thumbnail\" height=\"126\" alt=\"natural-power-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/naturalpowerthumbnail.png\" width=\"199\" border=\"0\" /> </p>\n<p>Natural Power is a two column theme, widget ready, that comes with four different colored header graphics that can easily be changed in the options panel included with the theme.</p>\n<p><a href=\"http://www.wpthemedesigner.com/2008/06/04/blue-block-theme/\" target=\"_blank\"><strong>Blue Block</strong></a></p>\n<p><img title=\"blue-block-thumbnail\" height=\"139\" alt=\"blue-block-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/blueblockthumbnail.png\" width=\"200\" border=\"0\" /> </p>\n<p>Blue Block is a two column widget-ready theme. The theme comes build in with banner ad integration. The content are is quite smaller compared to other two column themes.</p>\n<p><a href=\"http://curtishenson.com/checkmate-a-free-wordpress-theme/\" target=\"_blank\"><strong>CheckMate</strong></a></p>\n<p><img title=\"checkmate-thumbnail\" height=\"120\" alt=\"checkmate-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/checkmatethumbnail.png\" width=\"200\" border=\"0\" /> </p>\n<p>CheckMate is a two column theme with 10 widgetized areas. The themes comes with 7 color options.</p>\n<p><a href=\"http://www.themelab.com/2008/06/01/out-of-the-box-free-wordpress-theme/\" target=\"_blank\"><strong>Out of the Box</strong></a></p>\n<p><img title=\"out-of-the-box\" height=\"121\" alt=\"out-of-the-box\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/outofthebox.png\" width=\"200\" border=\"0\" /> </p>\n<p>Out of the Box is a two column widget ready theme. The theme has a widgetized sidebar and footer area along with Gravatar support for WordPress 2.5 and above.</p>\n<p><a href=\"http://www.themelab.com/2008/06/04/techjunkie-free-wordpress-theme/\" target=\"_blank\"><strong>TechJunkie</strong></a></p>\n<p><img title=\"techie-junkie-thumbnail\" height=\"121\" alt=\"techie-junkie-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/techiejunkiethumbnail.png\" width=\"199\" border=\"0\" /> </p>\n<p>TechJunkie is a two column widget-ready theme with 4 widgetized areas including 3 in the footer. The theme comes built in with Gravatar support.</p>\n<p><a href=\"http://blog.butterflymedia.ro/wordpress-theme-whiskey-air/\" target=\"_blank\"><strong>Whisky Air</strong></a></p>\n<p><img title=\"whisky-air\" height=\"150\" alt=\"whisky-air\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/whiskyair.png\" width=\"141\" border=\"0\" /> </p>\n<p>Whisky Air is a two column widget-ready theme which does not make use of any images. The them uses simple dotted grey lines, justified paragraphs, easy-for-the-eye headings and titles.</p>\n<h3>Three Column Themes</h3>\n<p><a href=\"http://layout.nu/?d=13\" target=\"_blank\"><strong>Wolf</strong></a></p>\n<p><img title=\"wolf-thumbnail\" height=\"119\" alt=\"wolf-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/wolfthumbnail.png\" width=\"200\" border=\"0\" /> </p>\n</p>\n<p>Wolf is a 3 column black and gray theme featuring wolf photos in header and footer. The theme is widget-ready and supports Gravatars.</p>\";}i:24;a:7:{s:5:\"title\";s:21:\"Matt: Kindle Thoughts\";s:4:\"guid\";s:37:\"http://ma.tt/2008/06/kindle-thoughts/\";s:4:\"link\";s:37:\"http://ma.tt/2008/06/kindle-thoughts/\";s:11:\"description\";s:409:\"<p><a href=\"http://sethgodin.typepad.com/seths_blog/2008/06/random-thoughts.html\">Seth&#8217;s Blog: Random thoughts about the Kindle</a>. I agree with most of this. I&#8217;ve been meaning to write a Kindle review forever. I&#8217;m probably not going to get to it, but I will say that it has fundamentally changed the way I read and buy books. It has also increased my book reading a non-trivial amount.</p>\";s:7:\"pubdate\";s:31:\"Sat, 21 Jun 2008 03:16:56 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:7:\"summary\";s:409:\"<p><a href=\"http://sethgodin.typepad.com/seths_blog/2008/06/random-thoughts.html\">Seth&#8217;s Blog: Random thoughts about the Kindle</a>. I agree with most of this. I&#8217;ve been meaning to write a Kindle review forever. I&#8217;m probably not going to get to it, but I will say that it has fundamentally changed the way I read and buy books. It has also increased my book reading a non-trivial amount.</p>\";}i:25;a:7:{s:5:\"title\";s:16:\"Matt: Ops People\";s:4:\"guid\";s:32:\"http://ma.tt/2008/06/ops-people/\";s:4:\"link\";s:32:\"http://ma.tt/2008/06/ops-people/\";s:11:\"description\";s:98:\"<p><a href=\"http://times.usefulinc.com/2008/06/16-ops-now\">We&#8217;re all ops people now</a>.</p>\";s:7:\"pubdate\";s:31:\"Sat, 21 Jun 2008 02:49:34 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:7:\"summary\";s:98:\"<p><a href=\"http://times.usefulinc.com/2008/06/16-ops-now\">We&#8217;re all ops people now</a>.</p>\";}i:26;a:7:{s:5:\"title\";s:22:\"Matt: Visiting Houston\";s:4:\"guid\";s:20:\"http://ma.tt/?p=6516\";s:4:\"link\";s:38:\"http://ma.tt/2008/06/visiting-houston/\";s:11:\"description\";s:6189:\"<p>Visiting home in Houston.</p>\n<p><span id=\"more-6516\"></span></p>\n\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1856/\" title=\"mcm_1856\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1856-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1857/\" title=\"mcm_1857\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1857-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1858/\" title=\"mcm_1858\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1858-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1859/\" title=\"mcm_1859\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1859-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1862/\" title=\"mcm_1862\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1862-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1883/\" title=\"mcm_1883\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1883-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1900/\" title=\"mcm_1900\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1900-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1910/\" title=\"mcm_1910\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1910-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1914/\" title=\"mcm_1914\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1914-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1915/\" title=\"mcm_1915\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1915-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1916/\" title=\"mcm_1916\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1916-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1919/\" title=\"mcm_1919\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1919-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1925/\" title=\"mcm_1925\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1925-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1926/\" title=\"mcm_1926\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1926-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1928/\" title=\"mcm_1928\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1928-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1929/\" title=\"mcm_1929\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1929-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1931/\" title=\"mcm_1931\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1931-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1932/\" title=\"mcm_1932\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1932-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1933/\" title=\"mcm_1933\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1933-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1934/\" title=\"mcm_1934\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1934-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1935/\" title=\"mcm_1935\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1935-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1937/\" title=\"mcm_1937\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1937-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1941/\" title=\"mcm_1941\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1941-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1942/\" title=\"mcm_1942\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1942-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1946/\" title=\"mcm_1946\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1946-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1951/\" title=\"mcm_1951\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1951-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1952/\" title=\"mcm_1952\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1952-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1956/\" title=\"mcm_1956\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1956-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1959/\" title=\"mcm_1959\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1959-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1963/\" title=\"mcm_1963\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1963-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\";s:7:\"pubdate\";s:31:\"Sat, 21 Jun 2008 02:15:22 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:7:\"summary\";s:6189:\"<p>Visiting home in Houston.</p>\n<p><span id=\"more-6516\"></span></p>\n\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1856/\" title=\"mcm_1856\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1856-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1857/\" title=\"mcm_1857\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1857-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1858/\" title=\"mcm_1858\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1858-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1859/\" title=\"mcm_1859\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1859-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1862/\" title=\"mcm_1862\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1862-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1883/\" title=\"mcm_1883\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1883-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1900/\" title=\"mcm_1900\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1900-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1910/\" title=\"mcm_1910\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1910-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1914/\" title=\"mcm_1914\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1914-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1915/\" title=\"mcm_1915\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1915-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1916/\" title=\"mcm_1916\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1916-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1919/\" title=\"mcm_1919\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1919-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1925/\" title=\"mcm_1925\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1925-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1926/\" title=\"mcm_1926\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1926-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1928/\" title=\"mcm_1928\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1928-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1929/\" title=\"mcm_1929\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1929-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1931/\" title=\"mcm_1931\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1931-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1932/\" title=\"mcm_1932\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1932-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1933/\" title=\"mcm_1933\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1933-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1934/\" title=\"mcm_1934\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1934-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1935/\" title=\"mcm_1935\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1935-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1937/\" title=\"mcm_1937\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1937-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1941/\" title=\"mcm_1941\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1941-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1942/\" title=\"mcm_1942\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1942-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1946/\" title=\"mcm_1946\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1946-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1951/\" title=\"mcm_1951\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1951-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1952/\" title=\"mcm_1952\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1952-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1956/\" title=\"mcm_1956\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1956-139x210.jpg\" width=\"139\" height=\"210\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1959/\" title=\"mcm_1959\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1959-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\n<a href=\"http://ma.tt/2008/06/visiting-houston/mcm_1963/\" title=\"mcm_1963\"><img src=\"http://s.ma.tt/files/2008/06/mcm_1963-195x129.jpg\" width=\"195\" height=\"129\" class=\"attachment-thumbnail\" alt=\"\" /></a>\";}i:27;a:7:{s:5:\"title\";s:37:\"Matt: Magento / Wordpress Integration\";s:4:\"guid\";s:51:\"http://ma.tt/2008/06/magento-wordpress-integration/\";s:4:\"link\";s:51:\"http://ma.tt/2008/06/magento-wordpress-integration/\";s:11:\"description\";s:196:\"<p><a href=\"http://www.magentocommerce.com/extension/296/lazzymonks-wordpress-integration\">There&#8217;s a new plugin for Magento, an open source ecommerce system, to integrate it with WP</a>.</p>\";s:7:\"pubdate\";s:31:\"Fri, 20 Jun 2008 17:22:56 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:7:\"summary\";s:196:\"<p><a href=\"http://www.magentocommerce.com/extension/296/lazzymonks-wordpress-integration\">There&#8217;s a new plugin for Magento, an open source ecommerce system, to integrate it with WP</a>.</p>\";}i:28;a:7:{s:5:\"title\";s:54:\"Peter Westwood: making the default install more secure\";s:4:\"guid\";s:32:\"http://westi.wordpress.com/?p=49\";s:4:\"link\";s:77:\"http://westi.wordpress.com/2008/06/20/making-the-default-install-more-secure/\";s:11:\"description\";s:1881:\"<div class=\"snap_preview\"><br /><p>WordPress 2.6 will be more secure out-of-the box including <a href=\"http://trac.wordpress.org/ticket/7001\">better support for running the admin over SSL</a> and changes to <a href=\"http://trac.wordpress.org/ticket/7157\">disable the remote publishing protocols by default</a>.</p>\n<p>We have choosen to disable Atom Publishing Protocol and the variety of XML-RPC protocols by default as they expose a potential to be a security risk.  So from WordPress 2.6 onwards you will need to go into the Settings-&gt;Write page and enable them individually if you want to use them.</p>\n<img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/categories/westi.wordpress.com/49/\" /> <img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/tags/westi.wordpress.com/49/\" /> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gocomments/westi.wordpress.com/49/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/comments/westi.wordpress.com/49/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godelicious/westi.wordpress.com/49/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/delicious/westi.wordpress.com/49/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gostumble/westi.wordpress.com/49/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/stumble/westi.wordpress.com/49/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godigg/westi.wordpress.com/49/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/digg/westi.wordpress.com/49/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/goreddit/westi.wordpress.com/49/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/reddit/westi.wordpress.com/49/\" /></a> <img alt=\"\" border=\"0\" src=\"http://stats.wordpress.com/b.gif?host=westi.wordpress.com&blog=15396&post=49&subd=westi&ref=&feed=1\" /></div>\";s:7:\"pubdate\";s:31:\"Fri, 20 Jun 2008 16:09:22 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:14:\"Peter Westwood\";}s:7:\"summary\";s:1881:\"<div class=\"snap_preview\"><br /><p>WordPress 2.6 will be more secure out-of-the box including <a href=\"http://trac.wordpress.org/ticket/7001\">better support for running the admin over SSL</a> and changes to <a href=\"http://trac.wordpress.org/ticket/7157\">disable the remote publishing protocols by default</a>.</p>\n<p>We have choosen to disable Atom Publishing Protocol and the variety of XML-RPC protocols by default as they expose a potential to be a security risk.  So from WordPress 2.6 onwards you will need to go into the Settings-&gt;Write page and enable them individually if you want to use them.</p>\n<img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/categories/westi.wordpress.com/49/\" /> <img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/tags/westi.wordpress.com/49/\" /> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gocomments/westi.wordpress.com/49/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/comments/westi.wordpress.com/49/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godelicious/westi.wordpress.com/49/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/delicious/westi.wordpress.com/49/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gostumble/westi.wordpress.com/49/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/stumble/westi.wordpress.com/49/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godigg/westi.wordpress.com/49/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/digg/westi.wordpress.com/49/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/goreddit/westi.wordpress.com/49/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/reddit/westi.wordpress.com/49/\" /></a> <img alt=\"\" border=\"0\" src=\"http://stats.wordpress.com/b.gif?host=westi.wordpress.com&blog=15396&post=49&subd=westi&ref=&feed=1\" /></div>\";}i:29;a:7:{s:5:\"title\";s:52:\"Weblog Tools Collection: How to:  Offsets and Paging\";s:4:\"guid\";s:40:\"http://weblogtoolscollection.com/?p=3670\";s:4:\"link\";s:79:\"http://weblogtoolscollection.com/archives/2008/06/19/how-to-offsets-and-paging/\";s:11:\"description\";s:3478:\"<p>Reader John writes in:</p>\n<blockquote><p>How does one use paging with an offset?  Doing so breaks the navigation controls.</p></blockquote>\n<p>The problem with using an offset in a query is that WordPress ignores any reference to paging.  In other words, you can use an offset and paging, but not both together.</p>\n<p>This can be solved by tapping into the <strong><a href=\"http://codex.wordpress.org/Custom_Queries#Display_Order_and_Post_Count_Limit\">post_limits</a></strong> filter.</p>\n<h3>Step 1:  Add the &#8216;my_post_limit&#8217; Function</h3>\n<p>The <strong>my_post_limit</strong> function is what the <strong>post_limits</strong> filter will use to update the standard loop query.</p>\n<p>We&#8217;ll use the function to use paging and offsets together.  This function should be placed in your theme&#8217;s <strong>functions.php</strong> file.</p>\n<blockquote><pre><code>\nfunction my_post_limit($limit) {\n	global $paged, $myOffset;\n	if (empty($paged)) {\n			$paged = 1;\n	}\n	$postperpage = intval(get_option(\'posts_per_page\'));\n	$pgstrt = ((intval($paged) -1) * $postperpage) + $myOffset . \', \';\n	$limit = \'LIMIT \'.$pgstrt.$postperpage;\n	return $limit;\n} //end function my_post_limit\n</code></pre>\n</blockquote>\n<h3>Step 2:  Add the Filter Reference</h3>\n<blockquote><pre><code>\n&lt;?php add_filter(\'post_limits\', \'my_post_limit\'); ?&gt;\n&lt;div id=\"recent\"&gt;\n&lt;h3&gt;Recent Articles&lt;/h3&gt;\n</code></pre>\n</blockquote>\n<p>In the above code, I add a filter for <strong>post_limits</strong>, which will call the <strong>my_post_limit</strong> function mentioned in Step 1.</p>\n<h3>Step 3:  Start Your Loop</h3>\n<p>In this loop, we&#8217;ll be declaring a global variable called <strong>$myOffset</strong>.  This will be the offset we&#8217;d like to have and is used by the <strong>my_post_limit</strong> function.</p>\n<blockquote><pre><code>\n&lt;ul&gt;\n&lt;?php\nglobal $myOffset;\n$myOffset = 1;\n$temp = $wp_query;\n$wp_query= null;\n$wp_query = new WP_Query();\n$wp_query-&gt;query(\'offset=\'.$myOffset.\'&amp;showposts=5\'.\'&amp;paged=\'.$paged);\n?&gt;\n&lt;?php while ($wp_query-&gt;have_posts()) : $wp_query-&gt;the_post(); ?&gt;\n        &lt;li&gt;&lt;a href=\"&lt;?php the_permalink() ?&gt;\" rel=\"bookmark\"&gt;&lt;?php the_title(); ?&gt;&lt;/a&gt;&lt;/li&gt;\n&lt;?php endwhile; ?&gt;\n&lt;/ul&gt;\n</code></pre>\n</blockquote>\n<h3>Step 4:  Enable Navigation</h3>\n<p>We&#8217;re just adding the standard next/previous links at the end of the loop.</p>\n<blockquote><pre><code>\n&lt;div class=\"navigation\"&gt;\n  &lt;div class=\"alignleft\"&gt;&lt;?php previous_posts_link(\'&amp;laquo; Previous\') ?&gt;&lt;/div&gt;\n  &lt;div class=\"alignright\"&gt;&lt;?php next_posts_link(\'More &amp;raquo;\') ?&gt;&lt;/div&gt;\n&lt;/div&gt;\n&lt;/div&gt;\n</code></pre>\n</blockquote>\n<h3>Step 5:  Remove the Filter</h3>\n<p>In this step, we&#8217;ll be removing the <strong>post_limits</strong> filter and assigning the query back to its original value thanks to a temporary placeholder.</p>\n<blockquote><pre><code>\n&lt;?php $wp_query = null; $wp_query = $temp;?&gt;\n&lt;?php remove_filter(\'post_limits\', \'my_post_limit\'); ?&gt;\n</code></pre>\n</blockquote>\n<h3>Step 6:  You&#8217;re done!</h3>\n<p>Using the above code, you should now be able to use offsets and paging together.</p>\n<p>Here is <a href=\"http://weblogtoolscollection.com/b2-img/2008/06/offsets-paging.zip\">a downloadable file of all the code</a> mentioned in this post. </p>\n<p>Thank you John for asking such a great question.</p>\";s:7:\"pubdate\";s:31:\"Thu, 19 Jun 2008 22:08:28 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:14:\"Ronald Huereca\";}s:7:\"summary\";s:3478:\"<p>Reader John writes in:</p>\n<blockquote><p>How does one use paging with an offset?  Doing so breaks the navigation controls.</p></blockquote>\n<p>The problem with using an offset in a query is that WordPress ignores any reference to paging.  In other words, you can use an offset and paging, but not both together.</p>\n<p>This can be solved by tapping into the <strong><a href=\"http://codex.wordpress.org/Custom_Queries#Display_Order_and_Post_Count_Limit\">post_limits</a></strong> filter.</p>\n<h3>Step 1:  Add the &#8216;my_post_limit&#8217; Function</h3>\n<p>The <strong>my_post_limit</strong> function is what the <strong>post_limits</strong> filter will use to update the standard loop query.</p>\n<p>We&#8217;ll use the function to use paging and offsets together.  This function should be placed in your theme&#8217;s <strong>functions.php</strong> file.</p>\n<blockquote><pre><code>\nfunction my_post_limit($limit) {\n	global $paged, $myOffset;\n	if (empty($paged)) {\n			$paged = 1;\n	}\n	$postperpage = intval(get_option(\'posts_per_page\'));\n	$pgstrt = ((intval($paged) -1) * $postperpage) + $myOffset . \', \';\n	$limit = \'LIMIT \'.$pgstrt.$postperpage;\n	return $limit;\n} //end function my_post_limit\n</code></pre>\n</blockquote>\n<h3>Step 2:  Add the Filter Reference</h3>\n<blockquote><pre><code>\n&lt;?php add_filter(\'post_limits\', \'my_post_limit\'); ?&gt;\n&lt;div id=\"recent\"&gt;\n&lt;h3&gt;Recent Articles&lt;/h3&gt;\n</code></pre>\n</blockquote>\n<p>In the above code, I add a filter for <strong>post_limits</strong>, which will call the <strong>my_post_limit</strong> function mentioned in Step 1.</p>\n<h3>Step 3:  Start Your Loop</h3>\n<p>In this loop, we&#8217;ll be declaring a global variable called <strong>$myOffset</strong>.  This will be the offset we&#8217;d like to have and is used by the <strong>my_post_limit</strong> function.</p>\n<blockquote><pre><code>\n&lt;ul&gt;\n&lt;?php\nglobal $myOffset;\n$myOffset = 1;\n$temp = $wp_query;\n$wp_query= null;\n$wp_query = new WP_Query();\n$wp_query-&gt;query(\'offset=\'.$myOffset.\'&amp;showposts=5\'.\'&amp;paged=\'.$paged);\n?&gt;\n&lt;?php while ($wp_query-&gt;have_posts()) : $wp_query-&gt;the_post(); ?&gt;\n        &lt;li&gt;&lt;a href=\"&lt;?php the_permalink() ?&gt;\" rel=\"bookmark\"&gt;&lt;?php the_title(); ?&gt;&lt;/a&gt;&lt;/li&gt;\n&lt;?php endwhile; ?&gt;\n&lt;/ul&gt;\n</code></pre>\n</blockquote>\n<h3>Step 4:  Enable Navigation</h3>\n<p>We&#8217;re just adding the standard next/previous links at the end of the loop.</p>\n<blockquote><pre><code>\n&lt;div class=\"navigation\"&gt;\n  &lt;div class=\"alignleft\"&gt;&lt;?php previous_posts_link(\'&amp;laquo; Previous\') ?&gt;&lt;/div&gt;\n  &lt;div class=\"alignright\"&gt;&lt;?php next_posts_link(\'More &amp;raquo;\') ?&gt;&lt;/div&gt;\n&lt;/div&gt;\n&lt;/div&gt;\n</code></pre>\n</blockquote>\n<h3>Step 5:  Remove the Filter</h3>\n<p>In this step, we&#8217;ll be removing the <strong>post_limits</strong> filter and assigning the query back to its original value thanks to a temporary placeholder.</p>\n<blockquote><pre><code>\n&lt;?php $wp_query = null; $wp_query = $temp;?&gt;\n&lt;?php remove_filter(\'post_limits\', \'my_post_limit\'); ?&gt;\n</code></pre>\n</blockquote>\n<h3>Step 6:  You&#8217;re done!</h3>\n<p>Using the above code, you should now be able to use offsets and paging together.</p>\n<p>Here is <a href=\"http://weblogtoolscollection.com/b2-img/2008/06/offsets-paging.zip\">a downloadable file of all the code</a> mentioned in this post. </p>\n<p>Thank you John for asking such a great question.</p>\";}i:30;a:7:{s:5:\"title\";s:57:\"Weblog Tools Collection: WordPress Theme Release for 6/18\";s:4:\"guid\";s:85:\"http://weblogtoolscollection.com/archives/2008/06/18/wordpress-theme-release-for-618/\";s:4:\"link\";s:85:\"http://weblogtoolscollection.com/archives/2008/06/18/wordpress-theme-release-for-618/\";s:11:\"description\";s:3309:\"<h3>One Column Themes</h3>\n<p><a href=\"http://www.wp-magazine.se/free-theme-wp-pix/\" target=\"_blank\"><strong>WP Pix</strong></a></p>\n<p><a href=\"http://weblogtoolscollection.com/b2-img/2008/06/wppixthumbnail.png\"><img title=\"wp-pix-thumbnail\" height=\"151\" alt=\"wp-pix-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/wppixthumbnail-thumb.png\" width=\"204\" border=\"0\" /></a> </p>\n<p>WP Pix is a one column theme with no sidebars. The theme can be use by photo bloggers. The theme comes in with two different page templates one for displaying a blog and another for displaying portfolios.</p>\n<h3>Two Column Themes</h3>\n<p><a href=\"http://www.designer-daily.com/wordpress-theme-happy-cyclope-849\" target=\"_blank\"><strong>Happy Cyclope</strong></a></p>\n<p>&#160;<a href=\"http://weblogtoolscollection.com/b2-img/2008/06/happycyclopethumbnail.png\"><img title=\"happy-cyclope-thumbnail\" height=\"193\" alt=\"happy-cyclope-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/happycyclopethumbnail-thumb.png\" width=\"202\" border=\"0\" /></a> </p>\n<p>Happy Cyclope is a 2 column widget-ready theme. The theme is easy-to-customize, and displays the category pages and archives as lists.</p>\n<p><a href=\"http://www.harrr.org/rrr/bigbrain/\" target=\"_blank\"><strong>Big Brain</strong></a></p>\n<p><a href=\"http://weblogtoolscollection.com/b2-img/2008/06/bigbrainthumbnail.png\"><img title=\"big-brain-thumbnail\" height=\"145\" alt=\"big-brain-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/bigbrainthumbnail-thumb.png\" width=\"204\" border=\"0\" /></a> </p>\n<p>Big Brain is a two column widget-ready theme. As the name suggests there is image of big brain in the header. The theme also uses bigger fonts for post titles and content.</p>\n<p><a href=\"http://www.kabatology.com/06/05/ndomche-summary-theme-released-a-free-theme-for-wordpress-25/\" target=\"_blank\"><strong>Ndomche Summary</strong></a></p>\n<p><a href=\"http://weblogtoolscollection.com/b2-img/2008/06/ndomchethumbnail.png\"><img title=\"ndomche-thumbnail\" height=\"122\" alt=\"ndomche-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/ndomchethumbnail-thumb.png\" width=\"203\" border=\"0\" /></a> </p>\n<p>Ndomche Summary is a two column theme. The theme has a grid based style and has only been tested with WordPress 2.5 and above.</p>\n<p><a href=\"http://www.mbwebdesign.co.uk/templates/wordpress/\" target=\"_blank\"><strong>Aurora</strong></a></p>\n<p><a href=\"http://weblogtoolscollection.com/b2-img/2008/06/aurorathumbnail.png\"><img title=\"aurora-thumbnail\" height=\"123\" alt=\"aurora-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/aurorathumbnail-thumb.png\" width=\"203\" border=\"0\" /></a> </p>\n<p>Aurora is a two column widget-ready colors. The theme makes use of dark colors.</p>\n<p><a href=\"http://buddha.voce.ro/2008/06/10/going-pro-elegant-wordpress-theme.html\" target=\"_blank\"><strong>Elegant</strong></a></p>\n<p><a href=\"http://weblogtoolscollection.com/b2-img/2008/06/elegantthumbnail.png\"><img title=\"elegant-thumbnail\" height=\"125\" alt=\"elegant-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/elegantthumbnail-thumb.png\" width=\"203\" border=\"0\" /></a> </p>\n<p>Elegant is a pleasant looking two column theme. The theme is widget-ready and is made up of different flavors of the blue color.</p>\";s:7:\"pubdate\";s:31:\"Thu, 19 Jun 2008 03:55:00 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:12:\"Keith Dsouza\";}s:7:\"summary\";s:3309:\"<h3>One Column Themes</h3>\n<p><a href=\"http://www.wp-magazine.se/free-theme-wp-pix/\" target=\"_blank\"><strong>WP Pix</strong></a></p>\n<p><a href=\"http://weblogtoolscollection.com/b2-img/2008/06/wppixthumbnail.png\"><img title=\"wp-pix-thumbnail\" height=\"151\" alt=\"wp-pix-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/wppixthumbnail-thumb.png\" width=\"204\" border=\"0\" /></a> </p>\n<p>WP Pix is a one column theme with no sidebars. The theme can be use by photo bloggers. The theme comes in with two different page templates one for displaying a blog and another for displaying portfolios.</p>\n<h3>Two Column Themes</h3>\n<p><a href=\"http://www.designer-daily.com/wordpress-theme-happy-cyclope-849\" target=\"_blank\"><strong>Happy Cyclope</strong></a></p>\n<p>&#160;<a href=\"http://weblogtoolscollection.com/b2-img/2008/06/happycyclopethumbnail.png\"><img title=\"happy-cyclope-thumbnail\" height=\"193\" alt=\"happy-cyclope-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/happycyclopethumbnail-thumb.png\" width=\"202\" border=\"0\" /></a> </p>\n<p>Happy Cyclope is a 2 column widget-ready theme. The theme is easy-to-customize, and displays the category pages and archives as lists.</p>\n<p><a href=\"http://www.harrr.org/rrr/bigbrain/\" target=\"_blank\"><strong>Big Brain</strong></a></p>\n<p><a href=\"http://weblogtoolscollection.com/b2-img/2008/06/bigbrainthumbnail.png\"><img title=\"big-brain-thumbnail\" height=\"145\" alt=\"big-brain-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/bigbrainthumbnail-thumb.png\" width=\"204\" border=\"0\" /></a> </p>\n<p>Big Brain is a two column widget-ready theme. As the name suggests there is image of big brain in the header. The theme also uses bigger fonts for post titles and content.</p>\n<p><a href=\"http://www.kabatology.com/06/05/ndomche-summary-theme-released-a-free-theme-for-wordpress-25/\" target=\"_blank\"><strong>Ndomche Summary</strong></a></p>\n<p><a href=\"http://weblogtoolscollection.com/b2-img/2008/06/ndomchethumbnail.png\"><img title=\"ndomche-thumbnail\" height=\"122\" alt=\"ndomche-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/ndomchethumbnail-thumb.png\" width=\"203\" border=\"0\" /></a> </p>\n<p>Ndomche Summary is a two column theme. The theme has a grid based style and has only been tested with WordPress 2.5 and above.</p>\n<p><a href=\"http://www.mbwebdesign.co.uk/templates/wordpress/\" target=\"_blank\"><strong>Aurora</strong></a></p>\n<p><a href=\"http://weblogtoolscollection.com/b2-img/2008/06/aurorathumbnail.png\"><img title=\"aurora-thumbnail\" height=\"123\" alt=\"aurora-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/aurorathumbnail-thumb.png\" width=\"203\" border=\"0\" /></a> </p>\n<p>Aurora is a two column widget-ready colors. The theme makes use of dark colors.</p>\n<p><a href=\"http://buddha.voce.ro/2008/06/10/going-pro-elegant-wordpress-theme.html\" target=\"_blank\"><strong>Elegant</strong></a></p>\n<p><a href=\"http://weblogtoolscollection.com/b2-img/2008/06/elegantthumbnail.png\"><img title=\"elegant-thumbnail\" height=\"125\" alt=\"elegant-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/elegantthumbnail-thumb.png\" width=\"203\" border=\"0\" /></a> </p>\n<p>Elegant is a pleasant looking two column theme. The theme is widget-ready and is made up of different flavors of the blue color.</p>\";}i:31;a:7:{s:5:\"title\";s:70:\"Weblog Tools Collection: $3500+ purse for WordPress Plugin Competition\";s:4:\"guid\";s:40:\"http://weblogtoolscollection.com/?p=3655\";s:4:\"link\";s:97:\"http://weblogtoolscollection.com/archives/2008/06/18/3500-purse-for-wordpress-plugin-competition/\";s:11:\"description\";s:2209:\"<p>. . . and growing! We have had a couple of really large (Thanks to <a href=\"http://ma.tt\">Matt</a> and <a href=\"http://automattic.com\">Automattic</a> for their generous contributions towards the competition) and some smaller donations towards the final set of <a href=\"http://weblogtoolscollection.com/plugin-competition-prizes/\">prizes for the plugin competition</a> with the total purse standing at <strong>over $3500 dollars</strong>. Thanks to everyone who has donated money and time towards the competition to help make it a success. The community really appreciates your involvement. We continue to look for donations and sponsors to make the pot sweeter.</p>\n<p>This is your chance to not only showcase your work to the WordPress community but to also win fabulous cash prizes. If you are planning on releasing a plugin for the competition, head over to the plugin competition blog, register yourself and post about your plugin there. We have had a spat of registrations but the number of announcements on the blog has been much less. Please tell us about your cool new entry; our readers would love to know and help test and fine tune your plugins for you!</p>\n<p>For those that are following the competition and looking for cool plugins to test, there already are a <a href=\"http://weblogtoolscollection.com/pluginblog/\">couple of posts on the Plugin Competition Blog</a> to whet your appetite. Please provide your feeback for the authors. It will help them make their plugins better and your votes (on the final entries, not on the announcements) will help tremendously in the judging.</p>\n<p><strong>The email address to submit entries is wpplugincomp [at] wltc [dot] net </strong>Please use this email address towards the end of the competition to send in your entries into the plugin competition. You do not need to send in your entries till the last day and sending in your entries early will not give you any advantage. On the contrary, waiting till the last moment will give you more chances to tweak and modify your plugin to your benefit. Please use the contact form in the menu above to send in your questions about the competition or if you want to pledge some money towards prizes.</p>\";s:7:\"pubdate\";s:31:\"Wed, 18 Jun 2008 21:12:59 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:10:\"Mark Ghosh\";}s:7:\"summary\";s:2209:\"<p>. . . and growing! We have had a couple of really large (Thanks to <a href=\"http://ma.tt\">Matt</a> and <a href=\"http://automattic.com\">Automattic</a> for their generous contributions towards the competition) and some smaller donations towards the final set of <a href=\"http://weblogtoolscollection.com/plugin-competition-prizes/\">prizes for the plugin competition</a> with the total purse standing at <strong>over $3500 dollars</strong>. Thanks to everyone who has donated money and time towards the competition to help make it a success. The community really appreciates your involvement. We continue to look for donations and sponsors to make the pot sweeter.</p>\n<p>This is your chance to not only showcase your work to the WordPress community but to also win fabulous cash prizes. If you are planning on releasing a plugin for the competition, head over to the plugin competition blog, register yourself and post about your plugin there. We have had a spat of registrations but the number of announcements on the blog has been much less. Please tell us about your cool new entry; our readers would love to know and help test and fine tune your plugins for you!</p>\n<p>For those that are following the competition and looking for cool plugins to test, there already are a <a href=\"http://weblogtoolscollection.com/pluginblog/\">couple of posts on the Plugin Competition Blog</a> to whet your appetite. Please provide your feeback for the authors. It will help them make their plugins better and your votes (on the final entries, not on the announcements) will help tremendously in the judging.</p>\n<p><strong>The email address to submit entries is wpplugincomp [at] wltc [dot] net </strong>Please use this email address towards the end of the competition to send in your entries into the plugin competition. You do not need to send in your entries till the last day and sending in your entries early will not give you any advantage. On the contrary, waiting till the last moment will give you more chances to tweak and modify your plugin to your benefit. Please use the contact form in the menu above to send in your questions about the competition or if you want to pledge some money towards prizes.</p>\";}i:32;a:7:{s:5:\"title\";s:105:\"Lorelle on WP: Layered Technologies Special Event: Meet Matt Mullenweg, Barry Abrahamson, and Me in Vegas\";s:4:\"guid\";s:36:\"http://lorelle.wordpress.com/?p=2583\";s:4:\"link\";s:128:\"http://lorelle.wordpress.com/2008/06/18/layered-technologies-special-event-meet-matt-mullenweg-barry-abrahamson-and-me-in-vegas/\";s:11:\"description\";s:10317:\"<div class=\"snap_preview\"><br /><p>Many of you know that I&#8217;m a geek at heart. I go nuts over all things geeky. Secretly, I adore <a href=\"http://barry.wordpress.com/\" title=\"Barry on WordPress\">Barry Abrahamson</a>, the server guru of <a href=\"http://automattic.com/about/\" title=\"About Us « Automattic\">Automattic</a> and <a href=\"http://www.wordpress.com/\" title=\"WordPress.com free blogs\" rel=\"tag\">WordPress.com</a>, especially when he starts <a href=\"http://en.blog.wordpress.com/2008/06/02/may-wrap-up/\" title=\"May Wrap-up\">talking numbers for WordPress.com</a>:</p>\n<blockquote><p>2,638,958 file uploads<br />\nAbout 1,074 gigabytes of new files.<br />\n417 terabytes of content transferred from our datacenters.<br />\n1,134,796,234 pageviews total across all WordPress.com and VIP blogs.</p></blockquote>\n<p><img src=\"http://lorelle.files.wordpress.com/2008/06/ltpact2008.jpg?w=164&h=120\" alt=\"LT Pact 2008\" width=\"164\" height=\"120\" class=\"alignright size-full wp-image-2582\" />That&#8217;s a lot of server traffic - heck, that is an overwhelmingly unbelievable amount of server activity! How it all fits together is something that has always intrigued me. Now, you can join me and have many of these mysteries answered at the <a href=\"http://www.layeredtech.com/ltpact/\" title=\"Layered Technologies PACT Conference\">2008 Layered Technologies PACT Conference</a> from June 26-27, 2008, in Las Vegas, Nevada.</p>\n<p>The famous Caesars Palace Resort is the spot to learn all about how the server industry works and learn from the masters in the server and hosting industry, including Barry Abrahamson and <a href=\"http://ma.tt/\" title=\"Matt Mullenweg\" rel=\"tag\">Matt Mullenweg</a> of WordPress to learn how they use the power of <a href=\"http://www.layeredtech.com/\" title=\"Layered Technologies\" rel=\"tag\">Layered Technologies</a> servers to handle more than 3 million blogs on <a href=\"http://www.wordpress.com/\" title=\"WordPress.com free blogs\" rel=\"tag\">WordPress.com</a>. And it&#8217;s all free!</p>\n<p>The <a href=\"http://www.layeredtech.com/ltpact/speakers.php\" title=\"Speakers at LT Pact 2008\">list of speakers</a> is a veritable who&#8217;s who of leading technology and hosting experts including the top brass from Layered Technologies, representing some amazing innovators and passionate technical geeks, and other top notch experts on a <a href=\"http://www.layeredtech.com/ltpact/agenda.php\" title=\"Agenda for LT Pact 2008\">variety of subjects beyond web hosting and servers</a> including:</p>\n<ul>\n<li>James Staten, Principal Analyst of IT Infrastructure and Operations at <a href=\"http://www.forrester.com/\" title=\"Forrester Research\">Forrester Research</a>, a leading expert on x86 servers and infrastructure and trends</li>\n<li><a href=\"http://www.mintz.com/people.php?BioID=459\" title=\"Andrew Skale\">Andrew Skale</a>, Attorney, Mintz-Levin and expert in patent, trademark, copyright litigation, and entertainment law</li>\n<li>Chris Samson, the Senior Hosting Technology Specialist from CSNA and Microsoft</li>\n<li>Deven Kampenhout, Web Platform Architect Evangelist at Microsoft, Tim Johnson, Senior Manager of Channel Market Development, Global Marketing, at Seagate</li>\n<li>Jonah Stein, blogger of <a href=\"http://itstheroi.com/\" title=\"It\'s The ROI\">It&#8217;s The ROI</a></li>\n<li>Jimmy Guerrero, the Senior Product Marketing Manager in the Database Group of Sun Microsystems coordinating MySQL’s Web 2.0 and SaaS marketing programs, Dennis Dayman, Chief Privacy Officer of Eloqua, an expert in security, legal, privacy, and public relations issues</li>\n<li>Barry Lynn, the Chairman and CEO of 3Tera, expert on data center innovations</li>\n<li>Andy Schroepfer, the Founder and Hosting Visionary of Tier 1 Research, an expert in the hosting industry</li>\n</ul>\n<p>And me. </p>\n<p>That&#8217;s right. From among all the big wigs and whose who of the industry, I will be on a panel with <a href=\"http://onemansblog.com/\" title=\"John Pozadzides\" rel=\"tag\">John Pozadzides</a> and Matt Mullenweg talking about <em>The Business of Blogging</em> on Friday, June 27, at 1:15PM. You are invited to attend for free. <a href=\"http://www.layeredtech.com/ltpact/register.php\" title=\"Register\">Advanced registration is requested</a> so there will be plenty of food and fun for everyone. </p>\n<p>Among the fun is the <a href=\"http://www.layeredtech.com/ltpact/agenda.php#pokertourny\" title=\"LT PACT Poker Tournament\">LT PACT Poker Tournament</a> hosted by Master of Ceremonies Robert W. Thompson II of <a href=\"http://www.bravotv.com/Celebrity_Poker_Showdown/index.shtml\" title=\"NBC\'s BravoTV Celebrity Poker Showdown\">NBC&#8217;s BravoTV Celebrity Poker Showdown</a> with a total purse of USD $5,000. Get your poker face on.</p>\n<p>On a serious note, it&#8217;s a worthwhile event to attend. It&#8217;s a rare opportunity to meet and learn from these industry leaders on how to increase your web revenue with better business practices and technology. It&#8217;s a chance to learn about the latest in new technology and where the web is going, not just where it&#8217;s been. Topics include SEO, industry trends, virtualization trends and technology, site optimization, web hosting, cloud/grid hosting technology, online storage technology, intellectual property, email and mailing list management and business practices, site automation, business of blogging, security vulnerabilities and threats, backups, and more.</p>\n<p>I hope to see you there!</p>\n<p><img src=\"http://lorelle.files.wordpress.com/2008/06/woopra_badge_160px.jpg?w=160&h=52\" alt=\"Woopra web analytics program\" width=\"160\" height=\"52\" class=\"alignright size-full wp-image-2584\" /><strong><font color=\"red\">UPDATE:</font></strong> I&#8217;ve just learned that the lead developers of <a href=\"http://www.woopra.com/\" title=\"Woopra - blog statistics package\" rel=\"tag\">Woopra</a>, Jad Younan and Elie Khoury, will be at the conference. They are flying in from Lebanon for this special event. If you are in love with this <a href=\"http://lorelle.wordpress.com/2008/03/30/whooping-woopra-blog-statistics-program/\" title=\"Whooping Woopra Blog Statistics Program\">amazing web analytics and statistics program</a> and/or are one of the 20,000 beta testers and want to meet the brains behind the program, come to the <a href=\"http://www.layeredtech.com/ltpact/\" title=\"Layered Technologies PACT Conference\">2008 Layered Technologies PACT Conference</a>. It&#8217;s free! </p>\n<p><img src=\"http://lorelle.files.wordpress.com/2006/08/sig.gif\" alt=\"\" /><br />\n<hr />\n<p><font size=\"-1\"><b>Site Search Tags:</b> <a href=\"http://lorelle.wordpress.com/index.php?s=wordpress+news\" rel=\"tag\">wordpress news</a>, <a href=\"http://lorelle.wordpress.com/index.php?s=wordpress+events\" rel=\"tag\">wordpress events</a>, <a href=\"http://lorelle.wordpress.com/index.php?s=layered+technologies\" rel=\"tag\">layered technologies</a>, <a href=\"http://lorelle.wordpress.com/index.php?s=lt+pact\" rel=\"tag\">lt pact</a>, <a href=\"http://lorelle.wordpress.com/index.php?s=events\" rel=\"tag\">events</a>, <a href=\"http://lorelle.wordpress.com/index.php?s=meet+lorelle\" rel=\"tag\">meet lorelle</a>, <a href=\"http://lorelle.wordpress.com/index.php?s=matt+mullenweg\" rel=\"tag\">matt mullenweg</a>, <a href=\"http://lorelle.wordpress.com/index.php?s=barry+abramson\" rel=\"tag\">barry abramson</a>, <a href=\"http://lorelle.wordpress.com/index.php?s=servers\" rel=\"tag\">servers</a>, <a href=\"http://lorelle.wordpress.com/index.php?s=server+technology\" rel=\"tag\">server technology</a>, <a href=\"http://lorelle.wordpress.com/index.php?s=las+vegas\" rel=\"tag\">las vegas</a>, <a href=\"http://lorelle.wordpress.com/index.php?s=free+event\" rel=\"tag\">free event</a> </p>\n<p><a href=\"http://lorelle.wordpress.com/feed/\" title=\"Feed on Lorelle on WordPress\"><img class=\"wp-smiley\" src=\"http://lorelle.files.wordpress.com/2007/02/rss.png\" alt=\"Feed on Lorelle on WordPress\" /></a> <a href=\"http://lorelle.wordpress.com/feed/\" title=\"Feed on Lorelle on WordPress\">Subscribe</a>  <a href=\"http://feeds.feedburner.com/LorelleOnWordPress\" title=\"Feedburner Lorelle on WordPress Feed\"><img class=\"wp-smiley\" src=\"http://lorelle.files.wordpress.com/2007/04/feedburnericon.gif\" alt=\"Feedburner icon\" />Via Feedburner</a>  <a href=\"http://www.feedblitz.com/f/?Sub=182399\" title=\"Lorelle on WordPress - full site feed email subscription\"><img class=\"wp-smiley\" src=\"http://lorelle.files.wordpress.com/2007/03/email.gif\" alt=\"\" />Subscribe by Email</a> <a href=\"http://lorelle.wordpress.com/\" title=\"Visit Lorelle on WordPress\"><img class=\"wp-smiley\" src=\"http://lorelle.files.wordpress.com/2008/03/pointer.gif\" alt=\"\" />Visit</a><br /><a href=\"http://lorelle.wordpress.com/about/\" title=\"Copyright Protected by Brent and Lorelle VanFossen\">Copyright Lorelle VanFossen</a>, the author of <a href=\"http://lorelle.wordpress.com/books/blogging-tips/\" title=\"Blogging Tips Book By Lorelle VanFossen\"><em>Blogging Tips, What Bloggers Won\'t Tell You About Blogging</em></a>.</font></p>\n<img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/categories/lorelle.wordpress.com/2583/\" /> <img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/tags/lorelle.wordpress.com/2583/\" /> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gocomments/lorelle.wordpress.com/2583/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/comments/lorelle.wordpress.com/2583/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godelicious/lorelle.wordpress.com/2583/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/delicious/lorelle.wordpress.com/2583/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gostumble/lorelle.wordpress.com/2583/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/stumble/lorelle.wordpress.com/2583/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godigg/lorelle.wordpress.com/2583/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/digg/lorelle.wordpress.com/2583/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/goreddit/lorelle.wordpress.com/2583/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/reddit/lorelle.wordpress.com/2583/\" /></a> <img alt=\"\" border=\"0\" src=\"http://stats.wordpress.com/b.gif?host=lorelle.wordpress.com&blog=72&post=2583&subd=lorelle&ref=&feed=1\" /></div>\";s:7:\"pubdate\";s:31:\"Wed, 18 Jun 2008 18:48:46 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:17:\"Lorelle VanFossen\";}s:7:\"summary\";s:10317:\"<div class=\"snap_preview\"><br /><p>Many of you know that I&#8217;m a geek at heart. I go nuts over all things geeky. Secretly, I adore <a href=\"http://barry.wordpress.com/\" title=\"Barry on WordPress\">Barry Abrahamson</a>, the server guru of <a href=\"http://automattic.com/about/\" title=\"About Us « Automattic\">Automattic</a> and <a href=\"http://www.wordpress.com/\" title=\"WordPress.com free blogs\" rel=\"tag\">WordPress.com</a>, especially when he starts <a href=\"http://en.blog.wordpress.com/2008/06/02/may-wrap-up/\" title=\"May Wrap-up\">talking numbers for WordPress.com</a>:</p>\n<blockquote><p>2,638,958 file uploads<br />\nAbout 1,074 gigabytes of new files.<br />\n417 terabytes of content transferred from our datacenters.<br />\n1,134,796,234 pageviews total across all WordPress.com and VIP blogs.</p></blockquote>\n<p><img src=\"http://lorelle.files.wordpress.com/2008/06/ltpact2008.jpg?w=164&h=120\" alt=\"LT Pact 2008\" width=\"164\" height=\"120\" class=\"alignright size-full wp-image-2582\" />That&#8217;s a lot of server traffic - heck, that is an overwhelmingly unbelievable amount of server activity! How it all fits together is something that has always intrigued me. Now, you can join me and have many of these mysteries answered at the <a href=\"http://www.layeredtech.com/ltpact/\" title=\"Layered Technologies PACT Conference\">2008 Layered Technologies PACT Conference</a> from June 26-27, 2008, in Las Vegas, Nevada.</p>\n<p>The famous Caesars Palace Resort is the spot to learn all about how the server industry works and learn from the masters in the server and hosting industry, including Barry Abrahamson and <a href=\"http://ma.tt/\" title=\"Matt Mullenweg\" rel=\"tag\">Matt Mullenweg</a> of WordPress to learn how they use the power of <a href=\"http://www.layeredtech.com/\" title=\"Layered Technologies\" rel=\"tag\">Layered Technologies</a> servers to handle more than 3 million blogs on <a href=\"http://www.wordpress.com/\" title=\"WordPress.com free blogs\" rel=\"tag\">WordPress.com</a>. And it&#8217;s all free!</p>\n<p>The <a href=\"http://www.layeredtech.com/ltpact/speakers.php\" title=\"Speakers at LT Pact 2008\">list of speakers</a> is a veritable who&#8217;s who of leading technology and hosting experts including the top brass from Layered Technologies, representing some amazing innovators and passionate technical geeks, and other top notch experts on a <a href=\"http://www.layeredtech.com/ltpact/agenda.php\" title=\"Agenda for LT Pact 2008\">variety of subjects beyond web hosting and servers</a> including:</p>\n<ul>\n<li>James Staten, Principal Analyst of IT Infrastructure and Operations at <a href=\"http://www.forrester.com/\" title=\"Forrester Research\">Forrester Research</a>, a leading expert on x86 servers and infrastructure and trends</li>\n<li><a href=\"http://www.mintz.com/people.php?BioID=459\" title=\"Andrew Skale\">Andrew Skale</a>, Attorney, Mintz-Levin and expert in patent, trademark, copyright litigation, and entertainment law</li>\n<li>Chris Samson, the Senior Hosting Technology Specialist from CSNA and Microsoft</li>\n<li>Deven Kampenhout, Web Platform Architect Evangelist at Microsoft, Tim Johnson, Senior Manager of Channel Market Development, Global Marketing, at Seagate</li>\n<li>Jonah Stein, blogger of <a href=\"http://itstheroi.com/\" title=\"It\'s The ROI\">It&#8217;s The ROI</a></li>\n<li>Jimmy Guerrero, the Senior Product Marketing Manager in the Database Group of Sun Microsystems coordinating MySQL’s Web 2.0 and SaaS marketing programs, Dennis Dayman, Chief Privacy Officer of Eloqua, an expert in security, legal, privacy, and public relations issues</li>\n<li>Barry Lynn, the Chairman and CEO of 3Tera, expert on data center innovations</li>\n<li>Andy Schroepfer, the Founder and Hosting Visionary of Tier 1 Research, an expert in the hosting industry</li>\n</ul>\n<p>And me. </p>\n<p>That&#8217;s right. From among all the big wigs and whose who of the industry, I will be on a panel with <a href=\"http://onemansblog.com/\" title=\"John Pozadzides\" rel=\"tag\">John Pozadzides</a> and Matt Mullenweg talking about <em>The Business of Blogging</em> on Friday, June 27, at 1:15PM. You are invited to attend for free. <a href=\"http://www.layeredtech.com/ltpact/register.php\" title=\"Register\">Advanced registration is requested</a> so there will be plenty of food and fun for everyone. </p>\n<p>Among the fun is the <a href=\"http://www.layeredtech.com/ltpact/agenda.php#pokertourny\" title=\"LT PACT Poker Tournament\">LT PACT Poker Tournament</a> hosted by Master of Ceremonies Robert W. Thompson II of <a href=\"http://www.bravotv.com/Celebrity_Poker_Showdown/index.shtml\" title=\"NBC\'s BravoTV Celebrity Poker Showdown\">NBC&#8217;s BravoTV Celebrity Poker Showdown</a> with a total purse of USD $5,000. Get your poker face on.</p>\n<p>On a serious note, it&#8217;s a worthwhile event to attend. It&#8217;s a rare opportunity to meet and learn from these industry leaders on how to increase your web revenue with better business practices and technology. It&#8217;s a chance to learn about the latest in new technology and where the web is going, not just where it&#8217;s been. Topics include SEO, industry trends, virtualization trends and technology, site optimization, web hosting, cloud/grid hosting technology, online storage technology, intellectual property, email and mailing list management and business practices, site automation, business of blogging, security vulnerabilities and threats, backups, and more.</p>\n<p>I hope to see you there!</p>\n<p><img src=\"http://lorelle.files.wordpress.com/2008/06/woopra_badge_160px.jpg?w=160&h=52\" alt=\"Woopra web analytics program\" width=\"160\" height=\"52\" class=\"alignright size-full wp-image-2584\" /><strong><font color=\"red\">UPDATE:</font></strong> I&#8217;ve just learned that the lead developers of <a href=\"http://www.woopra.com/\" title=\"Woopra - blog statistics package\" rel=\"tag\">Woopra</a>, Jad Younan and Elie Khoury, will be at the conference. They are flying in from Lebanon for this special event. If you are in love with this <a href=\"http://lorelle.wordpress.com/2008/03/30/whooping-woopra-blog-statistics-program/\" title=\"Whooping Woopra Blog Statistics Program\">amazing web analytics and statistics program</a> and/or are one of the 20,000 beta testers and want to meet the brains behind the program, come to the <a href=\"http://www.layeredtech.com/ltpact/\" title=\"Layered Technologies PACT Conference\">2008 Layered Technologies PACT Conference</a>. It&#8217;s free! </p>\n<p><img src=\"http://lorelle.files.wordpress.com/2006/08/sig.gif\" alt=\"\" /><br />\n<hr />\n<p><font size=\"-1\"><b>Site Search Tags:</b> <a href=\"http://lorelle.wordpress.com/index.php?s=wordpress+news\" rel=\"tag\">wordpress news</a>, <a href=\"http://lorelle.wordpress.com/index.php?s=wordpress+events\" rel=\"tag\">wordpress events</a>, <a href=\"http://lorelle.wordpress.com/index.php?s=layered+technologies\" rel=\"tag\">layered technologies</a>, <a href=\"http://lorelle.wordpress.com/index.php?s=lt+pact\" rel=\"tag\">lt pact</a>, <a href=\"http://lorelle.wordpress.com/index.php?s=events\" rel=\"tag\">events</a>, <a href=\"http://lorelle.wordpress.com/index.php?s=meet+lorelle\" rel=\"tag\">meet lorelle</a>, <a href=\"http://lorelle.wordpress.com/index.php?s=matt+mullenweg\" rel=\"tag\">matt mullenweg</a>, <a href=\"http://lorelle.wordpress.com/index.php?s=barry+abramson\" rel=\"tag\">barry abramson</a>, <a href=\"http://lorelle.wordpress.com/index.php?s=servers\" rel=\"tag\">servers</a>, <a href=\"http://lorelle.wordpress.com/index.php?s=server+technology\" rel=\"tag\">server technology</a>, <a href=\"http://lorelle.wordpress.com/index.php?s=las+vegas\" rel=\"tag\">las vegas</a>, <a href=\"http://lorelle.wordpress.com/index.php?s=free+event\" rel=\"tag\">free event</a> </p>\n<p><a href=\"http://lorelle.wordpress.com/feed/\" title=\"Feed on Lorelle on WordPress\"><img class=\"wp-smiley\" src=\"http://lorelle.files.wordpress.com/2007/02/rss.png\" alt=\"Feed on Lorelle on WordPress\" /></a> <a href=\"http://lorelle.wordpress.com/feed/\" title=\"Feed on Lorelle on WordPress\">Subscribe</a>  <a href=\"http://feeds.feedburner.com/LorelleOnWordPress\" title=\"Feedburner Lorelle on WordPress Feed\"><img class=\"wp-smiley\" src=\"http://lorelle.files.wordpress.com/2007/04/feedburnericon.gif\" alt=\"Feedburner icon\" />Via Feedburner</a>  <a href=\"http://www.feedblitz.com/f/?Sub=182399\" title=\"Lorelle on WordPress - full site feed email subscription\"><img class=\"wp-smiley\" src=\"http://lorelle.files.wordpress.com/2007/03/email.gif\" alt=\"\" />Subscribe by Email</a> <a href=\"http://lorelle.wordpress.com/\" title=\"Visit Lorelle on WordPress\"><img class=\"wp-smiley\" src=\"http://lorelle.files.wordpress.com/2008/03/pointer.gif\" alt=\"\" />Visit</a><br /><a href=\"http://lorelle.wordpress.com/about/\" title=\"Copyright Protected by Brent and Lorelle VanFossen\">Copyright Lorelle VanFossen</a>, the author of <a href=\"http://lorelle.wordpress.com/books/blogging-tips/\" title=\"Blogging Tips Book By Lorelle VanFossen\"><em>Blogging Tips, What Bloggers Won\'t Tell You About Blogging</em></a>.</font></p>\n<img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/categories/lorelle.wordpress.com/2583/\" /> <img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/tags/lorelle.wordpress.com/2583/\" /> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gocomments/lorelle.wordpress.com/2583/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/comments/lorelle.wordpress.com/2583/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godelicious/lorelle.wordpress.com/2583/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/delicious/lorelle.wordpress.com/2583/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gostumble/lorelle.wordpress.com/2583/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/stumble/lorelle.wordpress.com/2583/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godigg/lorelle.wordpress.com/2583/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/digg/lorelle.wordpress.com/2583/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/goreddit/lorelle.wordpress.com/2583/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/reddit/lorelle.wordpress.com/2583/\" /></a> <img alt=\"\" border=\"0\" src=\"http://stats.wordpress.com/b.gif?host=lorelle.wordpress.com&blog=72&post=2583&subd=lorelle&ref=&feed=1\" /></div>\";}i:33;a:7:{s:5:\"title\";s:24:\"Matt: 2008 Design Trends\";s:4:\"guid\";s:40:\"http://ma.tt/2008/06/2008-design-trends/\";s:4:\"link\";s:40:\"http://ma.tt/2008/06/2008-design-trends/\";s:11:\"description\";s:119:\"<p><a href=\"http://www.webdesignerwall.com/trends/2008-design-trends/\">2008 Design Trends</a>, a great visual list.</p>\";s:7:\"pubdate\";s:31:\"Wed, 18 Jun 2008 15:51:04 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:7:\"summary\";s:119:\"<p><a href=\"http://www.webdesignerwall.com/trends/2008-design-trends/\">2008 Design Trends</a>, a great visual list.</p>\";}i:34;a:7:{s:5:\"title\";s:75:\"Weblog Tools Collection: What is the simplest WordPress blog you’ve seen?\";s:4:\"guid\";s:40:\"http://weblogtoolscollection.com/?p=3654\";s:4:\"link\";s:100:\"http://weblogtoolscollection.com/archives/2008/06/17/what-is-the-simplest-wordpress-blog-youve-seen/\";s:11:\"description\";s:646:\"<p>This post is based on a question that was <a href=\"http://www.reddit.com/info/6mxjn/\">posted on reddit.com</a> which really intrigued me. I have seen a lot of WordPress blogs that are so simple and so well constructed that they blew me away and inspired me to do more to make my blogs simple(r). I just have a hard time finding them when the time is right and I am about to make changes to my designs.<br />\n<strong><em> In that spirit, what is the simplest and most elegant WordPress design that you have come across? </em></strong><a href=\"http://foolswisdom.com/\">Lloyd&#8217;s blog</a><strong> </strong>comes to mind as a good example.</p>\";s:7:\"pubdate\";s:31:\"Tue, 17 Jun 2008 22:44:27 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:10:\"Mark Ghosh\";}s:7:\"summary\";s:646:\"<p>This post is based on a question that was <a href=\"http://www.reddit.com/info/6mxjn/\">posted on reddit.com</a> which really intrigued me. I have seen a lot of WordPress blogs that are so simple and so well constructed that they blew me away and inspired me to do more to make my blogs simple(r). I just have a hard time finding them when the time is right and I am about to make changes to my designs.<br />\n<strong><em> In that spirit, what is the simplest and most elegant WordPress design that you have come across? </em></strong><a href=\"http://foolswisdom.com/\">Lloyd&#8217;s blog</a><strong> </strong>comes to mind as a good example.</p>\";}i:35;a:7:{s:5:\"title\";s:48:\"Weblog Tools Collection: All In One SEO Lives On\";s:4:\"guid\";s:40:\"http://weblogtoolscollection.com/?p=3651\";s:4:\"link\";s:77:\"http://weblogtoolscollection.com/archives/2008/06/17/all-in-one-seo-lives-on/\";s:11:\"description\";s:2206:\"<p>For those of you who may not have heard, it was recently discovered that the popular <a title=\"http://wordpress.org/extend/plugins/all-in-one-seo-pack/\" href=\"http://wordpress.org/extend/plugins/all-in-one-seo-pack/\" target=\"_blank\">All In One SEO Plugin for WordPress</a> would be no longer updated or supported past WordPress 2.5.1. The plugin is currently compatible with 2.5.1 but may not work with future versions of WordPress.</p>\n<p>However, I have managed to get a hold of the plugin author and he informed me that he believes the plugin will not die. He is currently in discussions with a number of other developers to continue maintenance of the plugin. He also expressed interest in the idea of having the plugin integrated into the core of WordPress.</p>\n<p>*<strong>UPDATE</strong>*</p>\n<p>It looks like the plugin has found a new home. A WordPress.org user who goes by the username hallsofmontezuma who is also the plugin author of the <a title=\"http://wordpress.org/extend/plugins/wp-security-scan/\" href=\"http://wordpress.org/extend/plugins/wp-security-scan/\" target=\"_blank\">WP Security Scan</a> plugin as well as the <a title=\"http://wordpress.org/extend/plugins/sms-text-message/\" href=\"http://wordpress.org/extend/plugins/sms-text-message/\" target=\"_blank\">SMS Text Message</a> plugin has taken over the development of the All In One SEO Plugin. The plugin can now be found on the <a title=\"http://semperfiwebdesign.com/portfolio/wordpress/wordpress-plugins/all-in-one-seo-pack/\" href=\"http://semperfiwebdesign.com/portfolio/wordpress/wordpress-plugins/all-in-one-seo-pack/\" target=\"_blank\">Semper Fi Web Design</a> website.</p>\n<p>I want to personally extend a hearty thanks to Uberdose who was the original author of the plugin. You can&#8217;t talk about WordPress and SEO without mentioning the All In One SEO Plugin. Thanks for deciding not to stop the project but instead, found a new home for it.</p>\n<p>I learned about the new plugin author via the <a title=\"http://www.imbloggingthat.com/2008/06/15/all-in-one-seo-pack-finds-new-developer/\" href=\"http://www.imbloggingthat.com/2008/06/15/all-in-one-seo-pack-finds-new-developer/\" target=\"_blank\">Imbloggingthat</a> blog.</p>\";s:7:\"pubdate\";s:31:\"Tue, 17 Jun 2008 08:21:15 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:13:\"Jeff Chandler\";}s:7:\"summary\";s:2206:\"<p>For those of you who may not have heard, it was recently discovered that the popular <a title=\"http://wordpress.org/extend/plugins/all-in-one-seo-pack/\" href=\"http://wordpress.org/extend/plugins/all-in-one-seo-pack/\" target=\"_blank\">All In One SEO Plugin for WordPress</a> would be no longer updated or supported past WordPress 2.5.1. The plugin is currently compatible with 2.5.1 but may not work with future versions of WordPress.</p>\n<p>However, I have managed to get a hold of the plugin author and he informed me that he believes the plugin will not die. He is currently in discussions with a number of other developers to continue maintenance of the plugin. He also expressed interest in the idea of having the plugin integrated into the core of WordPress.</p>\n<p>*<strong>UPDATE</strong>*</p>\n<p>It looks like the plugin has found a new home. A WordPress.org user who goes by the username hallsofmontezuma who is also the plugin author of the <a title=\"http://wordpress.org/extend/plugins/wp-security-scan/\" href=\"http://wordpress.org/extend/plugins/wp-security-scan/\" target=\"_blank\">WP Security Scan</a> plugin as well as the <a title=\"http://wordpress.org/extend/plugins/sms-text-message/\" href=\"http://wordpress.org/extend/plugins/sms-text-message/\" target=\"_blank\">SMS Text Message</a> plugin has taken over the development of the All In One SEO Plugin. The plugin can now be found on the <a title=\"http://semperfiwebdesign.com/portfolio/wordpress/wordpress-plugins/all-in-one-seo-pack/\" href=\"http://semperfiwebdesign.com/portfolio/wordpress/wordpress-plugins/all-in-one-seo-pack/\" target=\"_blank\">Semper Fi Web Design</a> website.</p>\n<p>I want to personally extend a hearty thanks to Uberdose who was the original author of the plugin. You can&#8217;t talk about WordPress and SEO without mentioning the All In One SEO Plugin. Thanks for deciding not to stop the project but instead, found a new home for it.</p>\n<p>I learned about the new plugin author via the <a title=\"http://www.imbloggingthat.com/2008/06/15/all-in-one-seo-pack-finds-new-developer/\" href=\"http://www.imbloggingthat.com/2008/06/15/all-in-one-seo-pack-finds-new-developer/\" target=\"_blank\">Imbloggingthat</a> blog.</p>\";}i:36;a:7:{s:5:\"title\";s:28:\"Matt: Friends Using Typepad?\";s:4:\"guid\";s:20:\"http://ma.tt/?p=6505\";s:4:\"link\";s:29:\"http://ma.tt/2008/06/hypepad/\";s:11:\"description\";s:1209:\"<p>Michael Krotscheck has an interesting post called <a href=\"http://www.krotscheck.net/2008/06/13/friends-dont-let-friends-use-typepad.html\">Friends don’t let Friends use TypePad</a>, which apparently ruffled some feathers and <a href=\"http://www.krotscheck.net/2008/06/13/friends-dont-let-friends-use-typepad.html#comments\">elicited a pretty venomous response from a Six Apart Vice President</a>. I guess is part of their new plan to &#8220;compete&#8221; but statements like &#8220;TypePad simply blows WordPress.com away on SEO&#8221; and &#8220;On WordPress.com, you’re kind of moving into a bad neighborhood — by their own admission, one-third of the blogs on WordPress.com are spam&#8221; don&#8217;t exactly lend credibility.  Michael responded eloquently in a comment and <a href=\"http://www.krotscheck.net/2008/06/14/wordpress-vs-typepad-round-2.html\">then again in a follow-up post</a>. Lloyd has jumped in with some <a href=\"http://foolswisdom.com/typepad-seo-blows/\">specific facts on Typepad&#8217;s (lack of) SEO</a>. In the meantime <a href=\"http://en.blog.wordpress.com/2008/06/16/xml-sitemaps/\">we just turned on sitemaps for everybody on WordPress.com</a>, a popular user request.</p>\";s:7:\"pubdate\";s:31:\"Tue, 17 Jun 2008 06:43:44 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:7:\"summary\";s:1209:\"<p>Michael Krotscheck has an interesting post called <a href=\"http://www.krotscheck.net/2008/06/13/friends-dont-let-friends-use-typepad.html\">Friends don’t let Friends use TypePad</a>, which apparently ruffled some feathers and <a href=\"http://www.krotscheck.net/2008/06/13/friends-dont-let-friends-use-typepad.html#comments\">elicited a pretty venomous response from a Six Apart Vice President</a>. I guess is part of their new plan to &#8220;compete&#8221; but statements like &#8220;TypePad simply blows WordPress.com away on SEO&#8221; and &#8220;On WordPress.com, you’re kind of moving into a bad neighborhood — by their own admission, one-third of the blogs on WordPress.com are spam&#8221; don&#8217;t exactly lend credibility.  Michael responded eloquently in a comment and <a href=\"http://www.krotscheck.net/2008/06/14/wordpress-vs-typepad-round-2.html\">then again in a follow-up post</a>. Lloyd has jumped in with some <a href=\"http://foolswisdom.com/typepad-seo-blows/\">specific facts on Typepad&#8217;s (lack of) SEO</a>. In the meantime <a href=\"http://en.blog.wordpress.com/2008/06/16/xml-sitemaps/\">we just turned on sitemaps for everybody on WordPress.com</a>, a popular user request.</p>\";}i:37;a:7:{s:5:\"title\";s:59:\"Weblog Tools Collection: WordPress Plugin Releases for 6/16\";s:4:\"guid\";s:87:\"http://weblogtoolscollection.com/archives/2008/06/16/wordpress-plugin-releases-for-616/\";s:4:\"link\";s:87:\"http://weblogtoolscollection.com/archives/2008/06/16/wordpress-plugin-releases-for-616/\";s:11:\"description\";s:2462:\"<p><a href=\"http://www.abundanceunlimited.com/products/au-footer/\" target=\"_blank\"><strong>AU Footer</strong></a></p>\n<p>AU Footer plugin lets you add html code to your website footer. A footer on a website is where you typically have your copyright notice. You no longer have to update the footer template manually which is helpful for theme updates and switching themes.</p>\n<p><a href=\"http://www.dailyblogtips.com/my-top-commentators-wordpress-plugin/\"><strong>My Top Commentators</strong></a></p>\n</p>\n<p>My Top Commentators is a plugin that allows a blogger to discover who are the most active commentators on his blog (via the WordPress Dashboard, so other people will not see this information). You can also use different time frames for the top commentators.</p>\n<p><a href=\"http://www.x-or.be/blog/134-show125-un-widget-pour-vos-annonces-125125\"><strong>Show125</strong></a></p>\n<p>A handy widget that allows you to display 125 x 125 ad banners in your sidebar. The links and images are configurable through the admin interface.</p>\n<p><a href=\"http://graphpaperpress.com/2008/05/31/photo-galleria-plugin-for-wordpress/\"><strong>Photo Galleria</strong></a></p>\n<p>Photo Galleria plugin synchronizes Galleria with built-in media manager in Wordpress. Simply upload photos as normal and the Photo Galleria plugin will create a photo gallery with fade-in and fade-out effects common on flash-based websites.</p>\n<h4><a href=\"http://www.sattes-faction.de/picasaview/\"><strong>picasaView</strong></a> </h4>\n<p> picasaView allows you to easily view your picasaweb albums in your blog. To achieve this, all you need to do is to insert a placeholder wherever you want to display your album in a post or page. You can also use a PHP code snippet to display the album in the sidebar.\n</p>\n</p>\n</p>\n</p>\n</p>\n<p><a href=\"http://wordpress.org/extend/plugins/maxref-widgets/\"><strong>MaxRef Widgets</strong></a></p>\n<p>MaxRef Widgets is a powerful way to add unlimited widgets on your sidebars to display posts only from a certain category, the links from a particular link category, comments and more.</p>\n<p><a href=\"http://www.weinschenker.name/plugin-feed-reading-blogroll/\"><strong>Feed reading blogroll</strong></a></p>\n<p>Feed reading blogroll provides a sidebar-widget and a template-tag that add an enhanced blogroll to your sidebar. It checks on all your linked sites for which you saved a separate feed-url, when the most recent update has occurred.</p>\";s:7:\"pubdate\";s:31:\"Tue, 17 Jun 2008 03:55:00 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:12:\"Keith Dsouza\";}s:7:\"summary\";s:2462:\"<p><a href=\"http://www.abundanceunlimited.com/products/au-footer/\" target=\"_blank\"><strong>AU Footer</strong></a></p>\n<p>AU Footer plugin lets you add html code to your website footer. A footer on a website is where you typically have your copyright notice. You no longer have to update the footer template manually which is helpful for theme updates and switching themes.</p>\n<p><a href=\"http://www.dailyblogtips.com/my-top-commentators-wordpress-plugin/\"><strong>My Top Commentators</strong></a></p>\n</p>\n<p>My Top Commentators is a plugin that allows a blogger to discover who are the most active commentators on his blog (via the WordPress Dashboard, so other people will not see this information). You can also use different time frames for the top commentators.</p>\n<p><a href=\"http://www.x-or.be/blog/134-show125-un-widget-pour-vos-annonces-125125\"><strong>Show125</strong></a></p>\n<p>A handy widget that allows you to display 125 x 125 ad banners in your sidebar. The links and images are configurable through the admin interface.</p>\n<p><a href=\"http://graphpaperpress.com/2008/05/31/photo-galleria-plugin-for-wordpress/\"><strong>Photo Galleria</strong></a></p>\n<p>Photo Galleria plugin synchronizes Galleria with built-in media manager in Wordpress. Simply upload photos as normal and the Photo Galleria plugin will create a photo gallery with fade-in and fade-out effects common on flash-based websites.</p>\n<h4><a href=\"http://www.sattes-faction.de/picasaview/\"><strong>picasaView</strong></a> </h4>\n<p> picasaView allows you to easily view your picasaweb albums in your blog. To achieve this, all you need to do is to insert a placeholder wherever you want to display your album in a post or page. You can also use a PHP code snippet to display the album in the sidebar.\n</p>\n</p>\n</p>\n</p>\n</p>\n<p><a href=\"http://wordpress.org/extend/plugins/maxref-widgets/\"><strong>MaxRef Widgets</strong></a></p>\n<p>MaxRef Widgets is a powerful way to add unlimited widgets on your sidebars to display posts only from a certain category, the links from a particular link category, comments and more.</p>\n<p><a href=\"http://www.weinschenker.name/plugin-feed-reading-blogroll/\"><strong>Feed reading blogroll</strong></a></p>\n<p>Feed reading blogroll provides a sidebar-widget and a template-tag that add an enhanced blogroll to your sidebar. It checks on all your linked sites for which you saved a separate feed-url, when the most recent update has occurred.</p>\";}i:38;a:7:{s:5:\"title\";s:36:\"Matt: Social Networking Sanity Check\";s:4:\"guid\";s:52:\"http://ma.tt/2008/06/social-networking-sanity-check/\";s:4:\"link\";s:52:\"http://ma.tt/2008/06/social-networking-sanity-check/\";s:11:\"description\";s:140:\"<p><a href=\"http://gigaom.com/2008/06/13/social-networking-gets-a-sanity-check/\">Social Networking Gets a Sanity Check</a>, from GigaOM.</p>\";s:7:\"pubdate\";s:31:\"Mon, 16 Jun 2008 16:18:07 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:7:\"summary\";s:140:\"<p><a href=\"http://gigaom.com/2008/06/13/social-networking-gets-a-sanity-check/\">Social Networking Gets a Sanity Check</a>, from GigaOM.</p>\";}i:39;a:7:{s:5:\"title\";s:80:\"Weblog Tools Collection: 10 Video Tutorials for Learning Basic Web Design Skills\";s:4:\"guid\";s:40:\"http://weblogtoolscollection.com/?p=3640\";s:4:\"link\";s:109:\"http://weblogtoolscollection.com/archives/2008/06/16/10-video-tutorials-for-learning-basic-web-design-skills/\";s:11:\"description\";s:471:\"<p><a href=\"http://mashable.com/2008/06/12/web-design-tutorials/\">10 Video Tutorials for Learning Basic Web Design Skills</a>: Some interesting video tutorials linked from Mashable that could help budding web designers or dabblers put a new sheen on their existing blog or website. Of interest are such video tutorials as &#8220;<a href=\"http://max.limpag.com/2006/09/01/video-tutorial-wordpress-theme/\">How to turn any web template into a WordPress theme</a>&#8220;.</p>\";s:7:\"pubdate\";s:31:\"Mon, 16 Jun 2008 13:42:53 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:10:\"Mark Ghosh\";}s:7:\"summary\";s:471:\"<p><a href=\"http://mashable.com/2008/06/12/web-design-tutorials/\">10 Video Tutorials for Learning Basic Web Design Skills</a>: Some interesting video tutorials linked from Mashable that could help budding web designers or dabblers put a new sheen on their existing blog or website. Of interest are such video tutorials as &#8220;<a href=\"http://max.limpag.com/2006/09/01/video-tutorial-wordpress-theme/\">How to turn any web template into a WordPress theme</a>&#8220;.</p>\";}i:40;a:7:{s:5:\"title\";s:45:\"Donncha: Catch website file changes with AIDE\";s:4:\"guid\";s:29:\"http://ocaoimh.ie/?p=89493743\";s:4:\"link\";s:66:\"http://ocaoimh.ie/2008/06/16/catch-website-file-changes-with-aide/\";s:11:\"description\";s:3512:\"<p>A week ago <a href=\"http://ocaoimh.ie/2008/06/08/did-your-wordpress-site-get-hacked/\">I suggested</a> installing <a href=\"http://www.cs.tut.fi/~rammer/aide.html\">AIDE</a> to track changes on your server in case it had been hacked. I think AIDE Is so useful that it deserves a post of it&#8217;s own. Here&#8217;s a short guide to get it working properly.</p>\n<p>The AIDE .deb package includes configuration files for over 80 different software packages or log files. That&#8217;s great if you have all that software installed or want to keep a paranoid eye on /var but what if you only care about the directory where your website lives? </p>\n<p>When I first installed AIDE (using <code>apt-get install aide</code>), it said I needed to run /usr/sbin/aideinit after installation. Every morning I&#8217;d get an email from AIDE with a list of changed files from all over my server, including mail logs, Apache logs, and more. I didn&#8217;t need all that so I removed the files from /etc/aide.conf.d/ except my WordPress config file:<br />\n<blockquote>/home/web/ Checksums<br />\n!/home/web/logs/.*<br />\n!/home/web/public_html/wp-content/cache/.*<br />\n!/home/web/.*/htdocs/wp-content/cache/.*</p></blockquote>\n<p>Unfortunately after I removed the configuration files the daily AIDE email was flooded with open_dir() errors:<br />\n<blockquote>Output is 40577 lines, truncated to 1000.<br />\nopen_dir():Not a directory: /home/donncha/.bashrc<br />\nopen_dir():Not a directory: /home/donncha/.bash_profile<br />\nopen_dir():Not a directory: /home/donncha/.viminfo<br />\nopen_dir():Not a directory: /home/donncha/.bash_history</p></blockquote>\n<p>AIDE was rendered useless by all the errors. Thankfully it was easy to fix. Run aideinit again and it regenerates the AIDE database.<br />\n<blockquote># /usr/sbin/aideinit<br />\nOverwrite existing /var/lib/aide/aide.db.new [Yn]? y<br />\nRunning aide &#8211;init&#8230;</p>\n<p>AIDE, version 0.13.1</p>\n<p>### AIDE database at /var/lib/aide/aide.db.new initialized.</p>\n<p>Overwrite /var/lib/aide/aide.db [yN]? y</p></blockquote>\n<p>For good measure, I ran <code>/etc/cron.daily/aide</code> again which sent me the &#8220;Daily AIDE report&#8221;, and yes, it reported that my .htaccess file had been changed. Nice.</p>\n<p>If your site is on a shared hosting account then you&#8217;re out of luck, but if you have a dedicated host, or virtual private server (VPS) then please consider using AIDE to keep track of changed files. It will send you a short email every day listing changed, added or deleted files. It may save you a lot of hassle and embarrassment if your site is hacked.</p>\n<p>Edit: By default, the nightly cron script doesn&#8217;t update the AIDE database leading to the same files changes reported every day. Edit /etc/default/aide and make sure COPYNEWDB is set to &#8220;yes&#8221;. That will update the database.</p>\n<p><img src=\"http://ocaoimh.ie/?voyeur=1\" /></p><p><strong>Related Posts</strong><ul><li><a href=\"http://ocaoimh.ie/2003/05/02/b2-snapshot-release/\" rel=\"bookmark\" title=\"Permanent Link: b2++ - snapshot release\">b2++ - snapshot release</a></li><li><a href=\"http://ocaoimh.ie/2003/05/20/new-spamassassin-out/\" rel=\"bookmark\" title=\"Permanent Link: New SpamAssassin Out!\">New SpamAssassin Out!</a></li><li><a href=\"http://ocaoimh.ie/2003/04/04/regular-expressions-finding-email-addresses/\" rel=\"bookmark\" title=\"Permanent Link: Regular Expressions: Finding Email Addresses\">Regular Expressions: Finding Email Addresses</a></li></ul></p>\";s:7:\"pubdate\";s:31:\"Mon, 16 Jun 2008 10:48:31 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:7:\"Donncha\";}s:7:\"summary\";s:3512:\"<p>A week ago <a href=\"http://ocaoimh.ie/2008/06/08/did-your-wordpress-site-get-hacked/\">I suggested</a> installing <a href=\"http://www.cs.tut.fi/~rammer/aide.html\">AIDE</a> to track changes on your server in case it had been hacked. I think AIDE Is so useful that it deserves a post of it&#8217;s own. Here&#8217;s a short guide to get it working properly.</p>\n<p>The AIDE .deb package includes configuration files for over 80 different software packages or log files. That&#8217;s great if you have all that software installed or want to keep a paranoid eye on /var but what if you only care about the directory where your website lives? </p>\n<p>When I first installed AIDE (using <code>apt-get install aide</code>), it said I needed to run /usr/sbin/aideinit after installation. Every morning I&#8217;d get an email from AIDE with a list of changed files from all over my server, including mail logs, Apache logs, and more. I didn&#8217;t need all that so I removed the files from /etc/aide.conf.d/ except my WordPress config file:<br />\n<blockquote>/home/web/ Checksums<br />\n!/home/web/logs/.*<br />\n!/home/web/public_html/wp-content/cache/.*<br />\n!/home/web/.*/htdocs/wp-content/cache/.*</p></blockquote>\n<p>Unfortunately after I removed the configuration files the daily AIDE email was flooded with open_dir() errors:<br />\n<blockquote>Output is 40577 lines, truncated to 1000.<br />\nopen_dir():Not a directory: /home/donncha/.bashrc<br />\nopen_dir():Not a directory: /home/donncha/.bash_profile<br />\nopen_dir():Not a directory: /home/donncha/.viminfo<br />\nopen_dir():Not a directory: /home/donncha/.bash_history</p></blockquote>\n<p>AIDE was rendered useless by all the errors. Thankfully it was easy to fix. Run aideinit again and it regenerates the AIDE database.<br />\n<blockquote># /usr/sbin/aideinit<br />\nOverwrite existing /var/lib/aide/aide.db.new [Yn]? y<br />\nRunning aide &#8211;init&#8230;</p>\n<p>AIDE, version 0.13.1</p>\n<p>### AIDE database at /var/lib/aide/aide.db.new initialized.</p>\n<p>Overwrite /var/lib/aide/aide.db [yN]? y</p></blockquote>\n<p>For good measure, I ran <code>/etc/cron.daily/aide</code> again which sent me the &#8220;Daily AIDE report&#8221;, and yes, it reported that my .htaccess file had been changed. Nice.</p>\n<p>If your site is on a shared hosting account then you&#8217;re out of luck, but if you have a dedicated host, or virtual private server (VPS) then please consider using AIDE to keep track of changed files. It will send you a short email every day listing changed, added or deleted files. It may save you a lot of hassle and embarrassment if your site is hacked.</p>\n<p>Edit: By default, the nightly cron script doesn&#8217;t update the AIDE database leading to the same files changes reported every day. Edit /etc/default/aide and make sure COPYNEWDB is set to &#8220;yes&#8221;. That will update the database.</p>\n<p><img src=\"http://ocaoimh.ie/?voyeur=1\" /></p><p><strong>Related Posts</strong><ul><li><a href=\"http://ocaoimh.ie/2003/05/02/b2-snapshot-release/\" rel=\"bookmark\" title=\"Permanent Link: b2++ - snapshot release\">b2++ - snapshot release</a></li><li><a href=\"http://ocaoimh.ie/2003/05/20/new-spamassassin-out/\" rel=\"bookmark\" title=\"Permanent Link: New SpamAssassin Out!\">New SpamAssassin Out!</a></li><li><a href=\"http://ocaoimh.ie/2003/04/04/regular-expressions-finding-email-addresses/\" rel=\"bookmark\" title=\"Permanent Link: Regular Expressions: Finding Email Addresses\">Regular Expressions: Finding Email Addresses</a></li></ul></p>\";}i:41;a:7:{s:5:\"title\";s:30:\"Matt: Seth’s Email Checklist\";s:4:\"guid\";s:20:\"http://ma.tt/?p=6498\";s:4:\"link\";s:43:\"http://ma.tt/2008/06/seths-email-checklist/\";s:11:\"description\";s:244:\"<p><a href=\"http://sethgodin.typepad.com/seths_blog/2008/06/email-checklist.html\">Seth&#8217;s Email Checklist</a>. &#8220;Before you hit send on that next email, perhaps you should run down this list, just to be sure.&#8221; <em>Amen!</em></p>\";s:7:\"pubdate\";s:31:\"Sun, 15 Jun 2008 22:24:35 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:7:\"summary\";s:244:\"<p><a href=\"http://sethgodin.typepad.com/seths_blog/2008/06/email-checklist.html\">Seth&#8217;s Email Checklist</a>. &#8220;Before you hit send on that next email, perhaps you should run down this list, just to be sure.&#8221; <em>Amen!</em></p>\";}i:42;a:7:{s:5:\"title\";s:50:\"Weblog Tools Collection: The Ownership Of Comments\";s:4:\"guid\";s:40:\"http://weblogtoolscollection.com/?p=3648\";s:4:\"link\";s:79:\"http://weblogtoolscollection.com/archives/2008/06/14/the-ownership-of-comments/\";s:11:\"description\";s:2958:\"<p>I&#8217;ve been reading a very interesting discussion over on the <a title=\"http://www.intensedebate.com\" href=\"http://www.intensedebate.com\" target=\"_blank\">IntenseDebate</a> blog with regards to the question, <a title=\"http://www.intensedebate.com/blog/2008/06/02/who-owns-your-comments/\" href=\"http://www.intensedebate.com/blog/2008/06/02/who-owns-your-comments/\" target=\"_blank\">Who Owns Your Comments</a>? IntenseDebate is one of three major third party commenting systems available. The other two are <a title=\"http://disqus.com/\" href=\"http://disqus.com/\" target=\"_blank\">Disqus</a> and <a title=\"http://www.cocomment.com/\" href=\"http://www.cocomment.com/\" target=\"_blank\">CoComment</a>.</p>\n<p>Before I dive into the WordPress aspect of this question, you need to know that IntenseDebate is asking this question because they are a third party commenting system. Therefor, they act as a link between the blog and the commenting system which is outside of the in-house solution provided by publishing platforms such as WordPress. Because of this fact, IntenseDebate needs to figure out how comment editing between the two will work out for the best.</p>\n<p>However, the question IntenseDebate raised can be addressed to the realm of blogging in general. When you browse around WordPress.com or self-hosted WordPress powered blogs and leave a comment, who really owns that comment? You or the blogger? Or should there be shared ownership between the two?</p>\n<p>A number of interesting points were made in the comments of the ID post. For instance, some users didn&#8217;t care who owned their comments just as long as they couldn&#8217;t be changed by the blog author. Others believe that because of the way U.S. copyright law works, commenter&#8217;s retain the ownership and copyright of their comments. While still others believe that a Creative Commons commenting license is in order for Blog owners to display on their site.</p>\n<p>Where do I fall on this issue? Bloggers have had the ability to edit comments for quite some time now. I believe that editing a comment to misconstrue the original comment is morally wrong. In fact, I believe the entire issue of what bloggers do with comments after they have been posted comes down to morality. In my own experience, I have only edited a few comments due to misspelling or the posting of personal information such as phone numbers or email addresses.</p>\n<p>I don&#8217;t believe a commenting bill of rights needs to be created in which all blogs should follow. However, I do think that each blogger should create and make publicly accessible a commenting policy. This policy should clearly explain what you as the blog author will do with comments posted on your site, who retains ownership of those comments and explain circumstances which would require you to edit an end user&#8217;s comment.</p>\n<p>That&#8217;s what I think. Let me know what you think in the <strong>COMMENTS</strong>!</p>\";s:7:\"pubdate\";s:31:\"Sat, 14 Jun 2008 14:09:18 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:13:\"Jeff Chandler\";}s:7:\"summary\";s:2958:\"<p>I&#8217;ve been reading a very interesting discussion over on the <a title=\"http://www.intensedebate.com\" href=\"http://www.intensedebate.com\" target=\"_blank\">IntenseDebate</a> blog with regards to the question, <a title=\"http://www.intensedebate.com/blog/2008/06/02/who-owns-your-comments/\" href=\"http://www.intensedebate.com/blog/2008/06/02/who-owns-your-comments/\" target=\"_blank\">Who Owns Your Comments</a>? IntenseDebate is one of three major third party commenting systems available. The other two are <a title=\"http://disqus.com/\" href=\"http://disqus.com/\" target=\"_blank\">Disqus</a> and <a title=\"http://www.cocomment.com/\" href=\"http://www.cocomment.com/\" target=\"_blank\">CoComment</a>.</p>\n<p>Before I dive into the WordPress aspect of this question, you need to know that IntenseDebate is asking this question because they are a third party commenting system. Therefor, they act as a link between the blog and the commenting system which is outside of the in-house solution provided by publishing platforms such as WordPress. Because of this fact, IntenseDebate needs to figure out how comment editing between the two will work out for the best.</p>\n<p>However, the question IntenseDebate raised can be addressed to the realm of blogging in general. When you browse around WordPress.com or self-hosted WordPress powered blogs and leave a comment, who really owns that comment? You or the blogger? Or should there be shared ownership between the two?</p>\n<p>A number of interesting points were made in the comments of the ID post. For instance, some users didn&#8217;t care who owned their comments just as long as they couldn&#8217;t be changed by the blog author. Others believe that because of the way U.S. copyright law works, commenter&#8217;s retain the ownership and copyright of their comments. While still others believe that a Creative Commons commenting license is in order for Blog owners to display on their site.</p>\n<p>Where do I fall on this issue? Bloggers have had the ability to edit comments for quite some time now. I believe that editing a comment to misconstrue the original comment is morally wrong. In fact, I believe the entire issue of what bloggers do with comments after they have been posted comes down to morality. In my own experience, I have only edited a few comments due to misspelling or the posting of personal information such as phone numbers or email addresses.</p>\n<p>I don&#8217;t believe a commenting bill of rights needs to be created in which all blogs should follow. However, I do think that each blogger should create and make publicly accessible a commenting policy. This policy should clearly explain what you as the blog author will do with comments posted on your site, who retains ownership of those comments and explain circumstances which would require you to edit an end user&#8217;s comment.</p>\n<p>That&#8217;s what I think. Let me know what you think in the <strong>COMMENTS</strong>!</p>\";}i:43;a:7:{s:5:\"title\";s:52:\"Gravatar: Intense Debate Introduces Gravatar Support\";s:4:\"guid\";s:35:\"http://gravatar.wordpress.com/?p=56\";s:4:\"link\";s:79:\"http://blog.gravatar.com/2008/06/13/intense-debate-introduces-gravatar-support/\";s:11:\"description\";s:2175:\"<div class=\"snap_preview\"><br /><p>The folks at comment tracking service <a href=\"http://www.intensedebate.com/\">Intense Debate</a> <a href=\"http://intensedebate.com/blog/\">have rolled out Gravatar support</a>. </p>\n<p>Now all Intense Debate users can choose to use a Gravatar instead of upload a file for their avatar. Just enter your email address and you&#8217;re done. Couldn&#8217;t get much simpler than that.</p>\n<p>Gravatars used through Intense Debate appear on all comments that a user makes on every site powered by Intense Debate, along with links to the user&#8217;s social profiles and other info &#8212; all of which goes along nicely with the &#8220;global&#8221; in Gravatar.</p>\n<p>For more details, check out the post over on <a href=\"http://www.intensedebate.com/blog/2008/06/13/gravatar-support-new-moderation-tools/\">Inside the Debate</a>.</p>\n<img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/categories/gravatar.wordpress.com/56/\" /> <img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/tags/gravatar.wordpress.com/56/\" /> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gocomments/gravatar.wordpress.com/56/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/comments/gravatar.wordpress.com/56/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godelicious/gravatar.wordpress.com/56/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/delicious/gravatar.wordpress.com/56/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gostumble/gravatar.wordpress.com/56/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/stumble/gravatar.wordpress.com/56/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godigg/gravatar.wordpress.com/56/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/digg/gravatar.wordpress.com/56/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/goreddit/gravatar.wordpress.com/56/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/reddit/gravatar.wordpress.com/56/\" /></a> <img alt=\"\" border=\"0\" src=\"http://stats.wordpress.com/b.gif?host=blog.gravatar.com&blog=1886259&post=56&subd=gravatar&ref=&feed=1\" /></div>\";s:7:\"pubdate\";s:31:\"Fri, 13 Jun 2008 22:55:47 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:7:\"Heather\";}s:7:\"summary\";s:2175:\"<div class=\"snap_preview\"><br /><p>The folks at comment tracking service <a href=\"http://www.intensedebate.com/\">Intense Debate</a> <a href=\"http://intensedebate.com/blog/\">have rolled out Gravatar support</a>. </p>\n<p>Now all Intense Debate users can choose to use a Gravatar instead of upload a file for their avatar. Just enter your email address and you&#8217;re done. Couldn&#8217;t get much simpler than that.</p>\n<p>Gravatars used through Intense Debate appear on all comments that a user makes on every site powered by Intense Debate, along with links to the user&#8217;s social profiles and other info &#8212; all of which goes along nicely with the &#8220;global&#8221; in Gravatar.</p>\n<p>For more details, check out the post over on <a href=\"http://www.intensedebate.com/blog/2008/06/13/gravatar-support-new-moderation-tools/\">Inside the Debate</a>.</p>\n<img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/categories/gravatar.wordpress.com/56/\" /> <img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/tags/gravatar.wordpress.com/56/\" /> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gocomments/gravatar.wordpress.com/56/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/comments/gravatar.wordpress.com/56/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godelicious/gravatar.wordpress.com/56/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/delicious/gravatar.wordpress.com/56/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gostumble/gravatar.wordpress.com/56/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/stumble/gravatar.wordpress.com/56/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godigg/gravatar.wordpress.com/56/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/digg/gravatar.wordpress.com/56/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/goreddit/gravatar.wordpress.com/56/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/reddit/gravatar.wordpress.com/56/\" /></a> <img alt=\"\" border=\"0\" src=\"http://stats.wordpress.com/b.gif?host=blog.gravatar.com&blog=1886259&post=56&subd=gravatar&ref=&feed=1\" /></div>\";}i:44;a:7:{s:5:\"title\";s:73:\"Weblog Tools Collection: Useful WordPress Tricks (for the Theme Designer)\";s:4:\"guid\";s:40:\"http://weblogtoolscollection.com/?p=3650\";s:4:\"link\";s:100:\"http://weblogtoolscollection.com/archives/2008/06/13/useful-wordpress-tricks-for-the-theme-designer/\";s:11:\"description\";s:351:\"<p><a href=\"http://stylizedweb.com/2008/06/13/useful-wordpress-tricks/\">Useful WordPress Tricks</a>: StylizedWeb has an interesting article on simple but useful WordPress functions and code tricks that can make your theme succint, short and keep it optimized. Examples are simple to understand an include code to copy and paste to try on your own.</p>\";s:7:\"pubdate\";s:31:\"Fri, 13 Jun 2008 20:59:28 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:10:\"Mark Ghosh\";}s:7:\"summary\";s:351:\"<p><a href=\"http://stylizedweb.com/2008/06/13/useful-wordpress-tricks/\">Useful WordPress Tricks</a>: StylizedWeb has an interesting article on simple but useful WordPress functions and code tricks that can make your theme succint, short and keep it optimized. Examples are simple to understand an include code to copy and paste to try on your own.</p>\";}i:45;a:7:{s:5:\"title\";s:72:\"Mark Jaquith: Ron Paul launches Campaign for Liberty with WordPress blog\";s:4:\"guid\";s:39:\"http://markjaquith.wordpress.com/?p=153\";s:4:\"link\";s:103:\"http://markjaquith.wordpress.com/2008/06/13/ron-paul-launches-campaign-for-liberty-with-wordpress-blog/\";s:11:\"description\";s:1674:\"<div class=\"snap_preview\"><br /><p>Ron Paul formally ended his U.S. presidential campaign and launched &#8220;phase 2&#8243;: the <a href=\"http://www.campaignforliberty.com/\">Campaign for Liberty</a> &#8212; with <a href=\"http://www.campaignforliberty.com/blog/\">the blog powered by WordPress</a>, natch.</p>\n<img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/categories/markjaquith.wordpress.com/153/\" /> <img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/tags/markjaquith.wordpress.com/153/\" /> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gocomments/markjaquith.wordpress.com/153/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/comments/markjaquith.wordpress.com/153/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godelicious/markjaquith.wordpress.com/153/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/delicious/markjaquith.wordpress.com/153/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gostumble/markjaquith.wordpress.com/153/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/stumble/markjaquith.wordpress.com/153/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godigg/markjaquith.wordpress.com/153/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/digg/markjaquith.wordpress.com/153/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/goreddit/markjaquith.wordpress.com/153/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/reddit/markjaquith.wordpress.com/153/\" /></a> <img alt=\"\" border=\"0\" src=\"http://stats.wordpress.com/b.gif?host=markjaquith.wordpress.com&blog=316&post=153&subd=markjaquith&ref=&feed=1\" /></div>\";s:7:\"pubdate\";s:31:\"Fri, 13 Jun 2008 18:16:14 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:12:\"Mark Jaquith\";}s:7:\"summary\";s:1674:\"<div class=\"snap_preview\"><br /><p>Ron Paul formally ended his U.S. presidential campaign and launched &#8220;phase 2&#8243;: the <a href=\"http://www.campaignforliberty.com/\">Campaign for Liberty</a> &#8212; with <a href=\"http://www.campaignforliberty.com/blog/\">the blog powered by WordPress</a>, natch.</p>\n<img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/categories/markjaquith.wordpress.com/153/\" /> <img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/tags/markjaquith.wordpress.com/153/\" /> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gocomments/markjaquith.wordpress.com/153/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/comments/markjaquith.wordpress.com/153/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godelicious/markjaquith.wordpress.com/153/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/delicious/markjaquith.wordpress.com/153/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gostumble/markjaquith.wordpress.com/153/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/stumble/markjaquith.wordpress.com/153/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godigg/markjaquith.wordpress.com/153/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/digg/markjaquith.wordpress.com/153/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/goreddit/markjaquith.wordpress.com/153/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/reddit/markjaquith.wordpress.com/153/\" /></a> <img alt=\"\" border=\"0\" src=\"http://stats.wordpress.com/b.gif?host=markjaquith.wordpress.com&blog=316&post=153&subd=markjaquith&ref=&feed=1\" /></div>\";}i:46;a:7:{s:5:\"title\";s:26:\"Matt: New BuddyPress Theme\";s:4:\"guid\";s:42:\"http://ma.tt/2008/06/new-buddypress-theme/\";s:4:\"link\";s:42:\"http://ma.tt/2008/06/new-buddypress-theme/\";s:11:\"description\";s:287:\"<p><a href=\"http://apeatling.wordpress.com/2008/06/12/new-buddypress-theme/\">A preview of the upcoming theme for BuddyPress</a>. Be sure to click through to the full-size versions. BuddyPress is coming along nicely, and a good example of what is possible with the WordPress platform.</p>\";s:7:\"pubdate\";s:31:\"Fri, 13 Jun 2008 14:57:00 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"Matt\";}s:7:\"summary\";s:287:\"<p><a href=\"http://apeatling.wordpress.com/2008/06/12/new-buddypress-theme/\">A preview of the upcoming theme for BuddyPress</a>. Be sure to click through to the full-size versions. BuddyPress is coming along nicely, and a good example of what is possible with the WordPress platform.</p>\";}i:47;a:7:{s:5:\"title\";s:58:\"Weblog Tools Collection: WordPress Theme Releases For 6/12\";s:4:\"guid\";s:86:\"http://weblogtoolscollection.com/archives/2008/06/12/wordpress-theme-releases-for-612/\";s:4:\"link\";s:86:\"http://weblogtoolscollection.com/archives/2008/06/12/wordpress-theme-releases-for-612/\";s:11:\"description\";s:2975:\"<h3>Two Column Themes</h3>\n<p><a href=\"http://www.flisterz.com/2008/05/31/faleonis-wordpress-theme/\" target=\"_blank\"><strong>Faleonis</strong></a></p>\n<p><img title=\"faleonis-thumbnail\" height=\"135\" alt=\"faleonis-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/faleonisthumbnail.png\" width=\"200\" border=\"0\" /> </p>\n<p>Faleonis is a two column widget-ready theme with inbuilt gravatar support WordPress 2.5 and above. The theme also shows a gravatar of the author at the end of each post.</p>\n<p><a href=\"http://www.pancak.es/themes/syrup/\" target=\"_blank\"><strong>Syrup</strong></a></p>\n<p><img title=\"syrup-thumbnail\" height=\"98\" alt=\"syrup-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/syrupthumbnail.png\" width=\"200\" border=\"0\" /> </p>\n<p>Syrup is a two-column widget-ready theme which provides users with extensive options panel to control the look and feel of the theme. You can manage custom color schemes, layouts, sidebar positioning, font sizes and more from the admin panel.</p>\n<p><a href=\"http://www.askgraphics.com/freetemplates/dark-classic-theme/\" target=\"_blank\"><strong>DarkClassic</strong></a></p>\n<p><img title=\"darkclassic-thumbnail\" height=\"125\" alt=\"darkclassic-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/darkclassicthumbnail.png\" width=\"200\" border=\"0\" /> </p>\n<p>DarkClassic is a two column widget-ready theme made up of black color with a green tinge. The theme has built in tag support.</p>\n<p><a href=\"http://vagrantconcept.com/downloads/gymmetry\" target=\"_blank\"><strong>Gymmetry</strong></a></p>\n<p><img title=\"gymmetry-thumbnail\" height=\"95\" alt=\"gymmetry-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/gymmetrythumbnail.png\" width=\"149\" border=\"0\" /> </p>\n<p>Gymmetry is a two column widget-ready theme. You can customize the blog title and sidebar styles along with the header of the theme. The theme can also be customized to use 4 different colors.</p>\n<p><a href=\"http://www.wp-magazine.se/free-wordpress-theme-wp-newsmag/\" target=\"_blank\"><strong>WP Newsmag</strong></a></p>\n<p><img title=\"wp-newsmag-thumbnail\" height=\"121\" alt=\"wp-newsmag-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/wpnewsmagthumbnail.png\" width=\"219\" border=\"0\" /> </p>\n<p>WP Newsmag is a two column widget-ready magazine style theme. The theme has slider functions for the bottom bar and featured articles.</p>\n<h3>Three Column Themes</h3>\n<p><a href=\"http://mindfaucet.com/comicpress/\" target=\"_blank\"><strong>ComicPress</strong></a></p>\n<p><img title=\"comicpress-thumbnail\" height=\"156\" alt=\"comicpress-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/comicpressthumbnail.png\" width=\"200\" border=\"0\" /> </p>\n<p>ComicPress is a theme made specially for blogs that publish comics, the latest edition of this theme has 5 different layouts which you can choose from. The theme also comes built in with its own manager to configure the theme and upload comics.</p>\";s:7:\"pubdate\";s:31:\"Fri, 13 Jun 2008 03:55:00 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:12:\"Keith Dsouza\";}s:7:\"summary\";s:2975:\"<h3>Two Column Themes</h3>\n<p><a href=\"http://www.flisterz.com/2008/05/31/faleonis-wordpress-theme/\" target=\"_blank\"><strong>Faleonis</strong></a></p>\n<p><img title=\"faleonis-thumbnail\" height=\"135\" alt=\"faleonis-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/faleonisthumbnail.png\" width=\"200\" border=\"0\" /> </p>\n<p>Faleonis is a two column widget-ready theme with inbuilt gravatar support WordPress 2.5 and above. The theme also shows a gravatar of the author at the end of each post.</p>\n<p><a href=\"http://www.pancak.es/themes/syrup/\" target=\"_blank\"><strong>Syrup</strong></a></p>\n<p><img title=\"syrup-thumbnail\" height=\"98\" alt=\"syrup-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/syrupthumbnail.png\" width=\"200\" border=\"0\" /> </p>\n<p>Syrup is a two-column widget-ready theme which provides users with extensive options panel to control the look and feel of the theme. You can manage custom color schemes, layouts, sidebar positioning, font sizes and more from the admin panel.</p>\n<p><a href=\"http://www.askgraphics.com/freetemplates/dark-classic-theme/\" target=\"_blank\"><strong>DarkClassic</strong></a></p>\n<p><img title=\"darkclassic-thumbnail\" height=\"125\" alt=\"darkclassic-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/darkclassicthumbnail.png\" width=\"200\" border=\"0\" /> </p>\n<p>DarkClassic is a two column widget-ready theme made up of black color with a green tinge. The theme has built in tag support.</p>\n<p><a href=\"http://vagrantconcept.com/downloads/gymmetry\" target=\"_blank\"><strong>Gymmetry</strong></a></p>\n<p><img title=\"gymmetry-thumbnail\" height=\"95\" alt=\"gymmetry-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/gymmetrythumbnail.png\" width=\"149\" border=\"0\" /> </p>\n<p>Gymmetry is a two column widget-ready theme. You can customize the blog title and sidebar styles along with the header of the theme. The theme can also be customized to use 4 different colors.</p>\n<p><a href=\"http://www.wp-magazine.se/free-wordpress-theme-wp-newsmag/\" target=\"_blank\"><strong>WP Newsmag</strong></a></p>\n<p><img title=\"wp-newsmag-thumbnail\" height=\"121\" alt=\"wp-newsmag-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/wpnewsmagthumbnail.png\" width=\"219\" border=\"0\" /> </p>\n<p>WP Newsmag is a two column widget-ready magazine style theme. The theme has slider functions for the bottom bar and featured articles.</p>\n<h3>Three Column Themes</h3>\n<p><a href=\"http://mindfaucet.com/comicpress/\" target=\"_blank\"><strong>ComicPress</strong></a></p>\n<p><img title=\"comicpress-thumbnail\" height=\"156\" alt=\"comicpress-thumbnail\" src=\"http://weblogtoolscollection.com/b2-img/2008/06/comicpressthumbnail.png\" width=\"200\" border=\"0\" /> </p>\n<p>ComicPress is a theme made specially for blogs that publish comics, the latest edition of this theme has 5 different layouts which you can choose from. The theme also comes built in with its own manager to configure the theme and upload comics.</p>\";}i:48;a:7:{s:5:\"title\";s:24:\"Mike Little: WordCamp UK\";s:4:\"guid\";s:60:\"http://zed1.com/journalized/archives/2008/06/12/wordcamp-uk/\";s:4:\"link\";s:60:\"http://zed1.com/journalized/archives/2008/06/12/wordcamp-uk/\";s:11:\"description\";s:470:\"<p><a href=\"http://uk.wordcamp.org/\"><img src=\"http://zed1.com/images/wordcamp-logo.png\" alt=\"WordCamp UK Logo\" /></a></p>\n<p>WordCamp UK is an informal gathering of WordPress bloggers, podcasters, designers and developers based in the United Kingdom.</p>\n<p>When? - Saturday and Sunday 19-20 July 2008<br />\nWhere? - Aston University Lakeside Centre, Birmingham</p>\n<p>I&#8217;ll be there, will you?</p>\n<p><a href=\"http://uk.wordcamp.org/\"><br />\nFind out more</a></p>\";s:7:\"pubdate\";s:31:\"Thu, 12 Jun 2008 23:06:40 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:4:\"mike\";}s:7:\"summary\";s:470:\"<p><a href=\"http://uk.wordcamp.org/\"><img src=\"http://zed1.com/images/wordcamp-logo.png\" alt=\"WordCamp UK Logo\" /></a></p>\n<p>WordCamp UK is an informal gathering of WordPress bloggers, podcasters, designers and developers based in the United Kingdom.</p>\n<p>When? - Saturday and Sunday 19-20 July 2008<br />\nWhere? - Aston University Lakeside Centre, Birmingham</p>\n<p>I&#8217;ll be there, will you?</p>\n<p><a href=\"http://uk.wordcamp.org/\"><br />\nFind out more</a></p>\";}i:49;a:7:{s:5:\"title\";s:49:\"Mark Jaquith: Burning Man starts a WordPress blog\";s:4:\"guid\";s:39:\"http://markjaquith.wordpress.com/?p=152\";s:4:\"link\";s:80:\"http://markjaquith.wordpress.com/2008/06/12/burning-man-starts-a-wordpress-blog/\";s:11:\"description\";s:1994:\"<div class=\"snap_preview\"><br /><p>Burning Man, the almost impossible to encapsulate festival of art, hedonism and counter-cultural mayhem has <a href=\"http://blog.burningman.com/\">launched a blog</a>, powered by WordPress.  Thanks to <a href=\"http://laughingsquid.com/burning-man-launches-burning-blog/\">Scott Beale</a> for the tip.</p>\n<p>Question for readers: what is the <em>last</em> organization in the world you&#8217;d expect to start blogging?</p>\n<p>My pick: The NSA &#8212; but when they do start blogging, they&#8217;ll just publish all those drafts you thought you deleted from your blog.  Snare, snare, cymbal.</p>\n<img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/categories/markjaquith.wordpress.com/152/\" /> <img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/tags/markjaquith.wordpress.com/152/\" /> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gocomments/markjaquith.wordpress.com/152/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/comments/markjaquith.wordpress.com/152/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godelicious/markjaquith.wordpress.com/152/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/delicious/markjaquith.wordpress.com/152/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gostumble/markjaquith.wordpress.com/152/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/stumble/markjaquith.wordpress.com/152/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godigg/markjaquith.wordpress.com/152/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/digg/markjaquith.wordpress.com/152/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/goreddit/markjaquith.wordpress.com/152/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/reddit/markjaquith.wordpress.com/152/\" /></a> <img alt=\"\" border=\"0\" src=\"http://stats.wordpress.com/b.gif?host=markjaquith.wordpress.com&blog=316&post=152&subd=markjaquith&ref=&feed=1\" /></div>\";s:7:\"pubdate\";s:31:\"Thu, 12 Jun 2008 17:02:40 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:12:\"Mark Jaquith\";}s:7:\"summary\";s:1994:\"<div class=\"snap_preview\"><br /><p>Burning Man, the almost impossible to encapsulate festival of art, hedonism and counter-cultural mayhem has <a href=\"http://blog.burningman.com/\">launched a blog</a>, powered by WordPress.  Thanks to <a href=\"http://laughingsquid.com/burning-man-launches-burning-blog/\">Scott Beale</a> for the tip.</p>\n<p>Question for readers: what is the <em>last</em> organization in the world you&#8217;d expect to start blogging?</p>\n<p>My pick: The NSA &#8212; but when they do start blogging, they&#8217;ll just publish all those drafts you thought you deleted from your blog.  Snare, snare, cymbal.</p>\n<img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/categories/markjaquith.wordpress.com/152/\" /> <img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/tags/markjaquith.wordpress.com/152/\" /> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gocomments/markjaquith.wordpress.com/152/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/comments/markjaquith.wordpress.com/152/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godelicious/markjaquith.wordpress.com/152/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/delicious/markjaquith.wordpress.com/152/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/gostumble/markjaquith.wordpress.com/152/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/stumble/markjaquith.wordpress.com/152/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/godigg/markjaquith.wordpress.com/152/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/digg/markjaquith.wordpress.com/152/\" /></a> <a rel=\"nofollow\" href=\"http://feeds.wordpress.com/1.0/goreddit/markjaquith.wordpress.com/152/\"><img alt=\"\" border=\"0\" src=\"http://feeds.wordpress.com/1.0/reddit/markjaquith.wordpress.com/152/\" /></a> <img alt=\"\" border=\"0\" src=\"http://stats.wordpress.com/b.gif?host=markjaquith.wordpress.com&blog=316&post=152&subd=markjaquith&ref=&feed=1\" /></div>\";}}s:7:\"channel\";a:5:{s:5:\"title\";s:16:\"WordPress Planet\";s:4:\"link\";s:28:\"http://planet.wordpress.org/\";s:8:\"language\";s:2:\"en\";s:11:\"description\";s:47:\"WordPress Planet - http://planet.wordpress.org/\";s:7:\"tagline\";s:47:\"WordPress Planet - http://planet.wordpress.org/\";}s:9:\"textinput\";a:0:{}s:5:\"image\";a:0:{}s:9:\"feed_type\";s:3:\"RSS\";s:12:\"feed_version\";s:3:\"2.0\";s:5:\"stack\";a:0:{}s:9:\"inchannel\";b:0;s:6:\"initem\";b:0;s:9:\"incontent\";b:0;s:11:\"intextinput\";b:0;s:7:\"inimage\";b:0;s:13:\"current_field\";s:0:\"\";s:17:\"current_namespace\";b:0;s:19:\"_CONTENT_CONSTRUCTS\";a:6:{i:0;s:7:\"content\";i:1;s:7:\"summary\";i:2;s:4:\"info\";i:3;s:5:\"title\";i:4;s:7:\"tagline\";i:5;s:9:\"copyright\";}s:4:\"etag\";s:25:\"\"22f80-4868fc43-c10597\"\r\n\";s:13:\"last_modified\";s:31:\"Mon, 30 Jun 2008 15:31:15 GMT\r\n\";}','no'),(88,0,'rss_867bd5c64f85878d03a060509cd2f92c_ts','1214839989','no'),(89,0,'rss_1a5f760f2e2b48827d4974a60857e7c2','O:9:\"MagpieRSS\":19:{s:6:\"parser\";i:0;s:12:\"current_item\";a:6:{s:5:\"title\";s:25:\"denishua on \"Top Friends\"\";s:4:\"link\";s:58:\"http://wordpress.org/extend/plugins/top-friends/#post-5829\";s:7:\"pubdate\";s:31:\"Tue, 03 Jun 2008 08:48:04 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:8:\"denishua\";}s:4:\"guid\";s:41:\"5829@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:90:\"Top Friends is a WordPress blogroll enhancement plugin. The plugin will fetch your friends\";}s:5:\"items\";a:11:{i:0;a:7:{s:5:\"title\";s:30:\"datafeedr.com on \"Plugin Name\"\";s:4:\"link\";s:57:\"http://wordpress.org/extend/plugins/random-ads/#post-6095\";s:7:\"pubdate\";s:31:\"Wed, 25 Jun 2008 19:33:50 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:13:\"datafeedr.com\";}s:4:\"guid\";s:41:\"6095@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:139:\"The Datafeedr Random Ads plugin allows you to simply and easily show random, rotating ads anywhere in your template files or using widgets.\";s:7:\"summary\";s:139:\"The Datafeedr Random Ads plugin allows you to simply and easily show random, rotating ads anywhere in your template files or using widgets.\";}i:1;a:7:{s:5:\"title\";s:27:\"dirigenteitis on \"WDM News\"\";s:4:\"link\";s:55:\"http://wordpress.org/extend/plugins/wdm-news/#post-4995\";s:7:\"pubdate\";s:31:\"Sun, 30 Mar 2008 09:17:12 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:13:\"dirigenteitis\";}s:4:\"guid\";s:41:\"4995@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:81:\"This plugin show like widget a little sentence with date and time of publication.\";s:7:\"summary\";s:81:\"This plugin show like widget a little sentence with date and time of publication.\";}i:2;a:7:{s:5:\"title\";s:23:\"Bueltge on \"Adminimize\"\";s:4:\"link\";s:57:\"http://wordpress.org/extend/plugins/adminimize/#post-5921\";s:7:\"pubdate\";s:31:\"Wed, 11 Jun 2008 10:28:52 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:7:\"Bueltge\";}s:4:\"guid\";s:41:\"5921@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:150:\"At first: Visually compresses the administratrive header so that more admin page content can be initially seen. Also moves \'Dashboard\' onto the main a\";s:7:\"summary\";s:150:\"At first: Visually compresses the administratrive header so that more admin page content can be initially seen. Also moves \'Dashboard\' onto the main a\";}i:3;a:7:{s:5:\"title\";s:38:\"microkid on \"Microkid\'s Related Posts\"\";s:4:\"link\";s:70:\"http://wordpress.org/extend/plugins/microkids-related-posts/#post-6098\";s:7:\"pubdate\";s:31:\"Thu, 26 Jun 2008 11:34:02 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:8:\"microkid\";}s:4:\"guid\";s:41:\"6098@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:113:\"Microkid\'s Related Posts plugin lets you manually select related posts using a nifty AJAX powered search utility.\";s:7:\"summary\";s:113:\"Microkid\'s Related Posts plugin lets you manually select related posts using a nifty AJAX powered search utility.\";}i:4;a:7:{s:5:\"title\";s:34:\"chrisbliss18 on \"WP Easy Uploader\"\";s:4:\"link\";s:63:\"http://wordpress.org/extend/plugins/wp-easy-uploader/#post-6117\";s:7:\"pubdate\";s:31:\"Sat, 28 Jun 2008 04:21:40 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:12:\"chrisbliss18\";}s:4:\"guid\";s:41:\"6117@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:188:\"Easily upload any type of content without the need for <a href=\"http://FTP.\" rel=\"nofollow\">FTP.</a> You can even upload plugin and theme archives, and the files will be extracted for you.\";s:7:\"summary\";s:188:\"Easily upload any type of content without the need for <a href=\"http://FTP.\" rel=\"nofollow\">FTP.</a> You can even upload plugin and theme archives, and the files will be extracted for you.\";}i:5;a:7:{s:5:\"title\";s:29:\"plugin-master on \"d13gallery\"\";s:4:\"link\";s:57:\"http://wordpress.org/extend/plugins/d13gallery/#post-5534\";s:7:\"pubdate\";s:31:\"Wed, 07 May 2008 13:52:12 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:13:\"plugin-master\";}s:4:\"guid\";s:41:\"5534@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:97:\"Simply and quickly add thumbnail image galleries to your posts and pages using a html-like tag...\";s:7:\"summary\";s:97:\"Simply and quickly add thumbnail image galleries to your posts and pages using a html-like tag...\";}i:6;a:7:{s:5:\"title\";s:34:\"Trouts on \"Rot13 Email Protection\"\";s:4:\"link\";s:69:\"http://wordpress.org/extend/plugins/rot13-email-protection/#post-6103\";s:7:\"pubdate\";s:31:\"Thu, 26 Jun 2008 19:35:02 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:6:\"Trouts\";}s:4:\"guid\";s:41:\"6103@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:122:\"Protects all email addresses and mailto: links on your blog from email harvesters using rot13 encryption and/or text mode.\";s:7:\"summary\";s:122:\"Protects all email addresses and mailto: links on your blog from email harvesters using rot13 encryption and/or text mode.\";}i:7;a:7:{s:5:\"title\";s:29:\"jsruok on \"Blog Introduction\"\";s:4:\"link\";s:64:\"http://wordpress.org/extend/plugins/blog-introduction/#post-5159\";s:7:\"pubdate\";s:31:\"Wed, 09 Apr 2008 12:38:43 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:6:\"jsruok\";}s:4:\"guid\";s:41:\"5159@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:130:\"Blog Introduction inserts a static intro before posts on home page.\nIntroduction content is taken from a page that can be defined.\";s:7:\"summary\";s:130:\"Blog Introduction inserts a static intro before posts on home page.\nIntroduction content is taken from a page that can be defined.\";}i:8;a:7:{s:5:\"title\";s:26:\"bcallaars on \"WP-AlertBox\"\";s:4:\"link\";s:58:\"http://wordpress.org/extend/plugins/wp-alertbox/#post-6079\";s:7:\"pubdate\";s:31:\"Tue, 24 Jun 2008 11:22:38 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:9:\"bcallaars\";}s:4:\"guid\";s:41:\"6079@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:100:\"A simple alert box configureable by the admin. User get\'s a overlay div with the alert message once.\";s:7:\"summary\";s:100:\"A simple alert box configureable by the admin. User get\'s a overlay div with the alert message once.\";}i:9;a:7:{s:5:\"title\";s:24:\"WebGeek on \"WP-SpamFree\"\";s:4:\"link\";s:58:\"http://wordpress.org/extend/plugins/wp-spamfree/#post-2654\";s:7:\"pubdate\";s:31:\"Sat, 15 Dec 2007 12:10:04 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:7:\"WebGeek\";}s:4:\"guid\";s:41:\"2654@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:139:\"Powerful anti-spam plugin that virtually eliminates comment spam. Finally, you can enjoy a spam-free WordPress blog! Includes contact form.\";s:7:\"summary\";s:139:\"Powerful anti-spam plugin that virtually eliminates comment spam. Finally, you can enjoy a spam-free WordPress blog! Includes contact form.\";}i:10;a:7:{s:5:\"title\";s:33:\"uberdose on \"All in One SEO Pack\"\";s:4:\"link\";s:65:\"http://wordpress.org/extend/plugins/all-in-one-seo-pack/#post-753\";s:7:\"pubdate\";s:31:\"Fri, 30 Mar 2007 20:08:18 +0000\";s:2:\"dc\";a:1:{s:7:\"creator\";s:8:\"uberdose\";}s:4:\"guid\";s:40:\"753@http://wordpress.org/extend/plugins/\";s:11:\"description\";s:92:\"Automatically optimizes your Wordpress blog for Search Engines (Search Engine Optimization).\";s:7:\"summary\";s:92:\"Automatically optimizes your Wordpress blog for Search Engines (Search Engine Optimization).\";}}s:7:\"channel\";a:6:{s:5:\"title\";s:16:\"Recently Updated\";s:4:\"link\";s:36:\"http://wordpress.org/extend/plugins/\";s:11:\"description\";s:16:\"Recently Updated\";s:8:\"language\";s:2:\"en\";s:7:\"pubdate\";s:31:\"Mon, 30 Jun 2008 15:33:12 +0000\";s:7:\"tagline\";s:16:\"Recently Updated\";}s:9:\"textinput\";a:0:{}s:5:\"image\";a:0:{}s:9:\"feed_type\";s:3:\"RSS\";s:12:\"feed_version\";s:3:\"2.0\";s:5:\"stack\";a:1:{i:0;s:11:\"description\";}s:9:\"inchannel\";b:1;s:6:\"initem\";b:1;s:9:\"incontent\";b:0;s:11:\"intextinput\";b:0;s:7:\"inimage\";b:0;s:13:\"current_field\";s:0:\"\";s:17:\"current_namespace\";b:0;s:19:\"_CONTENT_CONSTRUCTS\";a:6:{i:0;s:7:\"content\";i:1;s:7:\"summary\";i:2;s:4:\"info\";i:3;s:5:\"title\";i:4;s:7:\"tagline\";i:5;s:9:\"copyright\";}s:13:\"last_modified\";s:21:\"2008-06-25 19:33:50\r\n\";s:4:\"etag\";s:36:\"\"4ac009b93f30f633530425045ced3c4b\"\r\n\";}','no'),(90,0,'rss_1a5f760f2e2b48827d4974a60857e7c2_ts','1214839989','no'),(96,0,'category_children','a:0:{}','yes');
/*!40000 ALTER TABLE `options` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `postmeta`
--

DROP TABLE IF EXISTS `postmeta`;
CREATE TABLE `postmeta` (
  `meta_id` bigint(20) NOT NULL auto_increment,
  `post_id` bigint(20) NOT NULL default '0',
  `meta_key` varchar(255) default NULL,
  `meta_value` longtext,
  PRIMARY KEY  (`meta_id`),
  KEY `post_id` (`post_id`),
  KEY `meta_key` (`meta_key`)
) ENGINE=MyISAM AUTO_INCREMENT=35 DEFAULT CHARSET=utf8;

--
-- Dumping data for table `postmeta`
--

LOCK TABLES `postmeta` WRITE;
/*!40000 ALTER TABLE `postmeta` DISABLE KEYS */;
INSERT INTO `postmeta` VALUES (1,3,'_edit_lock','1214840190'),(2,3,'_edit_last','1'),(4,6,'_edit_lock','1214668676'),(5,6,'_edit_last','1'),(6,13,'_edit_lock','1214760872'),(7,13,'_edit_last','1'),(8,14,'_edit_last','1'),(9,14,'_edit_lock','1214668521'),(11,0,'_edit_lock','1214668676'),(12,0,'_edit_last','1'),(13,16,'_wp_attached_file','/home/kkfungne/public_html/abunchofgeniuses/blog/wp-content/uploads/2008/03/head-measurement-1-feb-23-2008.jpg'),(14,16,'_wp_attachment_metadata','a:6:{s:5:\"width\";i:480;s:6:\"height\";i:386;s:14:\"hwstring_small\";s:23:\"height=\'96\' width=\'119\'\";s:4:\"file\";s:110:\"/home/kkfungne/public_html/abunchofgeniuses/blog/wp-content/uploads/2008/03/head-measurement-1-feb-23-2008.jpg\";s:5:\"sizes\";a:2:{s:9:\"thumbnail\";a:3:{s:4:\"file\";s:42:\"head-measurement-1-feb-23-2008-150x150.jpg\";s:5:\"width\";i:150;s:6:\"height\";i:150;}s:6:\"medium\";a:3:{s:4:\"file\";s:42:\"head-measurement-1-feb-23-2008-300x241.jpg\";s:5:\"width\";i:300;s:6:\"height\";i:241;}}s:10:\"image_meta\";a:10:{s:8:\"aperture\";i:0;s:6:\"credit\";s:10:\"Picasa 2.7\";s:6:\"camera\";s:0:\"\";s:7:\"caption\";s:0:\"\";s:17:\"created_timestamp\";i:0;s:9:\"copyright\";s:0:\"\";s:12:\"focal_length\";i:0;s:3:\"iso\";i:0;s:13:\"shutter_speed\";i:0;s:5:\"title\";s:0:\"\";}}'),(15,16,'_wp_attachment_metadata','s:221:\"a:4:{s:5:\"width\";i:2288;s:6:\"height\";i:1838;s:14:\"hwstring_small\";s:23:\"height=\'96\' width=\'119\'\";s:4:\"file\";s:103:\"/home/kkfungne/public_html/chingfung/blog/wp-content/uploads/2008/03/head-measurement-1-feb-23-2008.jpg\";}\";'),(16,17,'_wp_attached_file','/home/kkfungne/public_html/abunchofgeniuses/blog/wp-content/uploads/2008/03/body-length-jan-26-2008.jpg'),(17,17,'_wp_attachment_metadata','a:6:{s:5:\"width\";i:480;s:6:\"height\";i:374;s:14:\"hwstring_small\";s:23:\"height=\'96\' width=\'123\'\";s:4:\"file\";s:103:\"/home/kkfungne/public_html/abunchofgeniuses/blog/wp-content/uploads/2008/03/body-length-jan-26-2008.jpg\";s:5:\"sizes\";a:2:{s:9:\"thumbnail\";a:3:{s:4:\"file\";s:35:\"body-length-jan-26-2008-150x150.jpg\";s:5:\"width\";i:150;s:6:\"height\";i:150;}s:6:\"medium\";a:3:{s:4:\"file\";s:35:\"body-length-jan-26-2008-300x233.jpg\";s:5:\"width\";i:300;s:6:\"height\";i:233;}}s:10:\"image_meta\";a:10:{s:8:\"aperture\";i:0;s:6:\"credit\";s:10:\"Picasa 2.7\";s:6:\"camera\";s:0:\"\";s:7:\"caption\";s:0:\"\";s:17:\"created_timestamp\";i:0;s:9:\"copyright\";s:0:\"\";s:12:\"focal_length\";i:0;s:3:\"iso\";i:0;s:13:\"shutter_speed\";i:0;s:5:\"title\";s:0:\"\";}}'),(18,17,'_wp_attachment_metadata','s:268:\"a:5:{s:5:\"width\";i:480;s:6:\"height\";i:374;s:14:\"hwstring_small\";s:23:\"height=\'96\' width=\'123\'\";s:4:\"file\";s:96:\"/home/kkfungne/public_html/chingfung/blog/wp-content/uploads/2008/03/body-length-jan-26-2008.jpg\";s:5:\"thumb\";s:37:\"body-length-jan-26-2008.thumbnail.jpg\";}\";'),(19,18,'_wp_attached_file','/home/kkfungne/public_html/abunchofgeniuses/blog/wp-content/uploads/2008/03/head-measurement-1-feb-23-20081.jpg'),(20,18,'_wp_attachment_metadata','a:6:{s:5:\"width\";i:480;s:6:\"height\";i:386;s:14:\"hwstring_small\";s:23:\"height=\'96\' width=\'119\'\";s:4:\"file\";s:111:\"/home/kkfungne/public_html/abunchofgeniuses/blog/wp-content/uploads/2008/03/head-measurement-1-feb-23-20081.jpg\";s:5:\"sizes\";a:2:{s:9:\"thumbnail\";a:3:{s:4:\"file\";s:43:\"head-measurement-1-feb-23-20081-150x150.jpg\";s:5:\"width\";i:150;s:6:\"height\";i:150;}s:6:\"medium\";a:3:{s:4:\"file\";s:43:\"head-measurement-1-feb-23-20081-300x241.jpg\";s:5:\"width\";i:300;s:6:\"height\";i:241;}}s:10:\"image_meta\";a:10:{s:8:\"aperture\";i:0;s:6:\"credit\";s:10:\"Picasa 2.7\";s:6:\"camera\";s:0:\"\";s:7:\"caption\";s:0:\"\";s:17:\"created_timestamp\";i:0;s:9:\"copyright\";s:0:\"\";s:12:\"focal_length\";i:0;s:3:\"iso\";i:0;s:13:\"shutter_speed\";i:0;s:5:\"title\";s:0:\"\";}}'),(21,18,'_wp_attachment_metadata','s:283:\"a:5:{s:5:\"width\";i:480;s:6:\"height\";i:386;s:14:\"hwstring_small\";s:23:\"height=\'96\' width=\'119\'\";s:4:\"file\";s:103:\"/home/kkfungne/public_html/chingfung/blog/wp-content/uploads/2008/03/head-measurement-1-feb-23-2008.jpg\";s:5:\"thumb\";s:44:\"head-measurement-1-feb-23-2008.thumbnail.jpg\";}\";'),(22,19,'_wp_attached_file','/home/kkfungne/public_html/abunchofgeniuses/blog/wp-content/uploads/2008/03/head-measurement-2-feb-23-2008.jpg'),(23,19,'_wp_attachment_metadata','a:6:{s:5:\"width\";i:480;s:6:\"height\";i:382;s:14:\"hwstring_small\";s:23:\"height=\'96\' width=\'120\'\";s:4:\"file\";s:110:\"/home/kkfungne/public_html/abunchofgeniuses/blog/wp-content/uploads/2008/03/head-measurement-2-feb-23-2008.jpg\";s:5:\"sizes\";a:2:{s:9:\"thumbnail\";a:3:{s:4:\"file\";s:42:\"head-measurement-2-feb-23-2008-150x150.jpg\";s:5:\"width\";i:150;s:6:\"height\";i:150;}s:6:\"medium\";a:3:{s:4:\"file\";s:42:\"head-measurement-2-feb-23-2008-300x238.jpg\";s:5:\"width\";i:300;s:6:\"height\";i:238;}}s:10:\"image_meta\";a:10:{s:8:\"aperture\";i:0;s:6:\"credit\";s:10:\"Picasa 2.7\";s:6:\"camera\";s:0:\"\";s:7:\"caption\";s:0:\"\";s:17:\"created_timestamp\";i:0;s:9:\"copyright\";s:0:\"\";s:12:\"focal_length\";i:0;s:3:\"iso\";i:0;s:13:\"shutter_speed\";i:0;s:5:\"title\";s:0:\"\";}}'),(24,19,'_wp_attachment_metadata','s:283:\"a:5:{s:5:\"width\";i:480;s:6:\"height\";i:382;s:14:\"hwstring_small\";s:23:\"height=\'96\' width=\'120\'\";s:4:\"file\";s:103:\"/home/kkfungne/public_html/chingfung/blog/wp-content/uploads/2008/03/head-measurement-2-feb-23-2008.jpg\";s:5:\"thumb\";s:44:\"head-measurement-2-feb-23-2008.thumbnail.jpg\";}\";'),(25,20,'_wp_attached_file','/home/kkfungne/public_html/abunchofgeniuses/blog/wp-content/uploads/2008/03/head-measurement-jan-26-2008.jpg'),(26,20,'_wp_attachment_metadata','a:6:{s:5:\"width\";i:480;s:6:\"height\";i:368;s:14:\"hwstring_small\";s:23:\"height=\'96\' width=\'125\'\";s:4:\"file\";s:108:\"/home/kkfungne/public_html/abunchofgeniuses/blog/wp-content/uploads/2008/03/head-measurement-jan-26-2008.jpg\";s:5:\"sizes\";a:2:{s:9:\"thumbnail\";a:3:{s:4:\"file\";s:40:\"head-measurement-jan-26-2008-150x150.jpg\";s:5:\"width\";i:150;s:6:\"height\";i:150;}s:6:\"medium\";a:3:{s:4:\"file\";s:40:\"head-measurement-jan-26-2008-300x230.jpg\";s:5:\"width\";i:300;s:6:\"height\";i:230;}}s:10:\"image_meta\";a:10:{s:8:\"aperture\";i:0;s:6:\"credit\";s:10:\"Picasa 2.7\";s:6:\"camera\";s:0:\"\";s:7:\"caption\";s:0:\"\";s:17:\"created_timestamp\";i:0;s:9:\"copyright\";s:0:\"\";s:12:\"focal_length\";i:0;s:3:\"iso\";i:0;s:13:\"shutter_speed\";i:0;s:5:\"title\";s:0:\"\";}}'),(27,20,'_wp_attachment_metadata','s:279:\"a:5:{s:5:\"width\";i:480;s:6:\"height\";i:368;s:14:\"hwstring_small\";s:23:\"height=\'96\' width=\'125\'\";s:4:\"file\";s:101:\"/home/kkfungne/public_html/chingfung/blog/wp-content/uploads/2008/03/head-measurement-jan-26-2008.jpg\";s:5:\"thumb\";s:42:\"head-measurement-jan-26-2008.thumbnail.jpg\";}\";'),(28,21,'_wp_attached_file','/home/kkfungne/public_html/abunchofgeniuses/blog/wp-content/uploads/2008/03/heart-beats-feb-23-2008.jpg'),(29,21,'_wp_attachment_metadata','a:6:{s:5:\"width\";i:480;s:6:\"height\";i:366;s:14:\"hwstring_small\";s:23:\"height=\'96\' width=\'125\'\";s:4:\"file\";s:103:\"/home/kkfungne/public_html/abunchofgeniuses/blog/wp-content/uploads/2008/03/heart-beats-feb-23-2008.jpg\";s:5:\"sizes\";a:2:{s:9:\"thumbnail\";a:3:{s:4:\"file\";s:35:\"heart-beats-feb-23-2008-150x150.jpg\";s:5:\"width\";i:150;s:6:\"height\";i:150;}s:6:\"medium\";a:3:{s:4:\"file\";s:35:\"heart-beats-feb-23-2008-300x228.jpg\";s:5:\"width\";i:300;s:6:\"height\";i:228;}}s:10:\"image_meta\";a:10:{s:8:\"aperture\";i:0;s:6:\"credit\";s:0:\"\";s:6:\"camera\";s:0:\"\";s:7:\"caption\";s:1:\"r\";s:17:\"created_timestamp\";i:0;s:9:\"copyright\";s:0:\"\";s:12:\"focal_length\";i:0;s:3:\"iso\";i:0;s:13:\"shutter_speed\";i:0;s:5:\"title\";s:0:\"\";}}'),(30,21,'_wp_attachment_metadata','s:268:\"a:5:{s:5:\"width\";i:480;s:6:\"height\";i:366;s:14:\"hwstring_small\";s:23:\"height=\'96\' width=\'125\'\";s:4:\"file\";s:96:\"/home/kkfungne/public_html/chingfung/blog/wp-content/uploads/2008/03/heart-beats-feb-23-2008.jpg\";s:5:\"thumb\";s:37:\"heart-beats-feb-23-2008.thumbnail.jpg\";}\";'),(31,0,'_edit_lock','1214760872'),(32,0,'_edit_last','1'),(33,0,'_edit_last','1'),(34,0,'_edit_lock','1214668521');
/*!40000 ALTER TABLE `postmeta` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `posts`
--

DROP TABLE IF EXISTS `posts`;
CREATE TABLE `posts` (
  `ID` bigint(20) unsigned NOT NULL auto_increment,
  `post_author` bigint(20) NOT NULL default '0',
  `post_date` datetime NOT NULL default '0000-00-00 00:00:00',
  `post_date_gmt` datetime NOT NULL default '0000-00-00 00:00:00',
  `post_content` longtext NOT NULL,
  `post_title` text NOT NULL,
  `post_category` int(4) NOT NULL default '0',
  `post_excerpt` text NOT NULL,
  `post_status` varchar(20) NOT NULL default 'publish',
  `comment_status` varchar(20) NOT NULL default 'open',
  `ping_status` varchar(20) NOT NULL default 'open',
  `post_password` varchar(20) NOT NULL default '',
  `post_name` varchar(200) NOT NULL default '',
  `to_ping` text NOT NULL,
  `pinged` text NOT NULL,
  `post_modified` datetime NOT NULL default '0000-00-00 00:00:00',
  `post_modified_gmt` datetime NOT NULL default '0000-00-00 00:00:00',
  `post_content_filtered` text NOT NULL,
  `post_parent` bigint(20) NOT NULL default '0',
  `guid` varchar(255) NOT NULL default '',
  `menu_order` int(11) NOT NULL default '0',
  `post_type` varchar(20) NOT NULL default 'post',
  `post_mime_type` varchar(100) NOT NULL default '',
  `comment_count` bigint(20) NOT NULL default '0',
  PRIMARY KEY  (`ID`),
  KEY `post_name` (`post_name`),
  KEY `type_status_date` (`post_type`,`post_status`,`post_date`,`ID`)
) ENGINE=MyISAM AUTO_INCREMENT=22 DEFAULT CHARSET=utf8;

--
-- Dumping data for table `posts`
--

LOCK TABLES `posts` WRITE;
/*!40000 ALTER TABLE `posts` DISABLE KEYS */;
INSERT INTO `posts` VALUES (1,1,'2008-04-25 11:43:50','2008-04-25 17:43:50','Welcome to WordPress. This is your first post. Edit or delete it, then start blogging!','Hello world!',0,'','publish','open','open','','hello-world','','','2008-04-25 11:43:50','2008-04-25 17:43:50','',0,'http://www.a-bunch-of-geniuses.net/blog/?p=1',0,'post','',1),(2,1,'2008-04-25 11:43:50','2008-04-25 17:43:50','This is an example of a WordPress page, you could edit this to put information about yourself or your site so readers know where you are coming from. You can create as many pages like this one or sub-pages as you like and manage all of your content inside of WordPress.','About',0,'','publish','open','open','','about','','','2008-04-25 11:43:50','2008-04-25 17:43:50','',0,'http://www.a-bunch-of-geniuses.net/blog/?page_id=2',0,'page','',0),(3,1,'0000-00-00 00:00:00','0000-00-00 00:00:00','dsfds 馮以正','test 馮以正',0,'','draft','open','open','','','','','2008-06-30 09:35:30','2008-06-30 15:35:30','',0,'http://www.a-bunch-of-geniuses.net/blog/?p=3',0,'post','',0),(5,2,'2008-01-11 11:11:07','2008-01-11 03:11:07','hi mom,\n\nthis is my first post to the world...\n\nching fung','First post',0,'','publish','open','open','','first-post','','','2008-01-11 11:11:07','2008-01-11 03:11:07','',0,'http://chingfung.net/blog/?p=6',0,'post','',0),(6,2,'2008-01-11 17:06:20','2008-01-12 00:06:20','This is Ching Fung. Although I am not born yet, and don\'t really know how to type, I am eager to come to this world, experience, and hopefully make this a better place for everyone.\n\n:-)','About myself',0,'','publish','open','open','','about-2','','','2008-01-11 17:06:20','2008-01-12 00:06:20','',0,'http://chingfung.net/blog/?page_id=5',0,'page','',0),(7,2,'2008-01-15 22:11:31','2008-01-16 05:11:31','Mom went teaching at IVE in the morning, and had Korean for lunch. Yes, she went to Chong Wan, and had a lunch special. It was good, but the portion was a bit large. Mom went home, and we rested for a bit. This evening, we are going to have lunch with auntie Winnie.\n\nDad is still in Vancouver, working and visiting gung gung and po po. Po po said banana is probably not good for me, but I kind of like it. Should not be a problem. Dad went in the office to do some work. He thought about going to Cypress for some skiing, but decided against it. Need to put some time into the thesis thing. He went home for dinner with gung gung and po po. Had lamp, fish and preserved duck. Po po bought some fig for mom and I. Should be yummy...','At home with mom',0,'','publish','open','open','','at-home-with-mom','','','2008-01-15 22:11:31','2008-01-16 05:11:31','',0,'http://chingfung.net/blog/?p=7',0,'post','',0),(8,2,'2008-02-12 09:42:53','2008-02-12 16:42:53','Mom and dad had a fairly quite CNY. They visited a couple of relatives, and had several big meals. Dad is feeling some pain in his right knee, and does not know whether he can participate in this weekend\'s half-marathon.\n\nAuntie Joan is in town, waiting to give birth to her second baby. U-ho... Some baby girl friend.','After CNY',0,'','publish','open','open','','after-cny','','','2008-02-12 09:42:53','2008-02-12 16:42:53','',0,'http://chingfung.net/blog/?p=8',0,'post','',0),(9,2,'2008-03-16 23:55:36','2008-03-16 15:55:36','Dad has been lazy, and has not helped me keep this page updated. I am officially in wk24 now, but mom still has not felt any kicking from me. I guess I am getting too comfy in here.\n\nDad finished his half-marathon after all, although he walked half of the way. His time was 2:14:45, which I guess is ok for a first-timer. He went back to Vancouver, and brought back a bunch of old clothes from my cousins. Second batch so far (the first was from auntie Joanne).  BTW, she gave birth to her second girl yesterday (Mar 15) afternoon.\n\nMom and dad are getting ready for their trip to Kyoto Tuesday. This is going to be their last big trip before my arrival, so hope they\'ll enjoy and have a good time there. I am sure they will pick up some nice stuffs for me. Hehe...','Mid March already',0,'','publish','open','open','','mid-march-already','','','2008-03-16 23:55:36','2008-03-16 15:55:36','',0,'http://chingfung.net/blog/?p=9',0,'post','',0),(10,3,'2008-05-14 23:30:25','2008-05-14 15:30:25','It has been a while since my last post. I am now due in 7 weeks. Mom and Dad are now savoring every moments  before my arrival :-)\n\nThey ordered a high-chair for me over the weekend. It can be adjusted to different height, and can lay flat to become my temporary travelling palace.  Great idea... They also got me a Baby Bjorn. Hurrayyyy... Now I am sure I\'ll get a front-row seat wherever Mom and Dad go.\n\nI must have grown a lot in the past few weeks. Mom\'s tummy is now fully developed, but she is still beautiful.\n\nMom and Dad went to <em>the</em> largest art fair in HK tonight. Uncle Yunbo was there with the gallery representing him. Auntie Joan was also there. They went to dinner with RMIT classmates afterwards. Had Shanghai food. It\'s always good to have dinner with good friends.','7 weeks to go',0,'','publish','open','open','','7-weeks-to-go','','','2008-05-14 23:30:25','2008-05-14 15:30:25','',0,'http://chingfung.net/blog/?p=17',0,'post','',0),(11,3,'2008-06-12 00:30:25','2008-06-11 16:30:25','Mom and Dad went to see Dr. Uncle Bill last Sat. He estimated I now weighted over 5 lb. No wonder mom now feels my weight quite a bit. Sorry Mom. Three more weeks to go...\n\nHong Kong had a lot of rain on the day we went to see Dr. Bill. There was so much water on the street that Mom would not let Dad go to Sheung Wan to have his weekly congee at Sang Kee.\n\nMom tidied up all my clothings, and organized them inside a cabinet. All my stuffs should be ready now. Grandpa and Grandma are coming next week. Mom and Dad are going out for dinner this Sat. This is going to be their last weekend alone. Hehehe...','I am now 5 lb...',0,'','publish','open','open','','i-am-now-5-lb','','','2008-06-12 00:30:25','2008-06-11 16:30:25','',0,'http://chingfung.net/blog/?p=18',0,'post','',0),(12,3,'2008-06-14 00:38:15','2008-06-13 16:38:15','Wow, only 3 more weeks to go. It\'s raining again, but Mom and Dad went out for a walk after dinner anyway. This is going to be the last weekend before Grandpa and Grandma come for visit. They found me a diary, to record my eat time, nap time, and poo time. Do I ever going to see it?\n\nMignon Cat, my big brother, has the pimple problem coming back. Hope he will make a full recovery very soon. At least weekend is coming up, so Dad can take care of him.','35 weeks...',0,'','publish','open','open','','35-weeks','','','2008-06-14 00:38:15','2008-06-13 16:38:15','',0,'http://chingfung.net/blog/?p=19',0,'post','',0),(13,3,'2008-06-20 01:05:13','2008-06-19 17:05:13','Only 2 more weeks to go. Mom had been feeling quite a bit of back arch lately, especially at night. May be it\'s me trying to stretch out. Quite often, she needs to lay down after meals, so her stomach can relax a bit. Hang in there, Mom, I will be out soon...\n\nDad went jogging this evening. Although it rained a bit, the cloud broke, and the full moon shrine onto Happy Valley with full force. Quite a view. After all these rain lately, Hong Kong night sky has not been this clear for a long time.\n\nMom and Dad had dinner with Auntie Vivian, Auntie Yvonne, Uncle Cousin, and Uncle Wallace yesterday, just before Grand Ma and Grand Pa arrived from Vancouver. My name is pretty much fixed. Chinese version is Fung Yee Ching; English is Ching Fung, or Yeeching Fung. Quite like it.','It\'s official...2 more weeks',0,'','publish','open','open','','its-official2-more-weeks','','','2008-06-20 01:05:13','2008-06-19 17:05:13','',0,'http://chingfung.net/blog/?p=20',0,'post','',0),(14,3,'2008-06-28 23:55:12','2008-06-28 15:55:12','Mom and dad went to see Dr Bill this morning. This was the last regular check up before my birth. Everything went ok. My estimated weight now is 7lb 1oz. The head is a bit big with diameter at 9.55 cm. After that, we went to Sheung Wan with grandpa to have fish-bone congee.\n\nThis weekend has been a busy one for mom and dad. Birthday of Auntie Winnie and Uncle Jackie are coming in a couple of days. We went to meet Auntie Winnie at One Bar around 2:30pm. Mom was nice enough to organize a 100-Day-Old party for me. The plan is to have a tar party at One Bar, and introduce me to the gang. Sound like fun, although I won\'t get to eat the adult food yet. We then went to Four Season for high-tea. My first time there, and quite an enjoyable experience. Mom had lots of treats, and decaff coffee. Although it was a bit rush (we had to return the table by 4:05pm), we enjoyed it immensely. A bit of window shopping afterwards, of cause, at Lane Crawford. Dad went upstairs to the roof to take photos, and the rain storm came in. He got some interesting shots, but he got a bit wet. Time to go home and feed Mignon and Bo. On her way home, mom picked up some more supply for me. I am fully stock now...\n\nThen 6:30pm, we went to Tin Ha Tai Ping to have dinner with Jackie. This is mom\'s favourite, and probably mine too. It was nice and quite, and food was good. Nine O\'clock. Time to walk home.','Last weekend before parenthood',0,'','publish','open','open','','last-weekend-before-parenthood','','','2008-06-28 23:55:12','2008-06-28 15:55:12','',0,'http://chingfung.net/blog/?p=21',0,'post','',0),(16,3,'2008-03-17 00:01:49','2008-03-16 16:01:49','','',0,'','inherit','open','open','','11','','','2008-03-17 00:01:49','2008-03-16 16:01:49','',0,'http://www.a-bunch-of-geniuses.net/blog/wp-content/uploads/2008/03/head-measurement-1-feb-23-2008.jpg',0,'attachment','image/jpeg',0),(17,3,'2008-03-17 00:05:00','2008-03-16 16:05:00','','',0,'','inherit','open','open','','12','','','2008-03-17 00:05:00','2008-03-16 16:05:00','',0,'http://www.a-bunch-of-geniuses.net/blog/wp-content/uploads/2008/03/body-length-jan-26-2008.jpg',0,'attachment','image/jpeg',0),(18,3,'2008-03-17 00:05:56','2008-03-16 16:05:56','','',0,'','inherit','open','open','','13','','','2008-03-17 00:05:56','2008-03-16 16:05:56','',0,'http://www.a-bunch-of-geniuses.net/blog/wp-content/uploads/2008/03/head-measurement-1-feb-23-20081.jpg',0,'attachment','image/jpeg',0),(19,3,'2008-03-17 00:06:19','2008-03-16 16:06:19','','',0,'','inherit','open','open','','14','','','2008-03-17 00:06:19','2008-03-16 16:06:19','',0,'http://www.a-bunch-of-geniuses.net/blog/wp-content/uploads/2008/03/head-measurement-2-feb-23-2008.jpg',0,'attachment','image/jpeg',0),(20,3,'2008-03-17 00:06:34','2008-03-16 16:06:34','','',0,'','inherit','open','open','','15','','','2008-03-17 00:06:34','2008-03-16 16:06:34','',0,'http://www.a-bunch-of-geniuses.net/blog/wp-content/uploads/2008/03/head-measurement-jan-26-2008.jpg',0,'attachment','image/jpeg',0),(21,3,'2008-03-17 00:06:52','2008-03-16 16:06:52','','',0,'','inherit','open','open','','16','','','2008-03-17 00:06:52','2008-03-16 16:06:52','',0,'http://www.a-bunch-of-geniuses.net/blog/wp-content/uploads/2008/03/heart-beats-feb-23-2008.jpg',0,'attachment','image/jpeg',0);
/*!40000 ALTER TABLE `posts` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `term_relationships`
--

DROP TABLE IF EXISTS `term_relationships`;
CREATE TABLE `term_relationships` (
  `object_id` bigint(20) NOT NULL default '0',
  `term_taxonomy_id` bigint(20) NOT NULL default '0',
  `term_order` int(11) NOT NULL default '0',
  PRIMARY KEY  (`object_id`,`term_taxonomy_id`),
  KEY `term_taxonomy_id` (`term_taxonomy_id`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8;

--
-- Dumping data for table `term_relationships`
--

LOCK TABLES `term_relationships` WRITE;
/*!40000 ALTER TABLE `term_relationships` DISABLE KEYS */;
INSERT INTO `term_relationships` VALUES (1,2,0),(2,2,0),(3,2,0),(4,2,0),(5,2,0),(6,2,0),(7,2,0),(1,1,0),(3,1,0),(5,1,0),(6,1,0),(7,1,0),(8,1,0),(9,1,0),(10,1,0),(11,1,0),(12,1,0),(13,1,0),(14,1,0),(16,1,0),(17,1,0),(18,1,0),(19,1,0),(20,1,0),(21,1,0);
/*!40000 ALTER TABLE `term_relationships` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `term_taxonomy`
--

DROP TABLE IF EXISTS `term_taxonomy`;
CREATE TABLE `term_taxonomy` (
  `term_taxonomy_id` bigint(20) NOT NULL auto_increment,
  `term_id` bigint(20) NOT NULL default '0',
  `taxonomy` varchar(32) NOT NULL default '',
  `description` longtext NOT NULL,
  `parent` bigint(20) NOT NULL default '0',
  `count` bigint(20) NOT NULL default '0',
  PRIMARY KEY  (`term_taxonomy_id`),
  UNIQUE KEY `term_id_taxonomy` (`term_id`,`taxonomy`)
) ENGINE=MyISAM AUTO_INCREMENT=3 DEFAULT CHARSET=utf8;

--
-- Dumping data for table `term_taxonomy`
--

LOCK TABLES `term_taxonomy` WRITE;
/*!40000 ALTER TABLE `term_taxonomy` DISABLE KEYS */;
INSERT INTO `term_taxonomy` VALUES (1,1,'category','',0,10),(2,2,'link_category','',0,7);
/*!40000 ALTER TABLE `term_taxonomy` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `terms`
--

DROP TABLE IF EXISTS `terms`;
CREATE TABLE `terms` (
  `term_id` bigint(20) NOT NULL auto_increment,
  `name` varchar(55) NOT NULL default '',
  `slug` varchar(200) NOT NULL default '',
  `term_group` bigint(10) NOT NULL default '0',
  PRIMARY KEY  (`term_id`),
  UNIQUE KEY `slug` (`slug`)
) ENGINE=MyISAM AUTO_INCREMENT=3 DEFAULT CHARSET=utf8;

--
-- Dumping data for table `terms`
--

LOCK TABLES `terms` WRITE;
/*!40000 ALTER TABLE `terms` DISABLE KEYS */;
INSERT INTO `terms` VALUES (1,'Uncategorized','uncategorized',0),(2,'Blogroll','blogroll',0);
/*!40000 ALTER TABLE `terms` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `usermeta`
--

DROP TABLE IF EXISTS `usermeta`;
CREATE TABLE `usermeta` (
  `umeta_id` bigint(20) NOT NULL auto_increment,
  `user_id` bigint(20) NOT NULL default '0',
  `meta_key` varchar(255) default NULL,
  `meta_value` longtext,
  PRIMARY KEY  (`umeta_id`),
  KEY `user_id` (`user_id`),
  KEY `meta_key` (`meta_key`)
) ENGINE=MyISAM AUTO_INCREMENT=15 DEFAULT CHARSET=utf8;

--
-- Dumping data for table `usermeta`
--

LOCK TABLES `usermeta` WRITE;
/*!40000 ALTER TABLE `usermeta` DISABLE KEYS */;
INSERT INTO `usermeta` VALUES (1,1,'nickname','admin'),(2,1,'rich_editing','true'),(3,1,'admin_color','fresh'),(4,1,'capabilities','a:1:{s:13:\"administrator\";b:1;}'),(5,1,'user_level','10'),(6,1,'autosave_draft_ids','a:1:{i:-1214840040;i:3;}'),(7,2,'nickname','chingfung'),(8,2,'rich_editing','true'),(9,2,'admin_color','fresh'),(10,2,'capabilities','a:1:{s:10:\"subscriber\";b:1;}'),(11,3,'nickname','kam'),(12,3,'rich_editing','true'),(13,3,'admin_color','fresh'),(14,3,'capabilities','a:1:{s:10:\"subscriber\";b:1;}');
/*!40000 ALTER TABLE `usermeta` ENABLE KEYS */;
UNLOCK TABLES;

--
-- Table structure for table `users`
--

DROP TABLE IF EXISTS `users`;
CREATE TABLE `users` (
  `ID` bigint(20) unsigned NOT NULL auto_increment,
  `user_login` varchar(60) NOT NULL default '',
  `user_pass` varchar(64) NOT NULL default '',
  `user_nicename` varchar(50) NOT NULL default '',
  `user_email` varchar(100) NOT NULL default '',
  `user_url` varchar(100) NOT NULL default '',
  `user_registered` datetime NOT NULL default '0000-00-00 00:00:00',
  `user_activation_key` varchar(60) NOT NULL default '',
  `user_status` int(11) NOT NULL default '0',
  `display_name` varchar(250) NOT NULL default '',
  PRIMARY KEY  (`ID`),
  KEY `user_login_key` (`user_login`),
  KEY `user_nicename` (`user_nicename`)
) ENGINE=MyISAM AUTO_INCREMENT=4 DEFAULT CHARSET=utf8;

--
-- Dumping data for table `users`
--

LOCK TABLES `users` WRITE;
/*!40000 ALTER TABLE `users` DISABLE KEYS */;
INSERT INTO `users` VALUES (1,'admin','$P$BpL3NPqLCEGJtKMw/za8aeQDy1mgHa1','admin','you@yourdomain.com','http://','2008-04-25 17:43:50','',0,'admin'),(2,'chingfung','$P$BEAR0w6qkqJy1bdE1TVRuHEF8lYLm40','chingfung','','','2008-06-30 15:40:57','',0,'chingfung'),(3,'kam','$P$BrIiWI2G3gHjI/32lqNu1AEzumiAEJ.','kam','','','2008-06-30 15:40:57','',0,'kam');
/*!40000 ALTER TABLE `users` ENABLE KEYS */;
UNLOCK TABLES;
/*!40103 SET TIME_ZONE=@OLD_TIME_ZONE */;

/*!40101 SET SQL_MODE=@OLD_SQL_MODE */;
/*!40014 SET FOREIGN_KEY_CHECKS=@OLD_FOREIGN_KEY_CHECKS */;
/*!40014 SET UNIQUE_CHECKS=@OLD_UNIQUE_CHECKS */;
/*!40101 SET CHARACTER_SET_CLIENT=@OLD_CHARACTER_SET_CLIENT */;
/*!40101 SET CHARACTER_SET_RESULTS=@OLD_CHARACTER_SET_RESULTS */;
/*!40101 SET COLLATION_CONNECTION=@OLD_COLLATION_CONNECTION */;
/*!40111 SET SQL_NOTES=@OLD_SQL_NOTES */;

-- Dump completed on 2008-07-17 16:09:37
